@@ -1,6 +1,8 @@
 # Copyright 2014-2018 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
+from lino.core.utils import login, show, shows
+from lino.core.exceptions import ChangedAPI
 from django.conf import settings
 
 from lino.core.utils import models_by_base
@@ -9,13 +11,12 @@ models = settings.SITE.models
 """
 """
 
-from lino.core.exceptions import ChangedAPI
 # actors = modules = ChangedAPI(
 #     "Replace rt.modules and rt.actors by rt.models")
 
 # actors = settings.SITE.models
 
-login = settings.SITE.login
+# login = rt.login
 startup = settings.SITE.startup
 # get_printable_context = settings.SITE.get_printable_context
 lookup_filter = settings.SITE.lookup_filter
@@ -29,12 +30,7 @@ makedirs_if_missing = settings.SITE.makedirs_if_missing
 
 
 def get_template(*args, **kw):
-    return settings.SITE.plugins.jinja.renderer.jinja_env.get_template(
-        *args, **kw)
-
-
-def show(*args, **kw):
-    return login().show(*args, **kw)
+    return settings.SITE.plugins.jinja.renderer.jinja_env.get_template(*args, **kw)
 
 
 # def send_email(*args, **kw):
@@ -43,5 +39,4 @@ def show(*args, **kw):
 
 
 def html_text(*args, **kw):
-    return settings.SITE.kernel.default_renderer.html_text(
-        *args, **kw)
+    return settings.SITE.kernel.default_renderer.html_text(*args, **kw)
