@@ -1,11 +1,15 @@
 """
 Contains Lino's core functionalities.
 
+For some modules the documentation has already been migrated to prosa:
+
+- `plugin` is documented in :doc:`/src/lino/core/plugin`
+- `site` is documented in :doc:`/src/lino/core/site`
+- `actions` is documented in :doc:`/src/lino/core/actions`
+
 .. autosummary::
    :toctree:
 
-    actions
-    actors
     boundaction
     callbacks
     choicelists
@@ -26,20 +30,15 @@ Contains Lino's core functionalities.
     layouts
     menus
     merge
-    model
     permissions
-    plugin
     renderer
     requests
     roles
     signals
-    site
     store
     tables
-    tablerequest
     urls
     userprefs
-    user_types
     utils
     views
     workflows
