@@ -17,13 +17,15 @@ from django.conf import settings
 from django.utils.translation import gettext_lazy as _
 from django.utils.encoding import force_str
 
-from etgen.html import tostring
+from lino.utils.html import tostring
 
 # from lino.core.requests import ActorRequest
 
-CLONEABLE_ATTRS = frozenset("""ah request user subst_user
+CLONEABLE_ATTRS = frozenset(
+    """ah request user subst_user
 bound_action create_kw known_values param_values
-action_param_values""".split())
+action_param_values""".split()
+)
 
 
 class CallbackChoice(object):
@@ -41,7 +43,8 @@ class Callback(object):
     that answers the question.
 
     """
-    title = _('Confirmation')
+
+    title = _("Confirmation")
 
     def __init__(self, ar, message, uid=None):
         self.message = message
@@ -49,8 +52,12 @@ class Callback(object):
         self.choices_dict = {}
         self.ar = ar
 
-        str_msg = tostring(message).encode()
-        self.uid = uid if uid is not None else str(md5(str_msg).digest())
+        str_msg = f"{ar.get_user().username}:{tostring(message)}"
+
+        # self.uid = uid if uid is not None else str(md5(str_msg).digest())
+        self.uid = uid if uid is not None else md5(str_msg.encode()).hexdigest()
+        # 20240407 .digest() returned bytestrings, yielding strings of style
+        # "b'8\x04\xeb\xf53\x9f\xf2\x90\x82l\x81~s\xe8\x90\xa6'"
 
     def __repr__(self):
         return "Callback(%r)" % self.message
@@ -65,9 +72,9 @@ class Callback(object):
         - func: a callable to be executed when user selects this choice
         - the label of the button
         """
-        assert not name in self.choices_dict
+        assert name not in self.choices_dict
         allowed_names = ("yes", "no", "ok", "cancel")
-        if not name in allowed_names:
+        if name not in allowed_names:
             raise Exception("Sorry, name must be one of %s" % allowed_names)
         cbc = CallbackChoice(name, func, label)
         self.choices.append(cbc)
@@ -76,14 +83,15 @@ class Callback(object):
 
     def run(self, choice):
         if choice not in self.choices_dict:
-            raise Exception("Can not run choice|%s| not in available choices %s" % (
-                choice, self.choices_dict))
+            raise Exception(
+                "Can not run choice|%s| not in available choices %s"
+                % (choice, self.choices_dict)
+            )
         self.choices_dict[choice].func(self.ar)
 
+
 class unused_CallbackManager(object):
-
     def __init__(self):
-
         self.pending_threads = {}
 
     def run_callback(self, request, thread_id, button_id):
@@ -102,8 +110,9 @@ class unused_CallbackManager(object):
         thread_id = thread_id
         cb = self.pending_threads.pop(thread_id, None)
         if cb is None:
-            logger.debug("No callback %r in %r" % (
-                thread_id, list(self.pending_threads.keys())))
+            logger.debug(
+                "No callback %r in %r" % (thread_id, list(self.pending_threads.keys()))
+            )
             raise Warning("Unknown callback %r" % thread_id)
             # ar = ActorRequest(request, renderer=self.default_renderer)
             # ar.error("Unknown callback %r" % thread_id)
@@ -123,8 +132,9 @@ class unused_CallbackManager(object):
             if c.name == button_id:
                 a = ar.bound_action.action
                 if settings.SITE.log_each_action_request and not a.readonly:
-                    logger.info("run_callback {0} {1} {2}".format(
-                        thread_id, cb.message, c.name))
+                    logger.info(
+                        "run_callback {0} {1} {2}".format(thread_id, cb.message, c.name)
+                    )
                     # ar.debug(cb.message)
                 func = c.func
                 try:
@@ -136,6 +146,7 @@ class unused_CallbackManager(object):
         ar.error("Invalid button %r for callback %r" % (button_id, thread_id))
         return ar.renderer.render_action_response(ar)
 
+
 def add_callback(ar, *msgs, uid=None):
     """
     Returns an *action callback* that will initiate a dialog thread by
@@ -151,12 +162,13 @@ def add_callback(ar, *msgs, uid=None):
 
     """
     if len(msgs) > 1:
-        msg = '\n'.join([force_str(s) for s in msgs])
+        msg = "\n".join([force_str(s) for s in msgs])
     else:
         msg = msgs[0]
     # logger.info("20160526 add_callback(%s)", msg)
     return Callback(ar, msg, uid=uid)
 
+
 def set_callback(ar, cb):
     """
     Runs when confimation is requred, checks if answer is present in request data, if so, runs corisponding function
@@ -166,33 +178,38 @@ def set_callback(ar, cb):
     answer = ar.xcallback_answers.get("xcallback__" + cb_id, None)
     if answer is None:
         buttons = []
-        xcallback = dict(
-            id=cb_id,
-            title=cb.title,
-            buttons=buttons)
+        xcallback = dict(id=cb_id, title=cb.title, buttons=buttons)
 
-        rqdata = { k:v[0] if len(v) == 1 else v
-            for k,v in (ar.rqdata.lists() if getattr(ar, "rqdata", None) is not None else {})}
+        rqdata = {
+            k: v[0] if len(v) == 1 else v
+            for k, v in (
+                ar.rqdata.lists() if getattr(ar, "rqdata", None) is not None else {}
+            )
+        }
         rqdata.pop("_dc", None)
 
         for c in cb.choices:
             buttons.append([c.name, c.label])
             xcallback[c.name + "_resendEvalJs"] = ar.renderer.ar2js(
-                ar, ar.selected_rows, rqdata= rqdata, xcallback={
-                    "xcallback_id" : cb_id,
-                    "choice" : c.name })
+                ar,
+                ar.selected_rows,
+                rqdata=rqdata,
+                xcallback={"xcallback_id": cb_id, "choice": c.name},
+            )
         # print(buttons)
         return ar.success(cb.message, xcallback=xcallback)
 
     else:
         cb.run(answer)
 
+
 def popCallBack(resp):
     """
     :param resp: a dict which has some callback choices in it
     :return: dict of all callback choices
     """
-    return {cb:c for cb,c in resp.items() if cb.startswith("xcallback__")}
+    return {cb: c for cb, c in resp.items() if cb.startswith("xcallback__")}
+
 
 def applyCallbackChoice(resp={}, data={}, choice="yes"):
     """
@@ -200,10 +217,11 @@ def applyCallbackChoice(resp={}, data={}, choice="yes"):
     Used in testing
     """
     data.update(popCallBack(resp))
-    callback = {"xcallback__" + resp["xcallback"]['id'] : choice}
+    callback = {"xcallback__" + resp["xcallback"]["id"]: choice}
     data.update(callback)
     return data
 
+
 # mpman = Manager()
 # mgr = CallbackManager()
 #
