@@ -1,25 +1,19 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2008-2023 Rumma & Ko Ltd
+# Copyright 2008-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Defines the classes :class:`Choice` and :class:`ChoiceList`.  See
 :doc:`/dev/choicelists`.
 """
 
-from future.utils import with_metaclass
-
 import warnings
-
 from django.utils.translation import gettext_lazy as _
 from django.utils.functional import lazy
 # from django.utils.deconstruct import deconstructible
 from django.db import models
 from django.conf import settings
-from django.db.models import NOT_PROVIDED
 from django.db.migrations.serializer import BaseSerializer
 from django.db.migrations.writer import MigrationWriter
-
 from lino.utils import MissingRow
 from lino.core.utils import resolve_field
 from lino.core import actions
@@ -29,16 +23,18 @@ from lino.core import fields
 from .exceptions import UnresolvedChoice
 from .signals import receiver, pre_analyze
 
-
 STRICT = True
 VALUE_FIELD = models.CharField(_("value"), max_length=20)
-VALUE_FIELD.attname = 'value'
+VALUE_FIELD.attname = "value"
 
 # @deconstructible
+
+
 class Choice(fields.TableRow):
-    """A constant value whose unicode representation depends on the
-    current language at runtime.  Every item of a :class:`ChoiceList`
-    must be an instance of :class:`Choice` or a subclass thereof.
+    """
+    A constant value whose string representation depends on the current language
+    at runtime.  Every item of a :class:`ChoiceList` must be an instance of
+    :class:`Choice` or a subclass thereof.
 
     .. attribute:: choicelist
 
@@ -67,12 +63,13 @@ class Choice(fields.TableRow):
         The text to appear on buttons representing this state.
 
     """
+
     choicelist = None
     remark = None
     pk = None
     value = None
     text = None
-    names = ''
+    names = ""
     button_text = None
 
     def __init__(self, value=None, text=None, names=None, **kwargs):
@@ -114,16 +111,15 @@ class Choice(fields.TableRow):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-
     @property
     def name(self):
-       if len(self.names) == 0:
-           return None
-       # elif len(self.names) == 1:
-       else:
-           return self.names[0]
-       # raise Exception("{} has multiple names".format(self))
-       # return ' '.join(self.names[0])
+        if len(self.names) == 0:
+            return None
+        # elif len(self.names) == 1:
+        else:
+            return self.names[0]
+        # raise Exception("{} has multiple names".format(self))
+        # return ' '.join(self.names[0])
 
     # def deconstruct(self):
     #     return ('str', self.value, {})
@@ -142,25 +138,27 @@ class Choice(fields.TableRow):
     def attach(i, cls):
         i.choicelist = cls
         if i.value is None:
-            i.pk = i.value = str(len(cls.choices)+1)
+            i.pk = i.value = str(len(cls.choices) + 1)
         elif len(i.value) > cls.max_length:
             if len(cls._fields) > 0:
                 raise Exception(
                     "%s cannot add value %r because fields exist "
                     "and max_length is %d."
-                    % (cls, i.value, cls.max_length) + """\
+                    % (cls, i.value, cls.max_length)
+                    + """\
 When fields have been created, we cannot simply change their max_length because
 Django creates copies of them when inheriting models.
-""")
+"""
+                )
             cls.max_length = len(i.value)
-            #~ for fld in cls._fields:
-                #~ fld.set_max_length(cls.max_length)
+            # ~ for fld in cls._fields:
+            # ~ fld.set_max_length(cls.max_length)
 
         dt = cls.display_text(i)
         cls.choices.append((i, dt))
         # cls.preferred_width = max(cls.preferred_width, len(unicode(dt)))
         cls.items_dict[i.value] = i
-        #~ cls.items_dict[i] = i
+        # ~ cls.items_dict[i] = i
 
     def remove(self):
         """Remove this choice from its list.
@@ -183,51 +181,50 @@ Django creates copies of them when inheriting models.
         # if self.value is None:
         #     return super(Choice, self).__eq__(other)
         if other.__class__ is self.__class__:
-            return (self.value == other.value)
+            return self.value == other.value
         return self.value == other
 
     def __ne__(self, other):
         # if self.value is None:
         #     return super(Choice, self).__neq__(other)
         if other.__class__ is self.__class__:
-            return (self.value != other.value)
-        return (self.value != other)
+            return self.value != other.value
+        return self.value != other
 
     def __lt__(self, other):
         # if self.value is None:
         #     return super(Choice, self).__lt__(other)
         if other.__class__ is self.__class__:
-            return (self.value < other.value)
-        return (self.value < other)
+            return self.value < other.value
+        return self.value < other
 
     def __le__(self, other):
         # if self.value is None:
         #     return super(Choice, self).__le__(other)
         if other.__class__ is self.__class__:
-            return (self.value <= other.value)
-        return (self.value <= other)
+            return self.value <= other.value
+        return self.value <= other
 
     def __gt__(self, other):
         # if self.value is None:
         #     return super(Choice, self).__gt__(other)
         if other.__class__ is self.__class__:
-            return (self.value > other.value)
-        return (self.value > other)
+            return self.value > other.value
+        return self.value > other
 
     def __ge__(self, other):
         # if self.value is None:
         #     return super(Choice, self).__ge__(other)
         if other.__class__ is self.__class__:
-            return (self.value >= other.value)
-        return (self.value >= other)
+            return self.value >= other.value
+        return self.value >= other
 
     __hash__ = object.__hash__
 
     def __repr__(self):
         # when there several names, we use the first one
         if len(self.names) > 0:
-            return "<%s.%s:%s>" % (
-                self.choicelist, self.names[0], self.value)
+            return "<%s.%s:%s>" % (self.choicelist, self.names[0], self.value)
         else:
             return "<%s:%s>" % (self.choicelist, self.value)
 
@@ -245,39 +242,31 @@ Django creates copies of them when inheriting models.
     def get_chooser_for_field(cls, fieldname):
         return None
 
-    def get_typed_instance(self, model):
-        """
-        Used when implementing :ref:`polymorphism`.
-        """
-        return self
-
-    def obj2href(self, ar):
+    def as_summary_item(self, ar, text=None, **kwargs):
         # needed by `detail_pointer`
-        return str(self)
+        return text or str(self)
 
 
 class UnresolvedValue(Choice):
     def __init__(self, choicelist, value):
         self.choicelist = choicelist
         self.value = value
-        self.text = "Unresolved value %r for %s" % (
-            value, choicelist.__name__)
+        self.text = "Unresolved value %r for %s" % (value, choicelist.__name__)
         self.names = {}
 
 
 class ChoiceListMeta(actors.ActorMetaClass):
-
     def __new__(meta, classname, bases, classDict):
-        #~ if not classDict.has_key('app_label'):
-            #~ classDict['app_label'] = cls.__module__.split('.')[-2]
+        # ~ if not classDict.has_key('app_label'):
+        # ~ classDict['app_label'] = cls.__module__.split('.')[-2]
         """
         UserGroups manually sets `max_length` because the
         default list has only one group with value "system",
         but applications may want to add longer group names
         """
-        if 'label' in classDict:
+        if "label" in classDict:
             raise Exception("label replaced by verbose_name_plural")
-        classDict.setdefault('max_length', 10)
+        classDict.setdefault("max_length", 10)
         cls = actors.ActorMetaClass.__new__(meta, classname, bases, classDict)
 
         cls.items_dict = {}
@@ -287,13 +276,13 @@ class ChoiceListMeta(actors.ActorMetaClass):
         return cls
 
 
-#~ choicelist_column_fields = dict()
-#~ choicelist_column_fields['value'] = fields.VirtualField(models.CharField())
-#~ choicelist_column_fields['text'] = fields.VirtualField(models.CharField())
-#~ choicelist_column_fields['name'] = fields.VirtualField(models.CharField())
+# ~ choicelist_column_fields = dict()
+# ~ choicelist_column_fields['value'] = fields.VirtualField(models.CharField())
+# ~ choicelist_column_fields['text'] = fields.VirtualField(models.CharField())
+# ~ choicelist_column_fields['name'] = fields.VirtualField(models.CharField())
+
 
 class CallableChoice:
-
     def __init__(self, choicelist, name):
         self.choicelist = choicelist
         self.name = name
@@ -310,9 +299,10 @@ class PointingChoice(Choice):
     """
     A choice that points to a database object.
 
-    Usage examples: :class:`lino_xl.lib.ledger.CommonAccounts`,
+    Usage examples: :class:`lino_xl.lib.accounting.CommonAccounts`,
     :class:`lino_xl.lib.properties.PropertyAreas`.
     """
+
     pointing_field_name = None
 
     _pointing_field = None
@@ -327,25 +317,30 @@ class PointingChoice(Choice):
     def set_object(self, obj):
         self._instance = obj
 
-    def get_object(self):
-        # return rt.models.ledger.Account.objects.get(ref=self.value)
+    def get_object(self, severe=False, **filter):
+        # return rt.models.accounting.Account.objects.get(ref=self.value)
         if self._instance is None:
             pf = self.pointing_field
-            try:
-                self._instance = pf.model.objects.get(**{pf.name: self})
-            except pf.model.DoesNotExist:
-                # self._instance = MissingRow(pf,self)
-                self._instance = MissingRow(
-                    _("No {} pointing to {}").format(
-                        pf.model._meta.verbose_name, repr(self)))
+            filter.update({pf.name: self})
+            if severe:
+                self._instance = pf.model.objects.get(**filter)
+            else:
+                try:
+                    self._instance = pf.model.objects.get(**filter)
+                except pf.model.DoesNotExist:
+                    # self._instance = MissingRow(pf,self)
+                    self._instance = MissingRow(
+                        _("No {} pointing to {}").format(
+                            pf.model._meta.verbose_name, repr(self)
+                        )
+                    )
         return self._instance
 
     def has_row(self):
-        return not isinstance(self.get_object(), MissingRow)
+        return not isinstance(self.get_object(severe=False), MissingRow)
 
 
-class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
-
+class ChoiceList(tables.AbstractTable, metaclass=ChoiceListMeta):
     """
     User-defined choice lists must inherit from this base class.
 
@@ -354,6 +349,7 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         The default `max_length` for fields using this choicelist.
 
     """
+
     abstract = True
     workflow_actions = []
 
@@ -368,28 +364,43 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
     """A string with the name of a choice to be used as default value for
     fields using this list.
 
+    Usage example in :mod:`lino_xl.lib.clients.models`::
+
+        class ClientStates(dd.Workflow):
+            required_roles = dd.login_required(ContactsStaff)
+            verbose_name_plural = _("Client states")
+            default_value = 'newcomer'
+
+        add = ClientStates.add_item
+        add('10', _("Newcomer"), 'newcomer')
+        add('20', _("Refused"), 'refused')
+        add('30', _("Coached"), 'coached')
+        add('50', _("Former"), 'former')
+
+    And :mod:`lino_avant.lib.avanti.models` overrides it::
+
+        from lino_xl.lib.clients.choicelists import ClientStates
+        ClientStates.default_value = 'coached'
+
     Note that this specifies the *default* default value for *all*
     :class:`ChoiceListField <lino.core.choicelists.ChoiceListField>`
     of this choicelist, including parameter fields.
 
+    The disadvantage is that when somebody *does not* want your default value,
+    then they must explicitly specify `default=''` when defining a field on your
+    choicelist.
+
     You can remove that "default default value" for all tables by
     specifying `default=''`. There are two places where you can
     specify this: (a) on the parameter field itself (which then
     applies to all subtables) or (b) just on one table. For example
     (excerpt from :mod:`lino_avanti.lib.avanti`)::
 
-        from lino_xl.lib.clients.choicelists import ClientStates
-        ClientStates.default_value = 'coached'
-
         parameters = ObservedDateRange(
             ...
             client_state=ClientStates.field(blank=True, default=''))
 
-    Note that the default values of parameter fields of a table that
-    is used as the *models default table* will apply for the choices
-    of pointers to that model. Concrete use case is the choicelist of
-    `cal.Guest.partner` in :ref:`avanti`, which should show only
-    coached clients.  So instead of above code we actually now do::
+    Example for (b)::
 
         class MyClients(My, Clients):
             @classmethod
@@ -398,9 +409,11 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
                 kw.update(client_state='')
                 return kw
 
-    The disadvantage is that when somebody *does not* want your default value,
-    then they must explicitly specify `default=''` when defining a field on your
-    choicelist.
+    Note that the default values of parameter fields of a table that
+    is used as the *model's default table* will apply for the choices
+    of pointers to that model. Concrete use case is the choicelist of
+    `cal.Guest.partner` in :ref:`avanti`, which we want to show only
+    coached clients.
 
     """
 
@@ -426,22 +439,22 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
     """
 
     preferred_width = None
-    """Preferred width (in characters) used by
-    choicelist fields that refer to
-    this list.
+    """
+    Preferred width (in characters) used by choicelist fields that refer to this
+    list.
 
-    If this is `None`, then Lino calculates the value at startup,
-    taking the length of the longest choice text.  The hard-coded
+    If this is `None`, then Lino calculates the default value at :term:`site
+    startup`, taking the length of the longest choice text.  The hard-coded
     absolute minimum in that case is 4.
 
-    Note that it calculates the value using the :attr:`default site language
+    Lino calculates the default value using the :attr:`default site language
     <lino.site.Site.languages>` and thus might guess wrong if the user language
     is not the default site language.
 
-    Note that by this we mean the width of the bare text field,
-    excluding any UI-specific control like the trigger button of a
-    combobox.  That's why e.g. :mod:`lino.modlib.extjs.ext_elems` adds
-    another value for the trigger button.
+    Note that by this we mean the width of the bare text field, excluding any
+    UI-specific control like the trigger button of a combobox.  That's why e.g.
+    :mod:`lino.modlib.extjs.ext_elems` adds another value for the trigger
+    button.
 
     """
 
@@ -451,21 +464,21 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
 
     @classmethod
     def get_default_action(cls):
-        return actions.ShowTable()
+        return actions.SHOW_TABLE
 
-    hidden_columns = frozenset(['workflow_buttons'])
+    hidden_columns = frozenset(["workflow_buttons"])
 
-    column_names = 'value name text *'
+    column_names = "value name text *"
 
     button_text = models.CharField(_("Button text"), blank=True)
 
     old2new = {}
     """
-    A dict which maps old values to their new values.
+    A dict that maps old values to their new values.
 
-    This dict is consulted when an unknown value is read from database
-    (e.g. during a migration).  If if contains a replacement for the
-    old value, Lino will return the choice with the new value.
+    This is consulted when an unknown value is read from database (e.g. during a
+    migration).  If it contains a replacement for the old value, Lino will
+    return the choice with the new value.
     """
 
     @classmethod
@@ -493,7 +506,7 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
 
     @fields.virtualfield(models.CharField(_("name"), max_length=20))
     def name(cls, choice, ar):
-        return ' '.join(choice.names)
+        return " ".join(choice.names)
 
     @fields.displayfield(_("Remark"))
     def remark(cls, choice, ar):
@@ -509,9 +522,7 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
 
     @classmethod
     def get_data_rows(self, ar=None):
-        """
-
-        """
+        """ """
         # return sorted(self.items())
         return self.get_list_items()
 
@@ -569,7 +580,7 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         Sort the items by their value.
 
         Used for example by :mod:`lino_xl.lib.orders` where we add a journal
-        group to :class:`lino_xl.lib.ledger.JournalGroups` and want it to come
+        group to :class:`lino_xl.lib.accounting.JournalGroups` and want it to come
         before the other groups.
 
         """
@@ -585,6 +596,7 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         Returns a `DummyField` if the plugin which defines this
         choicelist is not installed.
         """
+        # if not settings.SITE.is_hidden_plugin(cls.app_label):
         if settings.SITE.is_installed(cls.app_label):
             fld = ChoiceListField(cls, *args, **kw)
             # cls.setup_field(fld)
@@ -608,11 +620,12 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         """Run :meth:`add_item` with these arguments when all plugins have
         been loaded.
 
-        This is used e.g. when declaring :mod:`lino_xl.lib.ledger.VoucherTypes`
+        This is used e.g. when declaring :mod:`lino_xl.lib.accounting.VoucherTypes`
         : a voucher type is defined using its ByJournal table, but the model of
         that table is not necessarily resolvable at that moment.
 
         """
+
         @receiver(pre_analyze, weak=False)
         def func(sender, **ignored):
             cls.add_item(*args, **kwargs)
@@ -647,8 +660,8 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
 
     @classmethod
     def add_item_instance(cls, i):
-        #~ if cls is ChoiceList:
-            #~ raise Exception("Cannot define items on the base class")
+        # ~ if cls is ChoiceList:
+        # ~ raise Exception("Cannot define items on the base class")
         is_duplicate = False
         if i.value is not None and i.value in cls.items_dict:
             raise Exception("Duplicate value %r in %s." % (i.value, cls))
@@ -658,19 +671,19 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
             if not is_duplicate:
                 if name in cls.__dict__:
                     raise Exception(
-                        "An attribute named %r is already defined in %s" % (
-                            name, cls.__name__))
+                        "An attribute named %r is already defined in %s"
+                        % (name, cls.__name__)
+                    )
             # if cls.__name__ == "UserTypes":
             #     print("20200504 add {} {} {}".format(cls, i.name, i))
             setattr(cls, name, i)
-            #~ i.name = name
+            # ~ i.name = name
         i.attach(cls)
         return i
 
     @classmethod
     def get_pk_field(self):
-        """See :meth:`lino.core.actors.Actor.get_pk_field`.
-        """
+        """See :meth:`lino.core.actors.Actor.get_pk_field`."""
         return VALUE_FIELD
 
     @classmethod
@@ -693,30 +706,30 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
             raise UnresolvedChoice(
                 "Unresolved value %r (%s) for %s (set "
                 "Site.strict_choicelist_values to False "
-                "to ignore this)" % (
-                    value, value.__class__, cls))
+                "to ignore this)" % (value, value.__class__, cls)
+            )
         return UnresolvedValue(cls, value)
 
     @classmethod
     def get_choices(cls):
         return cls.choices
 
-    #~ @classmethod
-    #~ def get_choices(cls):
-        #~ """
-        #~ We must make it dynamic since e.g. UserTypes can change after
-        #~ the fields have been created.
+    # ~ @classmethod
+    # ~ def get_choices(cls):
+    # ~ """
+    # ~ We must make it dynamic since e.g. UserTypes can change after
+    # ~ the fields have been created.
 
-        #~ https://docs.djangoproject.com/en/5.0/ref/models/fields/
-        #~ note that choices can be any iterable object -- not necessarily
-        #~ a list or tuple. This lets you construct choices dynamically.
-        #~ But if you find yourself hacking choices to be dynamic, you're
-        #~ probably better off using a proper database table with a
-        #~ ForeignKey. choices is meant for static data that doesn't
-        #~ change much, if ever.
-        #~ """
-        #~ for c in cls.choices:
-            #~ yield c
+    # ~ https://docs.djangoproject.com/en/5.2/ref/models/fields/
+    # ~ note that choices can be any iterable object -- not necessarily
+    # ~ a list or tuple. This lets you construct choices dynamically.
+    # ~ But if you find yourself hacking choices to be dynamic, you're
+    # ~ probably better off using a proper database table with a
+    # ~ ForeignKey. choices is meant for static data that doesn't
+    # ~ change much, if ever.
+    # ~ """
+    # ~ for c in cls.choices:
+    # ~ yield c
 
     @classmethod
     def display_text(cls, bc):
@@ -725,12 +738,14 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
 
         Override this to customize the display text of choices.
 
+        Usage example: :class:`lino.modlib.system.DisplayColors`.
+
         :class:`lino.modlib.users.UserGroups` and
         :class:`lino.modlib.cv.models.CefLevel` used to do this before
         we had the :attr:`ChoiceList.show_values` option.
 
-        This must be lazily translatable because the return value is also used
-        to build the `choices` attribute of ChoiceListFields on this choicelist.
+        The return value must be lazily translatable because it is also used to
+        build the `choices` attribute of ChoiceListFields on this choicelist.
 
         Note that Django's `lazy` function has a list of "resultclasses" that
         are used "so that the automatic forcing of the lazy evaluation code is
@@ -763,9 +778,9 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         """
         if not isinstance(value, str):
             raise Exception("%r is not a string" % value)
-        #~ print "get_text_for_value"
-        #~ return self.items_dict.get(value, None)
-        #~ return self.items_dict.get(value)
+        # ~ print "get_text_for_value"
+        # ~ return self.items_dict.get(value, None)
+        # ~ return self.items_dict.get(value)
         return cls.items_dict.get(value, *args)
 
     @classmethod
@@ -787,9 +802,9 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         """
         return self.get_by_name(self.default_value)
 
-    #~ @classmethod
-    #~ def items(self):
-        #~ return [choice[0] for choice in self.choices]
+    # ~ @classmethod
+    # ~ def items(self):
+    # ~ return [choice[0] for choice in self.choices]
 
     @classmethod
     def find(cls, **fkw):
@@ -806,10 +821,11 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
     @classmethod
     def filter(self, **fkw):
         def f(item):
-            for k, v in list(fkw.items()):
+            for k, v in fkw.items():
                 if getattr(item, k) != v:
                     return False
             return True
+
         return [choice[0] for choice in self.choices if f(choice[0])]
 
     # @classmethod
@@ -830,8 +846,9 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
     @classmethod
     def get_list_items(self):
         return [choice[0] for choice in self.choices]
+
     objects = get_list_items  # deprecated alias for backwards compat
-    items = get_list_items   # deprecated alias for backwards compat
+    items = get_list_items  # deprecated alias for backwards compat
 
     @classmethod
     def get_text_for_value(self, value):
@@ -841,12 +858,12 @@ class ChoiceList(with_metaclass(ChoiceListMeta, tables.AbstractTable)):
         bc = self.get_by_value(value)
         if bc is None:
             return _("%(value)r (invalid choice for %(list)s)") % dict(
-                list=self.__name__, value=value)
+                list=self.__name__, value=value
+            )
         return self.display_text(bc)
 
 
 class ChoiceListField(models.CharField):
-
     """A field that stores a value to be selected from a
     :class:`ChoiceList`.
 
@@ -872,8 +889,15 @@ class ChoiceListField(models.CharField):
     strict: bool
     """Whether to fail at runtime if no :class:`Choice` exists for a value (used in from_db_value)."""
 
-    def __init__(self, choicelist=None, verbose_name=None, strict=True,
-                 force_selection=True, default=None, **kw):
+    def __init__(
+        self,
+        choicelist=None,
+        verbose_name=None,
+        strict=True,
+        force_selection=True,
+        default=None,
+        **kw,
+    ):
         if verbose_name is None:
             verbose_name = choicelist.verbose_name
         self.choicelist = choicelist
@@ -883,8 +907,7 @@ class ChoiceListField(models.CharField):
             if isinstance(default, str):
                 default = choicelist.as_callable(default)
             kw.update(default=default)
-        defaults = dict(
-            max_length=choicelist.max_length)
+        defaults = dict(max_length=choicelist.max_length)
         if choicelist.default_value:
             defaults.update(default=choicelist.get_default_value)
         # if not 'help_text' in kw:
@@ -907,21 +930,21 @@ class ChoiceListField(models.CharField):
     def deconstruct(self):
         """
         Needed for Django 1.7+, see
-        https://docs.djangoproject.com/en/5.0/howto/custom-model-fields/#custom-field-deconstruct-method
+        https://docs.djangoproject.com/en/5.2/howto/custom-model-fields/#custom-field-deconstruct-method
         """
 
         name, path, args, kwargs = super().deconstruct()
-        kwargs['choicelist'] = self.choicelist
+        kwargs["choicelist"] = self.choicelist
         # The available choices are *not* relevant for the database structure,
         # and Django might have problems for representing them in a migration.
-        kwargs.pop('choices', None)
+        kwargs.pop("choices", None)
         return name, path, args, kwargs
 
     def get_internal_type(self):
         return "CharField"
 
-    #~ def set_max_length(self,ml):
-        #~ self.max_length = ml
+    # ~ def set_max_length(self,ml):
+    # ~ self.max_length = ml
 
     def from_db_value(self, value, expression, connection, context=None):
         if value is None:
@@ -935,11 +958,11 @@ class ChoiceListField(models.CharField):
 
     def to_python(self, value):
         """See Django's docs about `to_python()
-        <https://docs.djangoproject.com/en/5.0/ref/models/fields/#django.db.models.Field.to_python>`__.
+        <https://docs.djangoproject.com/en/5.2/ref/models/fields/#django.db.models.Field.to_python>`__.
 
         """
-        #~ if self.attname == 'query_register':
-            #~ print '20120527 to_python', repr(value), '\n'
+        # ~ if self.attname == 'query_register':
+        # ~ print '20120527 to_python', repr(value), '\n'
         if isinstance(value, Choice):
             return value
         return self.choicelist.to_python(value)
@@ -962,16 +985,16 @@ class ChoiceListField(models.CharField):
     def get_prep_value(self, value):
         """
         Excerpt from `Django docs
-        <https://docs.djangoproject.com/en/5.0/howto/custom-model-fields/#converting-python-objects-to-query-values>`__:
+        <https://docs.djangoproject.com/en/5.2/howto/custom-model-fields/#converting-python-objects-to-query-values>`__:
         "If you override `to_python()
-        <https://docs.djangoproject.com/en/5.0/ref/models/fields/#django.db.models.Field.to_python>`__
+        <https://docs.djangoproject.com/en/5.2/ref/models/fields/#django.db.models.Field.to_python>`__
         you also have to override `get_prep_value()
-        <https://docs.djangoproject.com/en/5.0/ref/models/fields/#django.db.models.Field.get_prep_value>`__
+        <https://docs.djangoproject.com/en/5.2/ref/models/fields/#django.db.models.Field.get_prep_value>`__
         to convert Python objects back to query values."
         """
-        #~ if self.attname == 'query_register':
-            #~ print '20120527 get_prep_value()', repr(value)
-        #~ return value.value
+        # ~ if self.attname == 'query_register':
+        # ~ print '20120527 get_prep_value()', repr(value)
+        # ~ return value.value
         # Hamza, why did you add the following 2 lines?
         # if isinstance(value,unicode):
         #     return str(value)
@@ -980,18 +1003,18 @@ class ChoiceListField(models.CharField):
                 value = value()
             value = self.to_python(value)  # see Luc's blog 20160204
             return value.value
-        return ''
-        #~ return None
+        return ""
+        # ~ return None
 
     def value_to_string(self, obj):
         value = self.value_from_object(obj)
-        #~ if self.attname == 'query_register':
-            #~ print '20120527 value_to_string', repr(value)
+        # ~ if self.attname == 'query_register':
+        # ~ print '20120527 value_to_string', repr(value)
         return self.get_prep_value(value)
-        #~ return self.get_db_prep_value(value,connection)
+        # ~ return self.get_db_prep_value(value,connection)
 
-    #~ def save_form_data(self, instance, data):
-        #~ setattr(instance, self.name, data)
+    # ~ def save_form_data(self, instance, data):
+    # ~ setattr(instance, self.name, data)
 
     def get_text_for_value(self, value):
         return self.choicelist.get_text_for_value(value.value)
@@ -1002,21 +1025,20 @@ class MultiChoiceListField(ChoiceListField):
     A field whose value is a `list` of `Choice` instances.
     Stored in the database as a CharField using a delimiter character.
     """
-    delimiter_char = ','
+
+    delimiter_char = ","
     max_values = 1
 
     def __init__(self, choicelist, verbose_name=None, max_values=10, **kw):
         if verbose_name is None:
             verbose_name = choicelist.verbose_name_plural
         self.max_values = max_values
-        defaults = dict(
-            max_length=(choicelist.max_length + 1) * max_values
-        )
+        defaults = dict(max_length=(choicelist.max_length + 1) * max_values)
         defaults.update(kw)
         ChoiceListField.__init__(self, verbose_name, **defaults)
 
-    #~ def set_max_length(self,ml):
-        #~ self.max_length = (ml+1) * self.max_values
+    # ~ def set_max_length(self,ml):
+    # ~ self.max_length = (ml+1) * self.max_values
 
     def from_db_value(self, value, expression, connection, context=None):
         return [self.choicelist.to_python(v) for v in value.split(self.delimiter_char)]
@@ -1026,8 +1048,8 @@ class MultiChoiceListField(ChoiceListField):
             return []
         if isinstance(value, list):
             return value
-        value = [self.choicelist.to_python(v)
-                 for v in value.split(self.delimiter_char)]
+        value = [self.choicelist.to_python(
+            v) for v in value.split(self.delimiter_char)]
         return value
 
     def get_prep_value(self, value):
@@ -1042,10 +1064,7 @@ class MultiChoiceListField(ChoiceListField):
         return self.get_prep_value(value)
 
     def get_text_for_value(self, value):
-        return ', '.join([
-            self.choicelist.get_text_for_value(bc.value) for bc in value])
-
-
+        return ", ".join([self.choicelist.get_text_for_value(bc.value) for bc in value])
 
 
 class ChoiceSerializer(BaseSerializer):
@@ -1056,13 +1075,20 @@ class ChoiceSerializer(BaseSerializer):
         #         self.value.value),
         #     {'from lino.api.shell import rt'})
 
+
 MigrationWriter.register_serializer(Choice, ChoiceSerializer)
 
+
 class ChoiceListSerializer(BaseSerializer):
     def serialize(self):
-        return ("rt.models.{}.{}".format(
-            self.value.choicelist.app_label, self.value.choicelist.__name__),
-            {'from lino.api.shell import rt'})
+        return (
+            "rt.models.{}.{}".format(
+                self.value.choicelist.app_label, self.value.choicelist.__name__
+            ),
+            {"from lino.api.shell import rt"},
+        )
+
+
 MigrationWriter.register_serializer(ChoiceList, ChoiceListSerializer)
 
 
@@ -1073,4 +1099,5 @@ class CallableChoiceSerializer(BaseSerializer):
         #     choice.choicelist.app_label, choice.choicelist.__name__, choice.name),
         #     {'from lino.api.shell import rt'})
 
+
 MigrationWriter.register_serializer(CallableChoice, CallableChoiceSerializer)
