@@ -76,9 +76,9 @@ class DisableDeleteHandler(object):
                 continue
             if fk.null and fk.remote_field.on_delete == models.SET_NULL:
                 continue
-            n = m.objects.filter(**{fk.name: obj}).count()
-            if n:
-                return obj.delete_veto_message(m, n)
+            qs = m.objects.filter(**{fk.name: obj})
+            if (n := qs.count()):
+                return obj.delete_veto_message(m, n, qs, fk.name)
         kernel = settings.SITE.kernel
         # print "20141208 generic related objects for %s:" % obj
         for gfk, fk_field, qs in kernel.get_generic_related(obj):
@@ -86,9 +86,8 @@ class DisableDeleteHandler(object):
                 continue
             if fk_field.null:  # a nullable GFK is no reason to veto
                 continue
-            n = qs.count()
             # print "20141208 - %s %s %s" % (
             #     gfk.model, gfk.name, qs.query)
-            if n:
-                return obj.delete_veto_message(qs.model, n)
+            if (n := qs.count()):
+                return obj.delete_veto_message(qs.model, n, qs, gfk.name)
         return None
