@@ -8,11 +8,12 @@ from django.conf import settings
 from django.db.models import Model, ForeignKey
 # from lino.utils import MissingRow
 
-if settings.SITE.is_installed('contenttypes'):
+if settings.SITE.is_installed("contenttypes"):
     from lino.modlib.gfks.fields import GenericForeignKey
     from django.contrib.contenttypes.models import ContentType
-    from django.contrib.contenttypes.fields import GenericForeignKey \
-        as DjangoGenericForeignKey
+    from django.contrib.contenttypes.fields import (
+        GenericForeignKey as DjangoGenericForeignKey,
+    )
 
     from django.contrib.contenttypes.fields import GenericRelation, GenericRel
 
@@ -20,6 +21,7 @@ if settings.SITE.is_installed('contenttypes'):
         return isinstance(fld, (ForeignKey, DjangoGenericForeignKey))
 else:
     from lino.core.utils import UnresolvedField, UnresolvedModel
+
     GenericForeignKey = UnresolvedField
     ContentType = UnresolvedModel
     GenericRelation = UnresolvedField
