@@ -1,12 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2022 Rumma & Ko Ltd
+# Copyright 2009-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This defines the :class:`Kernel` class.
 
 The "kernel" of a Lino site is (like `SITE` itself) a "de facto
-singleton", available to application code as ``SITE.kernel`` (and its
-alias for backwards compatibility: ``SITE.ui``).
+singleton", available to application code as ``SITE.kernel``).
 
 The kernel is instantiated at the end of the startup process, when the
 :setting:`SITE` has been instantiated and models have been loaded.  It
@@ -22,51 +20,57 @@ application.
 """
 
 import os
-from os.path import join, dirname, exists
-
 import sys
 import time
 import codecs
 import atexit
+import signal
 import threading
 from importlib import import_module
-import json
-from etgen.html import E
 
-from django.apps import AppConfig
+# from django.apps import AppConfig
 from django.apps import apps
 from django.conf import settings
-from django.utils.encoding import force_str
+from django.http import HttpResponse
 from django.utils.text import format_lazy
 from django.utils.translation import gettext_lazy as _
 from django.core.exceptions import PermissionDenied, ValidationError
 from django.db.utils import DatabaseError
 from django.db import IntegrityError
+from django.utils.log import DEFAULT_LOGGING
 
 from django.db import models
 
-import lino  # for is_testing
+# import lino  # for is_testing
 from lino import logger
 from lino.utils import codetime
+from lino.utils.choosers import check_for_chooser
+from lino.utils.html import E
+# from lino.core.utils import format_request
 # from lino.utils import isiterable
 from lino.core import layouts
 from lino.core import actors
-from lino.core import actions
 from lino.core import frames
-from lino.core import fields
 from lino.core import dbtables
+from lino.core import atomizer
 from lino.core import choicelists
 from lino.core import workflows
 from lino.core import tables
 from lino.core import constants
 from lino.core.model import Model
-from lino.core.roles import UserRole
+# from lino.core.roles import UserRole
+from lino.utils.choosers import discover_choosers
 from lino.core.store import Store
 from lino.core.renderer import HtmlRenderer, TextRenderer
 from lino.core.gfks import ContentType, GenericForeignKey
-from lino.core.signals import (pre_ui_build, post_ui_build,
-                               pre_startup, post_startup,
-                               pre_analyze, post_analyze)
+from lino.core.signals import (
+    pre_ui_build,
+    # post_ui_build,
+    pre_startup,
+    post_startup,
+    pre_analyze,
+    post_analyze,
+)
 
 from .exceptions import ChangedAPI
 from .plugin import Plugin
@@ -77,12 +81,17 @@ from .utils import full_model_name as fmn
 from .utils import obj2str
 from .utils import get_models
 from .utils import resolve_fields_list
-# from .utils import djangoname
-from .utils import class_dict_items
 
-from .fields import set_default_verbose_name
+# from .utils import djangoname
+# from .utils import class_dict_items
+
+from lino.core.fields import set_default_verbose_name
 # from lino.core.requests import ActorRequest
 
+from lino.utils import dbhash
+from lino.core.signals import database_prepared
+database_prepared.connect(dbhash.mark_virgin)
+
 startup_rlock = threading.RLock()  # Lock() or RLock()?
 
 GFK_TARGETS = (models.AutoField, models.IntegerField)
@@ -98,8 +107,9 @@ class Kernel(object):
         Obsolete. Was moved to :mod:`lino.modlib.memo`.
 
     """
-    # default_ui = None  # TODO: rename this to front_end
+
     web_front_ends = None
+
     # admin_ui = None
 
     def __init__(self, site):
@@ -108,94 +118,127 @@ class Kernel(object):
         self.GFK_LIST = []
         # logger.info("20140227 Kernel.__init__() done")
 
-    _code_mtime = None
+    # On a production server we test only the timestamp of settings.py file
+    # because otherwise the result can differ depending on which modules have
+    # already been imported.
+
     _lino_version = None
 
+    def touch_lino_version(self):
+        if is_devserver():
+            self._lino_version = time.time()
+        else:
+            p = self.site.site_dir / "lino_version.txt"
+            p.touch()
+            self._lino_version = p.stat().st_mtime
+
     @property
-    def code_mtime(self):
-        if self._code_mtime is None:
-            # packages = [os.environ['DJANGO_SETTINGS_MODULE'], 'lino']
-            # self._code_mtime = codetime(*packages)
-            if self.site.developer_site_cache:
-                self._code_mtime = codetime()
+    def lino_version(self):
+        if self._lino_version is None:
+            if is_devserver():
+                self._lino_version = codetime()
             else:
-                # On a production server we test only the timestamp of
-                # settings.py file because otherwise the result can differ
-                # depending on which modules have already been imported.
-                self._code_mtime = codetime(settings.SETTINGS_MODULE)
-        return self._code_mtime
+                p = self.site.site_dir / "lino_version.txt"
+                if not p.exists():
+                    p.touch()
+                self._lino_version = p.stat().st_mtime
+        return self._lino_version
+
+    # _code_mtime = None
+    #
+    # @property
+    # def code_mtime(self):
+    #     if self._code_mtime is None:
+    #         # packages = [os.environ['DJANGO_SETTINGS_MODULE'], 'lino']
+    #         # self._code_mtime = codetime(*packages)
+    #         # if self.site.developer_site_cache:
+    #         if is_devserver():
+    #             self._code_mtime = codetime()
+    #         else:
+    #             self._code_mtime = self.lino_version
+    #             # self._code_mtime = codetime(settings.SETTINGS_MODULE)
+    #     return self._code_mtime
+
+    editing_front_end = None
 
     def kernel_startup(self, site):
         """This is a part of a Lino site startup.  The Django Model
         definitions are done, now Lino analyzes them and does certain
         actions:
 
-        - Install a :class:`DisableDeleteHandler
-          <lino.core.ddh.DisableDeleteHandler>` for each Model into
-          `_lino_ddh`.
-
-        - Install :class:`lino.core.model.Model` attributes and
-          methods into Models that don't inherit from it.
-
         """
         # logger.info("20231016 kernel_startup (%s)", site._history_aware_logging)
         # print("20231019 kernel_startup()", site, site._history_aware_logging)
-        if site._history_aware_logging:
-            if len(sys.argv) == 0:
-                process_name = 'WSGI'
-            else:
-                process_name = ' '.join(sys.argv)
 
-            logger.info("Started %s (using %s) --> PID %s",
-                        process_name, settings.SETTINGS_MODULE, os.getpid())
+        # if site._history_aware_logging:
+        # if not is_devserver():
+        if "file" in DEFAULT_LOGGING['handlers']:
+            if len(sys.argv) == 0:
+                process_name = "WSGI"
+            else:
+                process_name = " ".join(sys.argv)
+
+            logger.info(
+                "Started %s (using %s) --> PID %s",
+                process_name,
+                settings.SETTINGS_MODULE,
+                os.getpid(),
+            )
 
             def goodbye():
                 logger.info("Done %s (PID %s)", process_name, os.getpid())
-            atexit.register(goodbye)
+
+            site.register_shutdown_task(goodbye)
+
+        atexit.register(site.shutdown)
+
+        sig = getattr(signal, site.stopsignal)
+
+        def h(signum, frame):
+            site.shutdown()
+            sys.exit()
+
+        signal.signal(sig, h)
 
         models_list = get_models(include_auto_created=True)
         # this also triggers django.db.models.loading.cache._populate()
 
-        site.setup_model_spec(site, 'user_model')
-        site.setup_model_spec(site, 'project_model')
+        site.setup_model_spec(site, "user_model")
+        site.setup_model_spec(site, "project_model")
 
         for app_name_model, p in site.override_modlib_models.items():
             # app_name_model is the full installed app module name +
             # the model name. It certainly contains at least one dot.
-            m = '.'.join(app_name_model.split('.')[-2:])
+            m = ".".join(app_name_model.split(".")[-2:])
             resolve_model(
                 m,
                 strict="{!r} declares to extend unresolved model '%s'".format(
-                    p.__module__))
-                # strict="%s plugin tries to extend unresolved model '%%s'" %
-                # p.__class__.__module__)
+                    p.__module__
+                ),
+            )
+            # strict="%s plugin tries to extend unresolved model '%%s'" %
+            # p.__class__.__module__)
 
         for model in models_list:
-            #~ print 20130216, model
-            #~ fix_field_cache(model)
+            # ~ print 20130216, model
+            # ~ fix_field_cache(model)
 
             # if hasattr(model, '_lino_ddh'):
-            if '_lino_ddh' in model.__dict__:
+            if "_lino_ddh" in model.__dict__:
                 raise Exception("20150831 %s", model)
             model._lino_ddh = DisableDeleteHandler(model)
 
             Model.django2lino(model)
 
-            if isinstance(model.hidden_columns, str):
-                model.hidden_columns = frozenset(
-                    fields.fields_list(model, model.hidden_columns))
+            if model.allow_cascaded_copy is None:
+                model.allow_cascaded_copy = model.allow_cascaded_delete
 
-            if isinstance(model.active_fields, str):
-                model.active_fields = frozenset(
-                    fields.fields_list(model, model.active_fields))
-
-            if isinstance(model.allow_cascaded_delete, str):
-                model.allow_cascaded_delete = frozenset(
-                    fields.fields_list(model, model.allow_cascaded_delete))
-
-            if isinstance(model.allow_cascaded_copy, str):
-                model.allow_cascaded_copy = frozenset(
-                    fields.fields_list(model, model.allow_cascaded_copy))
+            # 'suppress_cascaded_copy'
+            for k in ('hidden_columns', 'active_fields',
+                      'allow_cascaded_delete', 'allow_cascaded_copy'):
+                # resolve_fields_list(model, k, frozenset, frozenset())
+                if (v := getattr(model, k, None)) is not None:
+                    setattr(model, k, atomizer.fields_list(model, v))
 
             # Note how to inherit this from from parent model.
             if model.quick_search_fields is None:
@@ -205,17 +248,16 @@ class Kernel(object):
                         fields_list.append(field)
                 model.quick_search_fields = tuple(fields_list)
             else:
-                resolve_fields_list(model, 'quick_search_fields')
+                resolve_fields_list(model, "quick_search_fields")
 
             if model.quick_search_fields_digit is None:
                 fields_list = []
                 for field in model._meta.fields:
-                    if isinstance(field, (
-                            models.IntegerField, models.AutoField)):
+                    if isinstance(field, (models.IntegerField, models.AutoField)):
                         fields_list.append(field)
                 model.quick_search_fields_digit = tuple(fields_list)
             else:
-                resolve_fields_list(model, 'quick_search_fields_digit')
+                resolve_fields_list(model, "quick_search_fields_digit")
 
             if model._meta.abstract:
                 raise Exception("Tiens?")
@@ -266,7 +308,7 @@ class Kernel(object):
         for model in models_list:
             if model._meta.auto_created:
                 continue  # automatic intermediate models created by
-                          # ManyToManyField should not disable delete
+                # ManyToManyField should not disable delete
             # for f, m in model._meta.get_fields_with_model():
             for f in model._meta.get_fields():
                 m = f.model
@@ -277,26 +319,28 @@ class Kernel(object):
                 # if f.__class__ is models.CharField and f.null:
                 #     msg = "Nullable CharField %s in %s" % (f.name, model)
                 #     raise Exception(msg)
+
                 if isinstance(f, models.ForeignKey):
+                    # f.remote_field.model = resolve_model(f.remote_field.model)
                     if isinstance(f.remote_field.model, str):
-                        raise Exception("Could not resolve target %r of "
-                                        "ForeignKey '%s' in %s "
-                                        "(models are %s)" %
-                                        (f.remote_field.model, f.name, model, models_list))
+                        # models_list = [f"{m}({m._meta.abstract})".format(m) for m in models_list]
+                        raise Exception(
+                            "Could not resolve target %r of "
+                            "ForeignKey '%s' in %s "
+                            "(models are %s)"
+                            % (f.remote_field.model, f.name, model, models_list)
+                        )
 
                     set_default_verbose_name(f)
-
-                    """
-                    If JobProvider is an MTI child of Company,
-                    then mti.delete_child(JobProvider) must not fail on a
-                    JobProvider being referred only by objects that can refer
-                    to a Company as well.
-                    """
-                    if not hasattr(f.remote_field.model, '_lino_ddh'):
-                        msg = "20150824 {1} (needed by {0}) "\
-                              "has no _lino_ddh"
-                        raise Exception(msg.format(
-                            f.remote_field, f.remote_field.model))
+                    # If JobProvider is an MTI child of Company,
+                    # then mti.delete_child(JobProvider) must not fail on a
+                    # JobProvider being referred only by objects that can refer
+                    # to a Company as well.
+                    if not hasattr(f.remote_field.model, "_lino_ddh"):
+                        msg = "20150824 {1} (needed by {0}) " "has no _lino_ddh"
+                        raise Exception(
+                            msg.format(f.remote_field, f.remote_field.model)
+                        )
                     f.remote_field.model._lino_ddh.add_fk(m or model, f)
 
             # if not site.quick_startup:
@@ -337,7 +381,7 @@ class Kernel(object):
         for model in models_list:
             model.on_analyze(site)
 
-        #~ logger.info("20130817 attached model vfs")
+        # ~ logger.info("20130817 attached model vfs")
 
         # Attach virtual fields to the model that declares them   This must be
         # done before calling Model.site_setup(), e.g. because
@@ -406,7 +450,7 @@ class Kernel(object):
                 continue
             # Not getattr but __dict__.get because of the mixins.Listings
             # trick:
-            rpt = model.__dict__.get('_lino_default_table', None)
+            rpt = model.__dict__.get("_lino_default_table", None)
             # rpt = getattr(model,'_lino_default_table',None)
             # logger.debug('20111113 %s._lino_default_table = %s',model,rpt)
             if rpt is None:
@@ -420,8 +464,13 @@ class Kernel(object):
                 # rpt.collect_actions()
                 model._lino_default_table = rpt
 
-        #~ choosers.discover()
-        actions.discover_choosers()
+        # ~ choosers.discover()
+        discover_choosers()
+
+        for a in actors.actors_list:
+            for name, field in a.virtual_fields.items():
+                assert name == field.name
+                check_for_chooser(a, field)
 
         for a in actors.actors_list:
             a.on_analyze(site)
@@ -433,20 +482,30 @@ class Kernel(object):
                 except DatabaseError:
                     logger.debug(
                         "Ignoring DatabaseError in %s.setup_columns", a)
+            if (
+                issubclass(a, dbtables.Table)
+                and a.model is not None
+                and a.live_panel_update
+            ):
+                if a.model.updatable_panels is None:
+                    a.model.updatable_panels = []
+                if a.actor_id not in a.model.updatable_panels:
+                    a.model.updatable_panels.append(a.actor_id)
 
         post_analyze.send(site, models_list=models_list)
 
         if False:
-            logger.info("Languages: %s. %d apps, %d models, %s actors.",
-                        ', '.join([li.django_code for li in site.languages]),
-                        len(site.modules),
-                        len(models_list),
-                        len(actors.actors_list))
+            logger.info(
+                "Languages: %s. %d apps, %d models, %s actors.",
+                ", ".join([li.django_code for li in site.languages]),
+                len(site.modules),
+                len(models_list),
+                len(actors.actors_list),
+            )
 
-        #~ logger.info(settings.INSTALLED_APPS)
-
-        site.on_each_app('site_setup')
+        # ~ logger.info(settings.INSTALLED_APPS)
 
+        site.on_each_app("site_setup")
 
         # Actor.after_site_setup() is called after the plugins's
         # site_setup().  Example: pcsw.site_setup() adds a detail to
@@ -462,8 +521,11 @@ class Kernel(object):
 
         later = []
         for a in actors.actors_list:
-            if isinstance(a.model, type) and issubclass(a.model, models.Model) \
-                    and a == a.model.get_default_table():
+            if (
+                isinstance(a.model, type)
+                and issubclass(a.model, models.Model)
+                and a == a.model.get_default_table()
+            ):
                 a.after_site_setup(site)
             else:
                 later.append(a)
@@ -491,7 +553,9 @@ class Kernel(object):
         for a in actors.actors_list:
             for i, ea in enumerate(a.collect_extra_actions()):
                 a._bind_action(ea.action_name, ea, False)
-            a._actions_list = sorted(a._actions_list, key=lambda ba: ba.action.sort_index)
+            a._actions_list = sorted(
+                a._actions_list, key=lambda ba: ba.action.sort_index
+            )
 
         # for p in site.installed_plugins:
         #     p.after_discover()
@@ -536,13 +600,18 @@ class Kernel(object):
             # print("\n".join(["- {0.url_prefix} --> {0.app_name} {1}".format(p, hash(p))
             #     for p in self.web_front_ends]))
 
+        editing_wf = None
+        primary_wf = self.web_front_ends[0]
         for p in self.web_front_ends:
+            # if primary_wf is None and not p.url_prefix:
+            #     primary_wf = p
             if p.ui_handle_attr_name is not None:
                 editing_wf = p
                 break
 
         self.editing_front_end = editing_wf
-        self.html_renderer = HtmlRenderer(self.web_front_ends[0])
+        self.primary_front_end = primary_wf
+        self.html_renderer = HtmlRenderer(primary_wf)
         self.text_renderer = TextRenderer(editing_wf)
 
         for p in site.installed_plugins:
@@ -567,11 +636,11 @@ class Kernel(object):
 
             if a.get_welcome_messages is not None:
                 site.add_welcome_handler(
-                    a.get_welcome_messages, a, "get_welcome_messages")
+                    a.get_welcome_messages, a, "get_welcome_messages"
+                )
             if a.welcome_message_when_count is not None:
 
                 def handler(cls):
-
                     def get_welcome_messages(ar):
                         sar = ar.spawn(cls)
                         # if not cls.get_view_permission(ar.get_user().user_type):
@@ -583,12 +652,13 @@ class Kernel(object):
                             chunks = [str(_("You have "))]
                             txt = _("{0} items in {1}").format(num, cls.label)
                             chunks.append(ar.href_to_request(sar, txt))
-                            chunks.append('.')
+                            chunks.append(".")
                             yield E.span(*chunks)
+
                     return get_welcome_messages
 
                 site.add_welcome_handler(
-                    handler(a), a, 'welcome_message_when_count')
+                    handler(a), a, "welcome_message_when_count")
 
         # Remove "meaningless" user roles, i.e. those which are either used by all
         # types or by no type. This seems to be
@@ -606,13 +676,36 @@ class Kernel(object):
         #         must_remove.add(r)
         # site.user_roles -= must_remove
 
-        post_ui_build.send(self)
+        # post_ui_build.send(self)
 
-        # trigger creation of params_layout.params_store
+        # trigger creation of params_layout.params_store and
+        # virtual fields in LightWeightContainer
         for res in actors.actors_list:
+            field_actions = dict()
             for ba in res.get_actions():
+                # ba._started = True
+                if ba.help_text is None:
+                    ba.help_text = ba.action.get_help_text(ba)
                 if ba.action.params_layout is not None:
                     ba.action.params_layout.get_layout_handle()
+                if ba.action.is_window_action():
+                    ba.get_layout_handel()
+                if ba.action.buddy_name:
+                    lst = field_actions.setdefault(ba.action.buddy_name, [])
+                    lst.append(ba)
+            res._field_actions = field_actions
+
+        # Loop over all actors, get their handle created and only then we
+        # initialize the stores. This is here to fix #6404 (Inconsistent dataKey
+        # in LightWeightContainer values). which happened because the virtual
+        # fields for slave panels were getting created multiple times
+        # (CommentsByRFC1 and CommentsByRFC2)
+        for a in actors.actors_list:
+            # if not a.abstract and a.model and a.get_handle_name is None:
+            if a.get_handle_name is None and not a.is_abstract():
+                # print("20251007", a)
+                h = a.get_handle()
+                h.store.init_store()
 
         # logger.info("20161219 kernel_startup done")
 
@@ -633,7 +726,7 @@ class Kernel(object):
 
         Note that this does not protect FK fields that get defined afterwards,
         e.g. during pre_analyze, e.g. the purchase_account field defined by
-        TradeTypes.purchases in ledger.
+        TradeTypes.purchases in accounting.
 
         """
 
@@ -666,28 +759,32 @@ class Kernel(object):
                         fk.remote_field.on_delete = models.PROTECT
                 else:
                     if fk.name in m.allow_cascaded_delete:
-                        msg = ("{0}.{1} specified in allow_cascaded_delete "
-                               "but on_delete is not CASCADE").format(
-                            fmn(m), fk.name)
+                        msg = (
+                            "{0}.{1} specified in allow_cascaded_delete "
+                            "but on_delete is not CASCADE"
+                        ).format(fmn(m), fk.name)
                         raise Exception(msg)
 
                     if fk.remote_field.on_delete == models.SET_NULL:
                         if not fk.null:
-                            msg = ("{0}.{1} has on_delete SET_NULL but "
-                                   "is not nullable ")
-                            msg = msg.format(fmn(m), fk.name, fk.remote_field.model)
+                            msg = (
+                                "{0}.{1} has on_delete SET_NULL but " "is not nullable "
+                            )
+                            msg = msg.format(
+                                fmn(m), fk.name, fk.remote_field.model)
                             raise Exception(msg)
 
                     else:
                         msg = ("{0}.{1} has custom on_delete").format(
-                            fmn(m), fk.name, fk.remote_field.on_delete)
+                            fmn(m), fk.name, fk.remote_field.on_delete
+                        )
                         logger.debug(msg)
 
     def get_generic_related(self, obj):
         """Yield a series of `(gfk, fk_field, queryset)` tuples which together
-         will return all database objects for which the given
-         GenericForeignKey gfk points to the object `obj`. See also
-         :doc:`/dev/gfks`.
+        will return all database objects for which the given
+        GenericForeignKey gfk points to the object `obj`. See also
+        :doc:`/dev/gfks`.
 
         """
         if len(self.GFK_LIST) == 0:
@@ -698,7 +795,7 @@ class Kernel(object):
         if not isinstance(obj._meta.pk, GFK_TARGETS):
             # raise Exception("20150330 %s", obj._meta.pk)
             return  # e.g. Country.iso_code is a CharField, cannot
-                    # point to a country using GFK
+            # point to a country using GFK
         assert not obj._meta.abstract
         obj_ct = ContentType.objects.get_for_model(obj.__class__)
         # logger.info("20150330 ok %s", obj_ct)
@@ -736,7 +833,7 @@ class Kernel(object):
                 fk_field = gfk.model._meta.get_field(gfk.fk_field)
                 # fk_field, remote_model, direct, m2m = \
                 #     gfk.model._meta.get_field_by_name(gfk.fk_field)
-                kw = {gfk.ct_field+'__isnull': False}
+                kw = {gfk.ct_field + "__isnull": False}
                 qs = model.objects.filter(**kw)
                 for obj in qs:
                     fk = getattr(obj, gfk.fk_field)
@@ -750,11 +847,11 @@ class Kernel(object):
                         obj._message = msg.format(
                             gfk.fk_field, fk, fmn(pointed_model))
                         if gfk.name in model.allow_cascaded_delete:
-                            obj._todo = 'delete'
+                            obj._todo = "delete"
                         elif fk_field.null:
-                            obj._todo = 'clear'
+                            obj._todo = "clear"
                         else:
-                            obj._todo = 'manual'
+                            obj._todo = "manual"
                         yield obj
 
     def abandon_response(self):
@@ -769,9 +866,11 @@ class Kernel(object):
 
         """
         if not ar.get_permission():
-            # msg = "{} has no permission to run this request".format(
-            #     ar.get_user())
-            msg = "No permission to run {}".format(ar)
+            if False:
+                msg = "{} has no permission to run {}".format(
+                    ar.get_user(), ar)
+            else:
+                msg = "No permission to run {}".format(ar)
             # raise Exception(msg)
             raise PermissionDenied(msg)
 
@@ -789,20 +888,43 @@ class Kernel(object):
                 A(ar.bound_action.full_name())
                 A(obj2str(ar.master_instance))
                 A(obj2str(ar.selected_rows))
+                # The following caused #5849 (You cannot access body after
+                # reading from request's data stream) on a production site:
                 # A(format_request(ar.request))
-                ar.logger.info("run_action {0}".format(' '.join(flds)))
+                ar.logger.info("run_action {0}".format(" ".join(flds)))
                 # logger.info("run_action {0}".format(ar))
+        # if str(a) != 'grid_put':
+        #     raise Exception(f"20240911 run_action() {a}")
         try:
-            a.run_from_ui(ar)
-            if ar.request.method != 'GET' and ar.master_instance is not None and len(ar.selected_rows):
-                ar.set_response(master_data=ar.selected_rows[0].get_master_data(ar))
-            if a.parameters and not a.no_params_window:
+            if isinstance((resp := a.run_from_ui(ar)), HttpResponse):
+                return resp
+            if (
+                ar.request.method != "GET"
+                and ar.master_instance is not None
+                and len(ar.selected_rows)
+            ):
+                ar.set_response(
+                    master_data=ar.selected_rows[0].get_master_data(ar))
+            if (
+                a.parameters
+                and not a.no_params_window
+                and ar.response.get("success", False)
+            ):
                 ar.set_response(close_window=True)
-        except (ValidationError, IntegrityError) as e:
-            # logger.info("20150127 run_action %r", e)
-            ar.error(ar.ah.actor.error2str(e), alert=True)
         except Warning as e:
             ar.error(e, alert=True)
+        except Exception as e:
+            # raise  # A plain traceback can be easier to debug.
+            # print(f"20240911 oops {repr(e)}")
+            msg = ar.ah.actor.error2str(e)
+            ar.error(msg, alert=True)
+            if not isinstance(e, (ValidationError, IntegrityError)):
+                logger.info(
+                    "Error during kernel.run_action() in %s: %s", ar, msg)
+                if is_devserver():
+                    # can be interesting during development but disturbs on a
+                    # production server
+                    logger.exception(e)
 
         return ar.renderer.render_action_response(ar)
 
@@ -812,7 +934,7 @@ class Kernel(object):
         every actor handle because it potentially requires other actor
         handles to be instantiated.
 
-        ar is usually None, except for actors with dynamic handle
+        `ar` is usually None, except for actors with dynamic handle
         """
         # logger.info('20121010 Kernel.setup_handle() %s', h.actor)
 
@@ -822,12 +944,11 @@ class Kernel(object):
 
         if isinstance(h, tables.TableHandle):
             he = set(h.actor.hidden_columns | h.actor.hidden_elements)
-            ll = layouts.ColumnsLayout(
-                h.actor.get_column_names(ar),
-                h.actor, hidden_elements=he)
-            h.list_layout = ll.get_layout_handle()
+            cl = layouts.ColumnsLayout(
+                h.actor.get_column_names(ar), h.actor, hidden_elements=he)
+            h.grid_layout = cl.get_layout_handle()
         else:
-            h.list_layout = None
+            h.grid_layout = None
 
         if h.actor.params_layout:
             h.params_layout_handle = h.actor.make_params_layout_handle()
@@ -846,7 +967,7 @@ class Kernel(object):
         self._must_build = True
 
     def make_cache_file(self, fn, write, force=False, verbosity=1):
-        """Make the specified cache file.  This is used internally at server
+        """Make the specified cache file.  This is used internally at site
         startup.
 
         """
@@ -855,11 +976,10 @@ class Kernel(object):
         #     logger.info("STATIC_ROOT does not exist: %s", settings.STATIC_ROOT)
         #     return 0
         fn = self.site.media_root / fn
-        # fn = join(settings.STATIC_ROOT, fn)
-        # fn = join(self.site.site_dir, fn)
-        if not force and not self._must_build and exists(fn):
+        if not force and not self._must_build and fn.exists():
             mtime = os.stat(fn).st_mtime
-            if mtime > self.code_mtime:
+            # if mtime > self.code_mtime:
+            if mtime > self.lino_version:
                 # logger.debug("%s (%s) is up to date.", fn, time.ctime(mtime))
                 return 0
 
@@ -876,8 +996,8 @@ class Kernel(object):
         if verbosity > 1:
             logger.info("Building %s ...", fn)
 
-        self.site.makedirs_if_missing(dirname(fn))
-        f = codecs.open(fn, 'w', encoding='utf-8')
+        self.site.makedirs_if_missing(fn.parent)
+        f = codecs.open(fn, "w", encoding="utf-8")
         try:
             write(f)
             f.close()
@@ -887,52 +1007,11 @@ class Kernel(object):
             if not self.site.keep_erroneous_cache_files:  #
                 os.remove(fn)
             raise
-        #~ logger.info("Wrote %s ...", fn)
+        # ~ logger.info("Wrote %s ...", fn)
 
     # def setup_static_link(self, urlpatterns, short_name,
     #                       attr_name=None, source=None):
 
-    def mark_virgin(self):
-        dbhash = self.get_dbhash()
-        fn = self.site.site_dir / "dbhash.json"
-        with fn.open("w") as fp:
-            json.dump(dbhash, fp)
-
-    def check_virgin(self):
-        new = self.get_dbhash()
-        fn = self.site.site_dir / "dbhash.json"
-        if not fn.exists():
-            raise Exception("No `dbhash.json` in {} "
-                "(did you run `django-admin prep`?)".format(self.site.site_dir))
-        with fn.open("r") as fp:
-            old = json.load(fp)
-
-        # noi1r has noi1e as master_site, but the react front end removes the
-        # tinymce plugin, i.e. noi1r doesn't care about
-        # tinymce.TextFieldTemplate model.
-
-        diffs = {}
-        for k, v in new.items():
-            oldv = old.get(k, None)
-            if oldv != v:
-                diffs[k] = (oldv, v)
-        # diff = set(old.items()) ^ set(new.items())
-        if diffs:
-            # db = self.site.django_settings.get('SETTINGS_MODULE')
-            db = self.site.site_dir
-            raise Exception("Database {} isn't virgin: {}".format(db, diffs))
-
-        # logger.info("Database certified as virgin")
-
-    def get_dbhash(self):
-        # TODO: we currently check only the number of rows per model. That's
-        # very naive.
-        rv = dict()
-        for m in get_models(include_auto_created=True):
-            k = fmn(m)
-            if k != "sessions.Session":
-                rv[k] = m.objects.count()
-        return rv
 
 def site_startup(self):
     """This is being imported and called from
@@ -941,7 +1020,6 @@ def site_startup(self):
 
     """
     with startup_rlock:
-
         # if self.site_dir is not None:
         #     raise Exception("No site_dir is defined. "
         #                     "Check your LINO_CACHE_ROOT and project_name.")
@@ -1009,7 +1087,7 @@ def site_startup(self):
         pre_startup.send(self)
 
         for p in self.installed_plugins:
-            p.on_site_startup(self)
+            p.pre_site_startup(self)
 
         # for k, v in self.models.items():
         #     self.actors.setdefault(k, v)
@@ -1032,7 +1110,7 @@ def site_startup(self):
         # get_atomizer(contacts.Persons, contacts.RolesByPerson,
         # contacts.RolesByPerson) returned None" because the store fields for
         # slave tables were not yet created. Activating the following code fixes
-        # this issue bu causes other problems:
+        # this issue but causes other problems:
         if False:
             for res in actors.actors_list:
                 if res.detail_layout is not None:
@@ -1058,6 +1136,7 @@ frames_list = []
 virtual_tables = []
 abstract_tables = []
 
+
 def get_choicelist(i):
     return CHOICELISTS[i]
 
@@ -1079,6 +1158,7 @@ def choicelist_choices():
     l.sort(key=lambda x: x[0])
     return l
 
+
 def is_candidate(T):
     if T.filter or T.exclude or T.known_values:
         return False
@@ -1118,7 +1198,6 @@ def register_actors():
             if rpt not in (choicelists.ChoiceList, workflows.Workflow):
                 register_choicelist(rpt)
 
-
     # logger.debug("Create default tables...")
     # for model in get_models():
     #     # Note that automatic models (created by ManyToManyField with
@@ -1147,8 +1226,8 @@ def register_actors():
             continue
         if not isinstance(rpt.master, type):
             raise Exception(
-                "20160712 invalid master {!r} in {}".format(
-                    rpt.master, rpt))
+                "20160712 invalid master {!r} in {}".format(rpt.master, rpt)
+            )
 
         if issubclass(rpt.master, models.Model):
             # rpt.master = resolve_model(rpt.master)
@@ -1162,21 +1241,25 @@ def register_actors():
 
     # logger.debug("reports.setup() done")
 
+
 def register_frame(frm):
     frames_list.append(frm)
 
+
 def register_choicelist(cl):
-    #~ print '20121209 register_choicelist', cl
-    #~ k = cl.stored_name or cl.__name__
+    # ~ print '20121209 register_choicelist', cl
+    # ~ k = cl.stored_name or cl.__name__
     k = cl.stored_name or cl.actor_id
     if k in CHOICELISTS:
         raise Exception(
             "Cannot register %r : actor name '%s' "
-            "already defined by %r" % (cl, k, CHOICELISTS[k]))
+            "already defined by %r" % (cl, k, CHOICELISTS[k])
+        )
         # logger.warning("ChoiceList name '%s' already defined by %s",
         #                k, CHOICELISTS[k])
     CHOICELISTS[k] = cl
 
+
 def register_model_table(rpt):
     # logger.debug("20120103 register_report %s", rpt.actor_id)
 
@@ -1191,7 +1274,7 @@ def register_model_table(rpt):
         if not rpt.is_abstract():
             # logger.debug("20120102 register %s : master report", rpt.actor_id)
             master_tables.append(rpt)
-        if '_lino_default_table' not in rpt.model.__dict__:
+        if "_lino_default_table" not in rpt.model.__dict__:
             if is_candidate(rpt):
                 rpt.model._lino_default_table = rpt
     elif rpt.master is ContentType:
