@@ -213,7 +213,7 @@ class Model(models.Model, fields.TableRow):
             raise Warning(msg)
         super(Model, self).delete(**kw)
 
-    def delete_veto_message(self, m, n):
+    def delete_veto_message(self, m, n, qs, fkname):
         msg = _(
             "Cannot delete %(model)s %(self)s "
             "because %(count)d %(refs)s refer to it."
@@ -224,6 +224,7 @@ class Model(models.Model, fields.TableRow):
             refs=m._meta.verbose_name_plural or m._meta.verbose_name + "s",
         )
         # ~ print msg
+        msg += f" ({qs} via {fkname})"
         return msg
 
     @classmethod
@@ -1038,9 +1039,8 @@ def pre_delete_handler(sender, instance=None, **kw):
             if fk_field.null:  # clear nullable GFKs
                 for obj in qs:
                     setattr(obj, gfk.name, None)
-            elif qs.count():
-                raise Warning(instance.delete_veto_message(
-                    qs.model, qs.count()))
+            elif (n := qs.count()):
+                raise Warning(instance.delete_veto_message(qs.model, n, qs, gfk.name))
     for qs in must_cascade:
         if qs.count():
             logger.info(
