@@ -1,6 +1,5 @@
 # Copyright 2013-2020 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This defines Lino's standard system signals.
 
 See :doc:`/dev/signals`.
@@ -22,7 +21,9 @@ pre_add_child = Signal()  # ['request'])
 on_ui_created = Signal()  # ['request'])
 on_ui_updated = Signal()  # ['request', 'watcher'])
 pre_ui_save = Signal()  # ['instance', 'ar'])
+post_ui_save = Signal()  # ['instance', 'ar'])
 pre_ui_delete = Signal()  # ['request'])
 pre_ui_build = Signal()
-post_ui_build = Signal()
+# post_ui_build = Signal()
 # database_connected = Signal()
+database_prepared = Signal()
