@@ -1,13 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2023 Rumma & Ko Ltd
+# Copyright 2009-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the classes :class:`AbstractTable` and
 :class:`VirtualTable`.
 
 """
-from lino import logger
-from future.utils import with_metaclass
 
 import os
 import yaml
@@ -16,35 +13,33 @@ from django.db import models
 from django.db.utils import OperationalError, ProgrammingError
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
+# from django.core.exceptions import BadRequest
 
+from lino import logger
+from lino.core import constants
 from lino.core import actors
 from lino.core import actions
 from lino.core import fields
-from lino.core import constants
+from lino.core import layouts
 from lino.core.utils import resolve_fields_list
-from lino.core.tablerequest import TableRequest
-
 
 # class InvalidRequest(Exception):
 #     pass
 
-
 if False:  # 20130710
-
     from lino.utils.config import Configured
 
     class GridConfig(Configured):
-
         def __init__(self, report, data, *args, **kw):
             self.report = report
             self.data = data
-            self.label_en = data.get('label')
+            self.label_en = data.get("label")
             self.data.update(label=_(self.label_en))
             super(GridConfig, self).__init__(*args, **kw)
             must_save = self.validate()
             if must_save:
                 msg = self.save_config()
-                #~ msg = self.save_grid_config()
+                # ~ msg = self.save_grid_config()
                 logger.debug(msg)
 
         def validate(self):
@@ -53,32 +48,32 @@ if False:  # 20130710
             """
             must_save = False
             gc = self.data
-            columns = gc['columns']
+            columns = gc["columns"]
             col_count = len(columns)
-            widths = gc.get('widths', None)
-            hiddens = gc.get('hiddens', None)
+            widths = gc.get("widths", None)
+            hiddens = gc.get("hiddens", None)
             if widths is None:
                 widths = [None for x in columns]
                 gc.update(widths=widths)
             elif col_count != len(widths):
-                raise Exception("%d columns, but %d widths" %
-                                (col_count, len(widths)))
+                raise Exception("%d columns, but %d widths" % (col_count, len(widths)))
             if hiddens is None:
                 hiddens = [False for x in columns]
                 gc.update(hiddens=hiddens)
             elif col_count != len(hiddens):
-                raise Exception("%d columns, but %d hiddens" %
-                                (col_count, len(hiddens)))
+                raise Exception(
+                    "%d columns, but %d hiddens" % (col_count, len(hiddens))
+                )
 
             valid_columns = []
             valid_widths = []
             valid_hiddens = []
-            for i, colname in enumerate(gc['columns']):
+            for i, colname in enumerate(gc["columns"]):
                 f = self.report.get_data_elem(colname)
                 if f is None:
                     logger.debug(
-                        "Removed unknown column %d (%r). Must save.",
-                        i, colname)
+                        "Removed unknown column %d (%r). Must save.", i, colname
+                    )
                     must_save = True
                 else:
                     valid_columns.append(colname)
@@ -114,12 +109,12 @@ class TableHandle(object):
         self.actor = actor
 
     def __str__(self):
-        return str(self.actor) + 'Handle'
+        return str(self.actor) + "Handle"
 
     def setup_layouts(self):
         if self._layouts is not None:
             return
-        self._layouts = [self.list_layout]
+        self._layouts = [self.grid_layout]
 
     def get_actor_url(self, *args, **kw):
         return settings.SITE.kernel.get_actor_url(self.actor, *args, **kw)
@@ -127,13 +122,13 @@ class TableHandle(object):
     def submit_elems(self):
         return []
 
-    def get_list_layout(self):
+    def get_grid_layout(self):
         self.setup_layouts()
         return self._layouts[0]
 
     def get_columns(self):
-        lh = self.get_list_layout()
-        #~ print 20110315, layout._main.columns
+        lh = self.get_grid_layout()
+        # ~ print 20110315, layout._main.columns
         return lh.main.columns
 
     def get_slaves(self):
@@ -141,20 +136,19 @@ class TableHandle(object):
 
 
 class Group(object):
-
     def __init__(self):
         self.sums = []
 
     def process_row(self, collector, row):
         collector.append(row)
 
-    #~ def add_to_table(self,table):
-        #~ self.table = table
-        #~ for col in table.computed_columns.values():
+    # ~ def add_to_table(self,table):
+    # ~ self.table = table
+    # ~ for col in table.computed_columns.values():
 
 
 class AbstractTable(actors.Actor):
-    """An AbstractTable is the definition of a tabular data view, usually
+    """An AbstractTable is the definition of a tabular data table, usually
     displayed in a Grid (but it's up to the user interface to decide
     how to implement this).
 
@@ -166,6 +160,8 @@ class AbstractTable(actors.Actor):
     abstract = True
 
     _handle_class = TableHandle
+    _detail_action_class = actions.ShowDetail
+    _params_layout_class = layouts.ParamsLayout
 
     hide_zero_rows = False
     """Set this to `True` if you want to remove rows which contain no
@@ -174,7 +170,7 @@ class AbstractTable(actors.Actor):
 
     """
 
-    column_names = '*'
+    column_names = "*"
 
     tablet_columns = None
     """
@@ -197,17 +193,15 @@ class AbstractTable(actors.Actor):
     will not pop-in.
     """
 
-    # start_at_bottom = False
-    # """Set this to `True` if you want your table to *start at the
-    # bottom*.  Unlike reverse ordering, the rows remain in their
-    # natural order, but when we open a grid on this table, we want it
-    # to start on the last page.
-    #
-    # Use cases would be :class:`lino_xl.lib.sales.InvoicesByJournal` and
-    # :class:`lino_xl.lib.ledger.InvoicesByJournal` but the result is not yet
-    # satisfying.
-    #
-    # """
+    start_at_bottom = False
+    """Set this to `True` if you want your table to *start at the
+    bottom*.  Unlike reverse ordering, the rows remain in their
+    natural order, but when we open a grid on this table, we want it
+    to start on the last page.
+
+    Use cases are in :class:`lino_xl.lib.trading.InvoicesByJournal` and
+    :class:`lino_xl.lib.accounting.InvoicesByJournal`.
+    """
 
     group_by = None
     """
@@ -235,7 +229,7 @@ class AbstractTable(actors.Actor):
     it. e.g. in case they need local filtering.
 
     This will be called with a
-    :class:`lino.core.requests.TableRequest` object and is expected to
+    :class:`lino.core.requests.ActionRequest` object and is expected to
     return or yield the list of "rows"::
 
         @classmethod
@@ -321,36 +315,56 @@ class AbstractTable(actors.Actor):
     Usually such a warning means that there is something wrong.
     """
 
-    display_mode = (
-        (70, constants.DISPLAY_MODE_LIST),
-        (None, constants.DISPLAY_MODE_TABLE),
-    )
-    """
-    A tuple of `(width, display_mode)` items used to select a :term:`display
-    mode` depending on the available width (in characters) on the client.
+    table_as_calendar = False
+    """Whether the primereact Datatable is used to display a calendar view.
 
-    The last item should have a width of `None`.
-
-    See also :meth:`get_display_mode` and :ref:`dev.format`.
+    Used only in :term:`React front end`.
     """
 
+    default_display_modes = {
+        70: constants.DISPLAY_MODE_SUMMARY,
+        None: constants.DISPLAY_MODE_GRID
+    }
+    """
+    Which :term:`display mode` to use in a :term:`slave panel`,
+    depending on available width.
+
+    See :ref:`dg.table.default_display_modes`.
+    """
+
+    parent_layout = None
+
     @classmethod
-    def get_display_mode(cls, available_width=None):
-        """Given the available width, return the :term:`display mode` to use.
-        """
-        current_choice = (0, None)
-        for dmi in cls.display_mode:
-            if available_width is not None:
-                if dmi[0] is None and current_choice[0] == 0:
-                    current_choice = (0, dmi[1])
-                elif available_width > dmi[0] > current_choice[0]:
-                    current_choice = dmi
-            elif dmi[0] is None:
-                return dmi[1]
-        return current_choice[1]
-
-
+    def attach_to_parent_layout(cls, parent):
+        cls.parent_layout = parent
 
+    @classmethod
+    # def get_display_mode(cls, available_width=None):
+    def get_display_mode(cls):
+        """Return the fallback :term:`display mode` to use."""
+        return cls.default_display_modes[None]
+        # rv = cls.default_display_modes[None]
+        # if available_width is not None:
+        #     found = None
+        #     for w, v in cls.default_display_modes.items():
+        #         if w is not None and w <= available_width:
+        #             if found is None:
+        #                 found = w
+        #                 rv = v
+        #             elif found > w:
+        #                 found = w
+        #                 rv = v
+        # return rv
+        # current_choice = (0, None)
+        # for dmi in cls.display_mode:
+        #     if available_width is not None:
+        #         if dmi[0] is None and current_choice[0] == 0:
+        #             current_choice = (0, dmi[1])
+        #         elif available_width > dmi[0] > current_choice[0]:
+        #             current_choice = dmi
+        #     elif dmi[0] is None:
+        #         return dmi[1]
+        # return current_choice[1]
 
     max_render_depth = 2
     """
@@ -362,7 +376,6 @@ class AbstractTable(actors.Actor):
     In a detail view if a slave table is empty, it's element will be hidden
     """
 
-
     stay_in_grid = False
     """Set this to True if Lino should prefer grid mode and not open a
     detail window on a newly created record.  :class:`SubmitDetail
@@ -382,7 +395,7 @@ class AbstractTable(actors.Actor):
     existing entries.  End users can double-click on the phantom row to open an
     insert window and create a new item.
 
-    Used for :class:`lino_xl.lib.ledger.ByJournal` where a phantom row
+    Used for :class:`lino_xl.lib.accounting.ByJournal` where a phantom row
     is disturbing and not needed.
 
     """
@@ -390,41 +403,42 @@ class AbstractTable(actors.Actor):
     order_by = None
     """If specified, this must be a tuple or list of field names that
 will be passed to Django's `order_by
-<https://docs.djangoproject.com/en/5.0/ref/models/querysets/#order-by>`__
+<https://docs.djangoproject.com/en/5.2/ref/models/querysets/#order-by>`__
 method in order to sort the rows of the queryset.
 
     """
 
     filter = None
-    """If specified, this must be a `models.Q` object (not a dict of
-    (fieldname -> value) pairs) which will be passed to Django's
-    `filter
-    <https://docs.djangoproject.com/en/5.0/ref/models/querysets/#filter>`__
+    """
+    If specified, this must be a :class:`django.db.models.Q` object that will be
+    passed to Django's `filter
+    <https://docs.djangoproject.com/en/5.2/ref/models/querysets/#filter>`__
     method.
 
-    Note that if you allow a user to insert rows into a filtered
-    table, you should make sure that new records satisfy your filter
-    condition, otherwise you can get surprising behaviour if the user
-    creates a new row.
+    If you allow a user to insert rows into a filtered table, you should make
+    sure that new records satisfy your filter condition, otherwise you can get
+    surprising behaviour if the user creates a new row.
 
-    If your filter consists of simple static values on some known
-    field, then you might prefer to use
-    :attr:`known_values  <lino.core.actors.Actor.known_values>`
-    instead because this will add automatic behaviour.
+    If your filter consists of simple static values on some known field, then
+    you might prefer to use :attr:`known_values
+    <lino.core.actors.Actor.known_values>` instead because this will add
+    automatic behaviour.
 
     One advantage of :attr:`filter` over
     :attr:`known_values  <lino.core.actors.Actor.known_values>`
     is that this can use the full range of Django's `field lookup methods
-    <https://docs.djangoproject.com/en/5.0/topics/db/queries/#field-lookups>`_
+    <https://docs.djangoproject.com/en/5.2/topics/db/queries/#field-lookups>`_
 
     """
 
     exclude = None
-    """If specified, this must be dict which will be passed to Django's
-    `exclude
-    <https://docs.djangoproject.com/en/5.0/ref/models/querysets/#exclude>`__
-    method on the queryset.
+    """
+    If specified, this must be a :class:`django.db.models.Q` object that will be
+    passed to Django's `exclude
+    <https://docs.djangoproject.com/en/5.2/ref/models/querysets/#exclude>`__
+    method.
 
+    This is the logical opposite of :attr:`filter`.
     """
 
     extra = None
@@ -467,10 +481,12 @@ method in order to sort the rows of the queryset.
     @classmethod
     def class_init(cls):
         super(AbstractTable, cls).class_init()
-        resolve_fields_list(cls, 'tablet_columns', set, {})
-        resolve_fields_list(cls, 'mobile_columns', set, {})
-        resolve_fields_list(cls, 'popin_columns', set, {})
+        resolve_fields_list(cls, "tablet_columns", set, {})
+        resolve_fields_list(cls, "mobile_columns", set, {})
+        resolve_fields_list(cls, "popin_columns", set, {})
         if cls.model is not None:
+            if not isinstance(cls.model, type):
+                raise Exception(f"{cls}.model is {repr(cls.model)}")
             if not issubclass(cls.model, models.Model):
                 if cls.model._lino_default_table is None:
                     cls.model._lino_default_table = cls
@@ -481,7 +497,7 @@ method in order to sort the rows of the queryset.
 
     @classmethod
     def spawn(cls, suffix, **kw):
-        kw['app_label'] = cls.app_label
+        kw["app_label"] = cls.app_label
         return type(cls.__name__ + str(suffix), (cls,), kw)
 
     @classmethod
@@ -505,34 +521,18 @@ method in order to sort the rows of the queryset.
         `dbtables.Table` overrides this.
         """
         try:
-            return ar.data_iterator[int(pk)-1]
+            return ar.data_iterator[int(pk) - 1]
         except (ValueError, IndexError):
             return None
 
     @classmethod
     def get_default_action(cls):
-        return actions.ShowTable()
-
-    @classmethod
-    def row_as_paragraph(cls, ar, row):
-        """Return an HTML string that represents the given row as a single
-        paragraph.
-
-        See :ref:`dev.as_paragraph`.
-        """
-        return row.as_paragraph(ar)
-
-    @classmethod
-    def row_as_page(cls, ar, row, **kwargs):
-        """
-        Return an HTML string that represent the given row as a plain page.
-        """
-        return row.as_page(ar, **kwargs)
+        return actions.SHOW_TABLE
 
     @classmethod
     def get_actor_editable(self):
         if self._editable is None:
-            return (self.get_data_rows is None)
+            return self.get_data_rows is None
         return self._editable
 
     @classmethod
@@ -542,7 +542,7 @@ method in order to sort the rows of the queryset.
     @classmethod
     def get_column_names(self, ar):
         """Dynamic tables can subclass this method and return a value for
-        :attr:`column_names` which depends on the request.
+        :attr:`column_names` that depends on the request.
 
         """
         # if settings.SITE.mobile_view:
@@ -559,27 +559,25 @@ method in order to sort the rows of the queryset.
     def wildcard_data_elems(self):
         for cc in self.virtual_fields.values():
             yield cc
-        #~ return []
+        # ~ return []
 
     @classmethod
     def get_filter_kw(self, ar, **kw):
         """
         Return a dict with the "master keywords" for this table
-        and a given `master_instance`.
+        and the given action request ``ar``.
 
-        For example, if you have two models :class:`Book` and
-        :class:`Author`, and a foreign key :attr:`Book.author` which
-        points to the author of the book, and a table `BooksByAuthor`
-        having `master_key` set to ``'author'``, then `get_filter_kw`
-        would return a dict `{'author': <PK>}` where PK is the primary
-        key of the action request's :attr:`master_instance
+        For example, if you have two models :class:`Book` and :class:`Author`,
+        and a foreign key :attr:`Book.author`, which points to the author of the
+        book, and a table `BooksByAuthor` having `master_key` set to
+        ``'author'``, then :meth:`BooksByAuthor.get_filter_kw` would return a
+        dict `{'author': <PK>}` where `<PK>` is the primary key of the action
+        request's :attr:`master_instance
         <lino.core.requests.BaseRequest.master_instance>`.
 
-        Another example is
-        :class:`lino_xl.lib.tickets.EntriesBySession`, where blog
-        entries are not directly linked to a session, but in the
-        detail of a session we want to display a table of related blog
-        entries.
+        Another example is :class:`lino_xl.lib.tickets.EntriesBySession`, where
+        blog entries are not directly linked to a session, but in the detail of
+        a session we want to display a table of related blog entries.
 
         :class:`lino_xl.lib.households.SiblingsByPerson` Household
         members are not directly linked to a Person, but usually a
@@ -587,6 +585,7 @@ method in order to sort the rows of the queryset.
         of a Person we want to display the members of that household.
         """
         from lino.core.gfks import gfk2lookup, GenericForeignKey
+
         master_instance = ar.master_instance
         if self.master is None:
             pass
@@ -599,20 +598,24 @@ method in order to sort the rows of the queryset.
         elif self.master_field is not None:
             if master_instance is None:
                 if not self.master_field.null:
-                    #~ logger.info('20120519 %s.get_filter_kw()--> None',self)
+                    # ~ logger.info('20120519 %s.get_filter_kw()--> None',self)
                     return  # cannot add rows to this table
             else:
-                master_instance = master_instance.get_typed_instance(
-                    self.master)
+                # logger.info("20240414 master_instance is %s", master_instance)
+                # master_instance = master_instance.get_typed_instance(self.master)
                 if not isinstance(master_instance, self.master):
                     # e.g. a ByUser table descendant called by AnonymousUser
-                    msg = "%r is not a %s (%s.master_key = '%s')" % (
-                        master_instance.__class__,
-                        self.master, self,
-                        self.master_key)
+                    msg = "20240731 %r is not a %s (%s.master_key = '%s')" % (
+                        master_instance,
+                        self.master,
+                        self,
+                        self.master_key,
+                    )
+                    # logger.exception(msg)
                     logger.warning(msg)
                     # raise Exception(msg)
                     # raise PermissionDenied(msg)
+                    # raise BadRequest(msg)
                     # master_instance = None
                     return  # cannot add rows to this table
             kw[self.master_field.name] = master_instance
@@ -621,33 +624,10 @@ method in order to sort the rows of the queryset.
         #     raise Exception(msg)
         return kw
 
-    #~ @classmethod
-    #~ def request(cls,ui=None,request=None,action=None,**kw):
-        #~ self = cls
-        #~ if action is None:
-            #~ action = self.default_action
-        #~ return TableRequest(ui,self,request,action,**kw)
-
     # @fields.displayfield(_("Details"))
     # def detail_pointer(cls, obj, ar):
     #     # print("20181230 detail_pointer() {}".format(cls))
-    #     return obj.obj2href(ar)
-
-    @classmethod
-    def request(self, master_instance=None, **kw):
-        """Return a new :class:`TableRequest
-        <lino.core.tablerequest.TableRequest>` on this table.
-
-        If this is a slave table, the :attr:`master_instance
-        <lino.core.tablerequest.TableRequest.master_instance>` can be
-        specified as optional first positional argument.
-
-        """
-        kw.update(actor=self)
-        if master_instance is not None:
-            kw.update(master_instance=master_instance)
-        kw.setdefault('action', self.default_action)
-        return TableRequest(**kw)
+    #     return obj.as_summary_item(ar)
 
     @classmethod
     def run_action_from_console(self, pk=None, an=None):
@@ -656,27 +636,23 @@ method in order to sort the rows of the queryset.
         To be combined with the `show` management command.
         """
         settings.SITE.startup()
-        ba = None
-        if pk is not None:
-            if an is None:
-                # an = self.default_elem_action_name
-                ba = self.detail_action
-        elif an is None:
-            an = self.default_list_action_name
-        if ba is None:
-            ba = self.get_action_by_name(an)
-        #~ print ba
-        if pk is None:
-            ar = self.request(action=ba)
+        # 20240404 stopped using default_list_action_name
+        if an is None:
+            ba = self.default_action if pk is None else self.detail_action
         else:
-            ar = self.request(action=ba, selected_pks=[pk])
+            ba = self.get_action_by_name(an)
+        # ~ print ba
+        if pk is None:
+            ar = self.create_request(action=ba)
+        else:
+            ar = self.create_request(action=ba, selected_pks=[pk])
 
         ba.action.run_from_ui(ar)
         kw = ar.response
-        msg = kw.get('message')
+        msg = kw.get("message")
         if msg:
             print(msg)
-        url = kw.get('open_url') or kw.get('open_webdav_url')
+        url = kw.get("open_url") or kw.get("open_webdav_url")
         if url:
             os.startfile(url)
 
@@ -699,6 +675,7 @@ class VirtualTable(AbstractTable):
     Subclasses must define a :meth:`get_data_rows` method.
 
     """
+
     abstract = True
 
 
@@ -707,9 +684,10 @@ class VentilatedColumns(VirtualTable):
     A mixin for tables that have a series of automatically generated
     columns.  TODO: rename this to DynamicColumns.
     """
-    ventilated_column_suffix = ':5'
 
-    column_names_template = ''
+    ventilated_column_suffix = ":5"
+
+    column_names_template = ""
     """
     The template to use for :attr:`column_names`.   It should contain a string
     ``{vcolumns}``, which will be replaced by a space-separated list of the column
@@ -723,26 +701,26 @@ class VentilatedColumns(VirtualTable):
         # if not "{vcolumns}" in self.column_names_template:
         #     return
         # self.column_names = 'description '
-        names = ''
+        names = ""
         try:
             for i, vf in enumerate(self.get_ventilated_columns()):
-                self.add_virtual_field('vc' + str(i), vf)
-                names += ' ' + vf.name + self.ventilated_column_suffix
+                self.add_virtual_field("vc" + str(i), vf)
+                names += " " + vf.name + self.ventilated_column_suffix
         except (OperationalError, ProgrammingError) as e:
             # Error can differ depending on the database engine.
             logger.debug("Failed to load ventilated columns : %s", e)
         self.column_names = self.column_names_template.format(vcolumns=names)
 
-        #~ logger.info("20131114 setup_columns() --> %s",self.column_names)
+        # ~ logger.info("20131114 setup_columns() --> %s",self.column_names)
 
     @classmethod
     def get_ventilated_columns(self):
         return []
 
-class VentilatingTable(VentilatedColumns):
 
+class VentilatingTable(VentilatedColumns):
     abstract = True
-    column_names_template = 'description {vcolumns}'
+    column_names_template = "description {vcolumns}"
 
     @fields.virtualfield(models.CharField(_("Description"), max_length=30))
     def description(self, obj, ar):
@@ -764,8 +742,9 @@ class ButtonsTable(VirtualTable):
     `lino_welfare.modlib.reception.models.FindDateByClientTable`.
 
     """
+
     abstract = True
-    column_names = 'button'
+    column_names = "button"
     auto_fit_column_widths = True
     window_size = (60, 20)
     hide_top_toolbar = True
