@@ -1,91 +1,114 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2010-2023 Rumma & Ko Ltd
+# Copyright 2010-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 A collection of utilities which require Django settings to be
 importable.
 """
 
+from lino.utils import IncompleteDate
 import copy
 import sys
 import datetime
 # import yaml
-
-from django.utils.html import mark_safe
+from importlib import import_module
+from django.utils.html import format_html, mark_safe, SafeString
+from django.utils.functional import Promise
 from django.db import models
 from django.db.models import Q
 from django.core.exceptions import FieldDoesNotExist
-from django.utils.functional import lazy
-from importlib import import_module
+# from django.utils.functional import lazy
+from django.core.validators import validate_email, ValidationError, URLValidator
+from django.apps import apps
 from django.utils.translation import gettext as _
 from django.conf import settings
 from django.core import exceptions
 from django.http import QueryDict
 
-from etgen.html import E
+from lino.utils.html import E, assert_safe, tostring
+from lino.utils import capture_output
+from lino.utils.ranges import isrange
 
-from django.core.validators import (
-    validate_email, ValidationError, URLValidator)
+from .exceptions import ChangedAPI
 
-from django.apps import apps
 get_models = apps.get_models
 
-from lino.utils import IncompleteDate
-from .exceptions import ChangedAPI
 
 validate_url = URLValidator()
 
+DEVSERVER_COMMANDS = {
+    "runserver", "testserver", "test", "demotest", "makescreenshots", "shell"}
+
+
 def djangoname(o):
-    return o.__module__.split('.')[-2] + '.' + o.__name__
+    return o.__module__.split(".")[-2] + "." + o.__name__
+
 
 # def comma():
 #     return ', '
 
+more_text = mark_safe("...")
 
-def qs2summary(ar, objects, separator=", ", max_items=5, **kw):
+
+def qs2summary(ar, objects, separator=", ", max_items=5, wraptpl="<p>{}</p>", **kw):
     """Render a collection of objects as a single paragraph.
 
     :param separator: separator to use between objects.
+
     :param max_items: don't include more than the specified number of items.
 
+    :param wraptpl: the string template to use as outer paragraph tag. If this
+                    is `None`, the returned string won't have a wrapper tag.
+
     """
-    elems = ""
+    elems = mark_safe("")
+    separator = mark_safe(separator)
     n = 0
     for i in objects:
         if n:
             elems += separator
         n += 1
-        elems += ar.row_as_summary(i, **kw)
+        # if ar is None:
+        #     elems += i.as_summary_row(None, **kw)
+        # else:
+        # s = ar.row_as_summary(i, **kw)
+        # assert_safe(s)  # temporary 20240506
+        # elems += s
+        e = ar.row_as_summary(i, **kw)
+        # print("20240613", repr(e))
+        elems += tostring(e)
         if n >= max_items:
-            elems += separator + '...'
+            elems += separator + more_text
             break
-    # return mark_safe("<p>{}</p>".format(elems))
-    return mark_safe(elems)
-
+    # assert isinstance(elems, SafeString)  # temporary 20240506
+    # assert not "<lt;" in elems
+    if wraptpl is None:
+        return elems
+    return format_html(wraptpl, elems)
 
 
 def getrqdata(request):
     """Return the request data.
 
     Unlike the now defunct `REQUEST
-    <https://docs.djangoproject.com/en/5.0/ref/request-response/#django.http.HttpRequest.REQUEST>`_
+    <https://docs.djangoproject.com/en/5.2/ref/request-response/#django.http.HttpRequest.REQUEST>`_
     attribute, this inspects the request's `method` in order to decide
     what to return.
 
     """
-    if request.method in ('PUT', 'DELETE'):
+    if request.method in ("PUT", "DELETE"):
         return QueryDict(request.body)
         # note that `body` was named `raw_post_data` before Django 1.4
         # print 20130222, rqdata
     # rqdata = request.REQUEST
-    if request.method == 'HEAD':
+    if request.method == "HEAD":
         return request.GET
     return getattr(request, request.method)
 
 
 def is_valid_url(s):
     """Returns `True` if the given string is a valid URL.  This calls
-Django's `URLValidator()`, but does not raise an exception.
+    Django's `URLValidator()`, but does not raise an exception.
 
     """
     try:
@@ -97,7 +120,7 @@ Django's `URLValidator()`, but does not raise an exception.
 
 def is_valid_email(s):
     """Returns `True` if the given string is a valid email.  This calls
-Django's `validate_email()`, but does not raise an exception.
+    Django's `validate_email()`, but does not raise an exception.
 
     """
     try:
@@ -122,7 +145,9 @@ def is_devserver():
     - pytest removes the first item from sys.argv when running doctests
 
     """
-    #~ print 20130315, sys.argv[1]
+    # ~ print 20130315, sys.argv[1]
+    if settings.DEBUG:
+        return True  # doctest under pytest
     if sys.argv[0].startswith("-"):
         return True  # doctest under pytest
     if len(sys.argv) <= 1:
@@ -132,9 +157,9 @@ def is_devserver():
     # if sys.argv[0].endswith("doctest.py") or sys.argv[0].endswith("doctest_utf8.py"):
     if sys.argv[0].endswith("doctest.py") or sys.argv[0].endswith("pytest"):
         return True
-    if sys.argv[1] in ('runserver', 'testserver', 'test', "makescreenshots", 'shell'):
+    if sys.argv[1] in DEVSERVER_COMMANDS:
         return True
-    # print(sys.argv[1])
+    # print(sys.argv)
     return False
 
 
@@ -143,7 +168,7 @@ def is_logserver():
         return False  # doctest under pytest
     if len(sys.argv) <= 1:
         return False
-    if sys.argv[1] in ('lino_runworker', 'linod'):
+    if sys.argv[1] in ("lino_runworker", "linod"):
         return True
     return False
 
@@ -157,7 +182,7 @@ def format_request(request):
 
     """
     s = "{0} {1}".format(request.method, request.path)
-    qs = request.META.get('QUERY_STRING')
+    qs = request.META.get("QUERY_STRING")
     if qs:
         s += "?" + qs
     # Exception: You cannot access body after reading from request's
@@ -174,11 +199,11 @@ def format_request(request):
 
 
 def model_class_path(model):
-    return model.__module__ + '.' + model.__name__
+    return model.__module__ + "." + model.__name__
 
-def full_model_name(model, sep='.'):
-    """Returns the "full name" of the given model, e.g. "contacts.Person" etc.
-    """
+
+def full_model_name(model, sep="."):
+    """Returns the "full name" of the given model, e.g. "contacts.Person" etc."""
     return model._meta.app_label + sep + model._meta.object_name
 
 
@@ -206,28 +231,25 @@ def obj2str(i, force_detailed=False):
         force_detailed = True
     if not force_detailed:
         if i.pk is None:
-            return '(Unsaved %s instance)' % (i.__class__.__name__)
+            return "(Unsaved %s instance)" % (i.__class__.__name__)
         try:
-            return u"%s #%s (%s)" % (
-                i.__class__.__name__, str(i.pk),
-                repr(str(i)))
+            return "%s #%s (%s)" % (i.__class__.__name__, str(i.pk), repr(str(i)))
         except Exception as e:
-        #~ except TypeError,e:
-            return "Unprintable %s(pk=%r,error=%r" % (
-                i.__class__.__name__, i.pk, e)
-            #~ return unicode(e)
-    #~ names = [fld.name for (fld,model) in i._meta.get_fields_with_model()]
-    #~ s = ','.join(["%s=%r" % (n, getattr(i,n)) for n in names])
+            # ~ except TypeError,e:
+            return "Unprintable %s(pk=%r,error=%r" % (i.__class__.__name__, i.pk, e)
+            # ~ return unicode(e)
+    # ~ names = [fld.name for (fld,model) in i._meta.get_fields_with_model()]
+    # ~ s = ','.join(["%s=%r" % (n, getattr(i,n)) for n in names])
     pairs = []
     fields_list = i._meta.concrete_fields
     for fld in fields_list:
-        #~ if fld.name == 'language':
-            #~ print 20120905, model, fld
+        # ~ if fld.name == 'language':
+        # ~ print 20120905, model, fld
         if isinstance(fld, models.ForeignKey):
             v = getattr(i, fld.attname, None)  # 20130709 Django 1.6b1
-            #~ v = getattr(i,fld.name+"_id")
-            #~ if getattr(i,fld.name+"_id") is not None:
-                #~ v = getattr(i,fld.name)
+            # ~ v = getattr(i,fld.name+"_id")
+            # ~ if getattr(i,fld.name+"_id") is not None:
+            # ~ v = getattr(i,fld.name)
         else:
             try:
                 v = getattr(i, fld.name, None)  # 20130709 Django 1.6b1
@@ -235,12 +257,12 @@ def obj2str(i, force_detailed=False):
                 v = str(e)
         if v:
             pairs.append("%s=%s" % (fld.name, obj2str(v)))
-    s = ','.join(pairs)
-    #~ s = ','.join(["%s=%s" % (n, obj2str(getattr(i,n))) for n in names])
-    #~ print i, i._meta.get_all_field_names()
-    #~ s = ','.join(["%s=%r" % (n, getattr(i,n)) for n in i._meta.get_all_field_names()])
+    s = ",".join(pairs)
+    # ~ s = ','.join(["%s=%s" % (n, obj2str(getattr(i,n))) for n in names])
+    # ~ print i, i._meta.get_all_field_names()
+    # ~ s = ','.join(["%s=%r" % (n, getattr(i,n)) for n in i._meta.get_all_field_names()])
     return "%s(%s)" % (i.__class__.__name__, s)
-    #~ return "%s(%s)" % (i.__class__,s)
+    # ~ return "%s(%s)" % (i.__class__,s)
 
 
 def sorted_models_list():
@@ -289,27 +311,50 @@ def range_filter(value, f1, f2):
     and `f2` encompass the given value `value`.
 
     """
-    q1 = Q(**{f1 + '__isnull': True}) | Q(**{f1 + '__lte': value})
-    q2 = Q(**{f2 + '__isnull': True}) | Q(**{f2 + '__gte': value})
+    q1 = Q(**{f1 + "__isnull": True}) | Q(**{f1 + "__lte": value})
+    q2 = Q(**{f2 + "__isnull": True}) | Q(**{f2 + "__gte": value})
     return Q(q1, q2)
 
 
 def inrange_filter(fld, rng, **kw):
     """Assuming a database model with a field named `fld`, return a Q
-    object to select those rows whose `fld` value is not null and
-    within the given range `rng`.  `rng` must be a tuple or list with
-    two items.
+    object to select the rows having value for `fld` within the given range `rng`.
+    `rng` must be a tuple or list with two items.
+    """
+
+    # assert rng[0] <= rng[1]
+    kw[fld + "__isnull"] = False
+    if rng[0] is not None:
+        kw[fld + "__gte"] = rng[0]
+    if rng[1] is not None:
+        kw[fld + "__lte"] = rng[1]
+    return Q(**kw)
+
+
+def overlap_range_filter(sv, ev, f1, f2, **kw):
+    """
+
+    Return a Q object to select all objects having fields `f1` and `f2` define a
+    range that overlaps with the range specified by `sv` (start value) and `ev`
+    (end value).
+
+    For example, when I specify filter parameters `Date from` 2024-02-21 and
+    `until` 2024-03-12 in a :class:`lino.modlib.periods.StoredPeriods` table, I
+    want both February and March. Tested examples see
+    :ref:`dg.plugins.periods.period_filter`.
 
     """
-    assert rng[0] <= rng[1]
-    kw[fld + '__isnull'] = False
-    kw[fld + '__gte'] = rng[0]
-    kw[fld + '__lte'] = rng[1]
-    return Q(**kw)
+    # if not isrange(rng[0], rng[1]):
+    #     raise ValueError(f"{rng} is not a valid range")
+    if not ev:
+        ev = sv
+    return Q(**{f1+"__lte": ev, f2+"__gte": sv})
 
 
 def babelkw(*args, **kw):
     return settings.SITE.babelkw(*args, **kw)
+
+
 babel_values = babelkw  # old alias for backwards compatibility
 
 
@@ -318,7 +363,6 @@ def babelattr(*args, **kw):
 
 
 class UnresolvedModel(object):
-
     """The object returned by :func:`resolve_model` if the specified model
     is not installed.
 
@@ -337,14 +381,13 @@ class UnresolvedModel(object):
     def __init__(self, model_spec, app_label):
         self.model_spec = model_spec
         self.app_label = app_label
-        #~ print(self)
+        # ~ print(self)
 
     def __repr__(self):
-        return self.__class__.__name__ + '(%s,%s)' % (
-            self.model_spec, self.app_label)
+        return self.__class__.__name__ + "(%r, %s)" % (self.model_spec, self.app_label)
 
-    #~ def __getattr__(self,name):
-        #~ raise AttributeError("%s has no attribute %r" % (self,name))
+    # ~ def __getattr__(self,name):
+    # ~ raise AttributeError("%s has no attribute %r" % (self,name))
 
 
 def resolve_model(model_spec, app_label=None, strict=False):
@@ -372,7 +415,7 @@ def resolve_model(model_spec, app_label=None, strict=False):
     """
     # ~ models.get_apps() # trigger django.db.models.loading.cache._populate()
     if isinstance(model_spec, str):
-        if '.' in model_spec:
+        if "." in model_spec:
             app_label, model_name = model_spec.split(".")
         else:
             model_name = model_spec
@@ -383,7 +426,11 @@ def resolve_model(model_spec, app_label=None, strict=False):
             # settings.SITE.logger.info("20181230 resolve %s --> %r, %r",
             #                           model_spec, app, model)
         else:
+            # 20241112 Helped to explore #5797 (Could not resolve target
+            # 'uploads.UploadType' of ForeignKey 'type' in <class
+            # 'lino.modlib.uploads.models.Upload'>)
             from django.apps import apps
+
             try:
                 model = apps.get_model(app_label, model_name)
             except LookupError:
@@ -395,6 +442,7 @@ def resolve_model(model_spec, app_label=None, strict=False):
         if strict:
             if False:
                 from django.db.models import loading
+
                 print((20130219, settings.INSTALLED_APPS))
                 print([full_model_name(m) for m in get_models()])
                 if len(loading.cache.postponed) > 0:
@@ -404,9 +452,15 @@ def resolve_model(model_spec, app_label=None, strict=False):
                 raise Exception(strict % model_spec)
             raise ImportError(
                 "resolve_model(%r,app_label=%r) found %r "
-                "(settings %s, INSTALLED_APPS=%s)" % (
-                    model_spec, app_label, model,
-                    settings.SETTINGS_MODULE, settings.INSTALLED_APPS))
+                "(settings %s, INSTALLED_APPS=%s)"
+                % (
+                    model_spec,
+                    app_label,
+                    model,
+                    settings.SETTINGS_MODULE,
+                    settings.INSTALLED_APPS,
+                )
+            )
         return UnresolvedModel(model_spec, app_label)
     return model
 
@@ -424,29 +478,30 @@ def resolve_app(app_label, strict=False):
 
     For example, instead of writing::
 
-        from lino.modlib.sales import models as sales
+        from lino_xl.lib.trading import models as trading
 
     it is recommended to write::
 
-        sales = dd.resolve_app('sales')
+        trading = dd.resolve_app('trading')
 
     because it makes your code usable (1) in applications that don't
-    have the 'sales' module installed and (2) in applications who have
-    another implementation of the `sales` module
-    (e.g. :mod:`lino.modlib.auto.sales`)
+    have the 'trading' module installed and (2) in applications who have
+    another implementation of the `trading` module
+    (e.g. :mod:`lino.modlib.auto.trading`)
 
     """
-    #~ app_label = app_label
+    # ~ app_label = app_label
     for app_name in settings.INSTALLED_APPS:
-        if app_name == app_label or app_name.endswith('.' + app_label):
-            return import_module('.models', app_name)
+        if app_name == app_label or app_name.endswith("." + app_label):
+            return import_module(".models", app_name)
     try:
-        return import_module('lino.modlib.%s.dummy' % app_label)
+        return import_module("lino.modlib.%s.dummy" % app_label)
     except ImportError:
         if strict:
-            #~ raise
-            raise ImportError("No app_label %r in %s" %
-                              (app_label, settings.INSTALLED_APPS))
+            # ~ raise
+            raise ImportError(
+                "No app_label %r in %s" % (app_label, settings.INSTALLED_APPS)
+            )
 
 
 def require_app_models(app_label):
@@ -471,7 +526,6 @@ def get_field(model, name):
 
 
 class UnresolvedField(object):
-
     """
     Returned by :func:`resolve_field` if the specified field doesn't exist.
     This case happens when sphinx autodoc tries to import a module.
@@ -488,15 +542,15 @@ def resolve_field(name, app_label=None):
     should be either `model.field` or `app_label.model.field`.
 
     """
-    l = name.split('.')
+    l = name.split(".")
     if len(l) == 3:
         app_label = l[0]
         del l[0]
     if len(l) == 2:
         model = apps.get_model(app_label, l[0])
         if model is None:
-            raise FieldDoesNotExist("No model named '%s.%s'" %
-                                    (app_label, l[0]))
+            raise FieldDoesNotExist(
+                "No model named '%s.%s'" % (app_label, l[0]))
         return model._meta.get_field(l[1])
         # fld, remote_model, direct, m2m = model._meta.get_field_by_name(l[1])
         # assert remote_model is None or issubclass(model, remote_model), \
@@ -506,6 +560,7 @@ def resolve_field(name, app_label=None):
     raise FieldDoesNotExist(name)
     # return UnresolvedField(name)
 
+
 def navinfo(qs, elem, limit=None):
     """Return a dict with navigation information for the given model
     instance `elem` within the given queryset.
@@ -534,20 +589,20 @@ def navinfo(qs, elem, limit=None):
     page_num = None
     offset = None
 
-    #~ LEN = ar.get_total_count()
+    # ~ LEN = ar.get_total_count()
     if isinstance(qs, (list, tuple)):
         num = len(qs)
         id_list = [obj.pk for obj in qs]
-        #~ logger.info('20130714')
+        # ~ logger.info('20130714')
     else:
         num = qs.count()
         # this algorithm is clearly quicker on queries with a few thousand rows
-        id_list = list(qs.values_list('pk', flat=True))
+        id_list = list(qs.values_list("pk", flat=True))
     if num > 0:
         # Uncommented the following assert because it failed in certain circumstances
         # (see `/blog/2011/1220`)
         # assert len(id_list) == ar.total_count, \
-            # "len(id_list) is %d while ar.total_count is %d" % (len(id_list),ar.total_count)
+        # "len(id_list) is %d while ar.total_count is %d" % (len(id_list),ar.total_count)
         # print 20111220, id_list
         try:
             i = id_list.index(elem.pk)
@@ -561,42 +616,32 @@ def navinfo(qs, elem, limit=None):
                 page_num = i // limit
                 offset = limit * page_num
             if i > 0:
-                prev = id_list[i-1]
+                prev = id_list[i - 1]
                 if limit:
-                    prev_page = id_list[max(0, offset-limit+1)]
+                    prev_page = id_list[max(0, offset - limit + 1)]
             if i < len(id_list) - 1:
                 next = id_list[i + 1]
                 if limit:
-                    next_page = id_list[min(num-1, offset+limit+2)]
+                    next_page = id_list[min(num - 1, offset + limit + 2)]
             message = _("Row %(rowid)d of %(rowcount)d") % dict(
-                rowid=recno, rowcount=num)
+                rowid=recno, rowcount=num
+            )
     if message is None:
         message = _("No navigation")
     return dict(
-        first=first, prev=prev, next=next, last=last, recno=recno,
-        message=message, prev_page=prev_page, next_page=next_page,
-        offset=offset)
+        first=first,
+        prev=prev,
+        next=next,
+        last=last,
+        recno=recno,
+        message=message,
+        prev_page=prev_page,
+        next_page=next_page,
+        offset=offset,
+    )
 
 
-# class Handle(object):
-#     """Base class for :class:`lino.core.tables.TableHandle`,
-#     :class:`lino.core.frames.FrameHandle` etc.
-
-#     The "handle" of an actor is responsible for expanding layouts into
-#     sets of (renderer-specific) widgets (called "elements"). This
-#     operation is done once per actor per renderer.
-
-#     """
-#     # def __init__(self):
-#     #     self.ui = settings.SITE.kernel.default_ui
-
-#     def setup(self, ar):
-#         settings.SITE.kernel.setup_handle(self, ar)
-#         # self.ui.setup_handle(self, ar)
-
-
-
-class Parametrizable(object):
+class Parametrizable:
     """
     Base class for both Actors and Actions.  See :doc:`/dev/parameters`.
 
@@ -619,10 +664,11 @@ class Parametrizable(object):
     parameters = None
     params_layout = None
     params_panel_hidden = True
-    params_panel_pos = "top"  # allowed values "top", "bottom", "left" and "right"
+    params_panel_pos = "bottom"  # allowed values "top", "bottom", "left" and "right"
     use_detail_param_panel = False
 
     _params_layout_class = NotImplementedError
+    _field_actions = dict()
 
     def get_window_layout(self, actor):
         return self.params_layout
@@ -640,6 +686,7 @@ class Parametrizable(object):
     def hide_editing(cls, user_type):
         return False
 
+
 class ParameterPanel(object):
     """
     A utility class for defining reusable definitions for
@@ -647,8 +694,9 @@ class ParameterPanel(object):
 
     Subclassed e.g. by
     :class:`lino.mixins.periods.ObservedDateRange`.
-    :class:`lino_xl.lib.ledger.AccountingPeriodRange`.
+    :class:`lino_xl.lib.accounting.PeriodRangeParameters`.
     """
+
     def __init__(self, **kw):
         self.fields = kw
 
@@ -716,7 +764,8 @@ class PseudoRequest(object):
             cw.send_update(REQUEST)
 
     """
-    method = 'GET'
+
+    method = "GET"
     subst_user = None
     requesting_panel = None
     success = None
@@ -724,15 +773,17 @@ class PseudoRequest(object):
     def __init__(self, username):
         self.username = username
         self._user = None
-        self.GET = QueryDict('')
+        self.GET = QueryDict("")
 
     def get_user(self):
         if self._user is None:
             if settings.SITE.user_model is not None:
-                #~ print 20130222, self.username
+                # ~ print 20130222, self.username
                 self._user = settings.SITE.user_model.objects.get(
-                    username=self.username)
+                    username=self.username
+                )
         return self._user
+
     user = property(get_user)
 
 
@@ -741,54 +792,49 @@ def error2str(self, e):
     ValidationError specially.
     """
     if isinstance(e, list):
-        return ', '.join([self.error2str(v) for v in e])
+        return ", ".join([self.error2str(v) for v in e])
     if isinstance(e, exceptions.ValidationError):
-        md = getattr(e, 'message_dict', None)
+        md = getattr(e, "message_dict", None)
         if md is not None:
+
             def fieldlabel(name):
                 de = self.get_data_elem(name)
-                return str(getattr(de, 'verbose_name', name))
-            return '\n'.join([
-                "%s : %s" % (
-                    fieldlabel(k), self.error2str(v))
-                for k, v in md.items()])
-        return '\n'.join(e.messages)
+                return str(getattr(de, "verbose_name", name))
+
+            return "\n".join(
+                ["%s : %s" % (fieldlabel(k), self.error2str(v))
+                 for k, v in md.items()]
+            )
+        return "\n".join(e.messages)
     return str(e)
 
 
-def lazy_format(tpl, *args, **kwargs):
-    """See :ref:`book.specs.i18n`.
-
-    """
-    def f():
-        return tpl.format(*args, **kwargs)
-    return lazy(f, str)()
-
-
 def resolve_fields_list(model, k, collection_type=tuple, default=None):
-    qsf = getattr(model, k)
-    if qsf is None:
+    value = getattr(model, k)
+    if value is None:
         setattr(model, k, default)
-    elif qsf == default:
-        pass
-    elif isinstance(qsf, collection_type):
-        pass
-    elif isinstance(qsf, str):
+        return
+    elif value == default:
+        return
+    elif isinstance(value, collection_type):
+        return
+    if isinstance(value, str):
+        value = value.split()
+    if isinstance(value, (list, tuple)):
         lst = []
-        for n in qsf.split():
+        for n in value:
             f = model.get_data_elem(n)
             if f is None:
-                msg = "Invalid field {} in {} of {}"
-                msg = msg.format(n, k, model)
+                msg = f"Invalid field {n} in {k} of {model}"
                 raise Exception(msg)
             lst.append(f)
         setattr(model, k, collection_type(lst))
-            # fields.fields_list(model, model.quick_search_fields))
+        # fields.fields_list(model, model.quick_search_fields))
     else:
         raise ChangedAPI(
-            "{0}.{1} must be None or a string "
-            "of space-separated field names (not {2})".format(
-                model, k, qsf))
+            f"{model}.{k} must be None or a string "
+            f"of space-separated field names (not {value})")
+
 
 def class_dict_items(cl, exclude=None):
     if exclude is None:
@@ -801,8 +847,8 @@ def class_dict_items(cl, exclude=None):
         for i in class_dict_items(b, exclude):
             yield i
 
-def dbfield2params_field(db_field):
 
+def dbfield2params_field(db_field):
     """originally just for setting up actor parameters from get_simple_parameters()
     but also used in calview."""
 
@@ -813,6 +859,7 @@ def dbfield2params_field(db_field):
     fld.editable = True
     return fld
 
+
 def db2param(spec):
     """
     Return a copy of the specified :term:`database field` for usage as an
@@ -873,7 +920,7 @@ def traverse_ddh_fklist(model, ignore_mti_parents=True):
     #     return
     found = set()
     for base in model.mro():
-        ddh = getattr(base, '_lino_ddh', None)
+        ddh = getattr(base, "_lino_ddh", None)
         if ddh is not None:
             for k in ddh.fklist:
                 # k is a tuple (m, fk)
@@ -893,8 +940,12 @@ def traverse_ddh_fklist(model, ignore_mti_parents=True):
 def DelayedValue(ar, fieldname, obj):
     # return dict(delayed_value_url=settings.SITE.buildurl("values",
     #     str(ar.actor).replace(".","/"), str(obj.pk), fieldname, mk=ar.master_instance.pk))
-    return dict(delayed_value_url="values/{}/{}/{}".format(
-            str(ar.actor).replace(".","/"), obj.pk, fieldname))
+    return dict(
+        delayed_value_url="values/{}/{}/{}".format(
+            str(ar.actor).replace(".", "/"), obj.pk, fieldname
+        )
+    )
+
 
 # class DelayedValue:
 #     """
@@ -919,3 +970,291 @@ def DelayedValue(ar, fieldname, obj):
 #
 #     def __repr__(self):
 #         return "DelayedValue({})".format(self.url)
+
+
+class InstanceAction:
+    """
+    Volatile object that wraps a given action to be run on a given
+    model instance.
+
+    .. attribute:: bound_action
+
+        The bound action that will run.
+
+    .. attribute:: instance
+
+        The database object on which the action will run.
+
+    .. attribute:: owner
+
+
+    """
+
+    def __init__(self, action, actor, instance, owner):
+        # ~ print "Bar"
+        # ~ self.action = action
+        if actor is None:
+            actor = instance.get_default_table()
+        self.bound_action = actor.get_action_by_name(action.action_name)
+        if self.bound_action is None:
+            raise Exception("%s has not action %r" % (actor, action))
+            # Happened 20131020 from lino_xl.lib.beid.eid_info() :
+            # When `use_eid_jslib` was False, then
+            # `Action.attach_to_actor` returned False.
+        self.instance = instance
+        self.owner = owner
+
+    def __str__(self):
+        return "{0} on {1}".format(self.bound_action, obj2str(self.instance))
+
+    def run_from_code(self, ar, *args, **kw):
+        """
+        Probably to be deprecated.
+        Run this action on this instance in the given session, updating
+        the response of the session.  Returns the return value of the
+        action.
+        """
+        # raise Exception("20170129 is this still used?")
+        ar.selected_rows = [self.instance]
+        return self.bound_action.action.run_from_code(ar, *args, **kw)
+
+    def run_from_ui(self, ar, *args, **kwargs):
+        """
+        Run this action on this instance in the given session, updating
+        the response of the session.  Returns nothing.
+        """
+        # raise Exception("20170129 is this still used?")
+        # kw.update(selected_rows=[self.instance])
+        ar.selected_rows = [self.instance]
+        self.bound_action.action.run_from_ui(ar, *args, **kwargs)
+
+    def request_from(self, ses, **kwargs):
+        """
+        Create an action request on this instance action without running
+        the action.
+        """
+        kwargs.update(selected_rows=[self.instance])
+        kwargs.update(parent=ses)
+        ar = self.bound_action.create_request(**kwargs)
+        return ar
+
+    def run_from_session(self, ses, **kwargs):
+        """
+        Run this instance action in a child request of given session.
+
+        Additional arguments are forwarded to the action.
+        Returns the response of the child request.
+        Doesn't modify response of parent request.
+        """
+        ar = self.request_from(ses, **kwargs)
+        self.bound_action.action.run_from_code(ar)
+        return ar.response
+
+    def __call__(self, *args, **kwargs):
+        """
+        Run this instance action in an anonymous base request.
+
+        Additional arguments are forwarded to the action.
+        Returns the response of the base request.
+        """
+        if len(args) and isinstance(args[0], BaseRequest):
+            raise ChangedAPI("20181004")
+        ar = self.bound_action.create_request(
+            renderer=settings.SITE.kernel.text_renderer)
+        self.run_from_code(ar, *args, **kwargs)
+        return ar.response
+
+    def as_button_elem(self, ar, label=None, **kwargs):
+        return settings.SITE.kernel.row_action_button(
+            self.instance, ar, self.bound_action, label, **kwargs
+        )
+
+    def as_button(self, *args, **kwargs):
+        """Return a HTML chunk with a "button" which, when clicked, will
+        execute this action on this instance.  This is being used in
+        the :ref:`lino.tutorial.polls`.
+
+        """
+        return tostring(self.as_button_elem(*args, **kwargs))
+
+    def get_row_permission(self, ar):
+        state = self.bound_action.actor.get_row_state(self.instance)
+        # logger.info("20150202 ia.get_row_permission() %s using %s",
+        #             self, state)
+        return self.bound_action.get_row_permission(ar, self.instance, state)
+
+
+class VirtualRow(object):
+    def __init__(self, **kw):
+        self.update(**kw)
+
+    def update(self, **kw):
+        for k, v in list(kw.items()):
+            setattr(self, k, v)
+
+    def get_row_permission(self, ar, state, ba):
+        if ba.action.readonly:
+            return True
+        return False
+
+
+class PhantomRow(VirtualRow):
+    def __init__(self, request, **kw):
+        self._ar = request
+        VirtualRow.__init__(self, **kw)
+
+    def __str__(self):
+        return str(self._ar.get_action_title())
+
+
+def login(username=None, **kwargs):
+    """Return a basic :term:`action request` with the specified user signed in.
+    """
+    from lino.core.requests import BaseRequest  # avoid circular import
+    # settings.SITE.startup()
+    User = settings.SITE.user_model
+    if User and username:
+        try:
+            kwargs.update(user=User.objects.get(username=username))
+        except User.DoesNotExist:
+            raise User.DoesNotExist(f"'{username}' : no such user")
+
+    kwargs.setdefault("show_urls", False)
+    # import lino.core.urls  # hack: trigger ui instantiation
+    return BaseRequest(**kwargs)
+
+
+def show(*args, **kwargs):
+    """Print the specified data table to stdout."""
+    return login().show(*args, **kwargs)
+
+
+def shows(*args, **kwargs):
+    """Return the output of :func:`show`."""
+    return capture_output(show, *args, **kwargs)
+
+
+class Panel:
+    """
+    To be used when a panel cannot be expressed using a simple
+    template string because it requires one or more options. These
+    `options` parameters can be:
+
+    - label
+    - required_roles
+    - window_size
+    - label_align
+
+    Unlike a :class:`BaseLayout` it cannot have any child panels
+    and cannot become a tabbed panel.
+    """
+
+    def __init__(self, desc, label=None, **options):
+        # assert not 'required' in options
+        self.desc = desc
+        if label is not None:
+            options.update(label=label)
+        self.options = options
+
+    def replace(self, *args, **kw):
+        """
+        Calls the standard :meth:`string.replace`
+        method on this Panel's template.
+        """
+        self.desc = self.desc.replace(*args, **kw)
+
+
+def resolve_layout(cls, k, spec, layout_class, **options):
+    # k: just for naming the culprit in error messages
+    if isinstance(spec, Promise):
+        spec = str(spec)
+    if isinstance(spec, str):
+        if "\n" in spec or "." not in spec:
+            return layout_class(spec, cls, **options)
+        else:
+            layout_class = settings.SITE.models.resolve(spec)
+            if layout_class is None:
+                raise Exception(
+                    "Unresolved {} {!r} for {}".format(k, spec, cls))
+            return layout_class(None, cls, **options)
+    elif isinstance(spec, Panel):
+        options.update(spec.options)
+        return layout_class(spec.desc, cls, **options)
+    else:
+        if not isinstance(spec, layout_class):
+            if not isinstance(cls, type):
+                # cls is an action instance
+                cls = cls.__class__
+            msg = (
+                "{}.{}.{} must be a string, " "a Panel or an instance of {} (not {!r})"
+            )
+            raise Exception(
+                msg.format(cls.__module__, cls.__name__,
+                           k, layout_class.__name__, spec)
+            )
+        if spec._datasource is None:
+            spec.set_datasource(cls)
+            return spec
+        elif not issubclass(cls, spec._datasource):
+            raise Exception(
+                "Cannot reuse %s instance (%s of %r) for %r"
+                % (spec.__class__, k, spec._datasource, cls)
+            )
+    return spec
+
+
+def install_layout(cls, k, layout_class, **options):
+    """
+    - `cls` is the actor (a class object)
+
+    - `k` is one of 'grid_layout', 'detail_layout', 'insert_layout',
+      'params_layout', 'card_layout'
+
+    - `layout_class`
+
+    """
+    # if str(cls) == 'courses.Pupils':
+    #     print("20160329 install_layout", k, layout_class)
+    dl = cls.__dict__.get(k, None)
+    if dl is None:  # and not cls._class_init_done:
+        dl = getattr(cls, k, None)
+    if dl is None:
+        return
+    setattr(cls, k, resolve_layout(cls, k, dl, layout_class, **options))
+
+
+def register_params(cls):
+    """`cls` is either an actor (a class object) or an action (an
+    instance).
+
+    """
+    if cls.parameters is not None:
+        for k, v in cls.parameters.items():
+            v.set_attributes_from_name(k)
+            v.table = cls
+            # v.model = cls  # 20181023 experimentally
+
+        if cls.params_layout is None:
+            if cls._params_layout_class is None:
+                raise Exception(f"{cls} has no _params_layout_class")
+            cls.params_layout = cls._params_layout_class.join_str.join(
+                cls.parameters.keys()
+            )
+    if cls.params_layout is not None:
+        install_layout(cls, "params_layout", cls._params_layout_class)
+
+    # # e.g. accounting.ByJournal is just a mixin but provides a default value for its children
+    # elif cls.params_layout is not None:
+    #     raise Exception("{} has a params_layout but no parameters".format(cls))
+
+    # if isinstance(cls, type) and cls.__name__.endswith("Users"):
+    # # if isinstance(cls, type) and cls.model is not None and cls.model.__name__ == "User":
+    #     # if str(cls.model) != "users.User":
+    #     #     raise Exception("{} {}".format(cls, cls.model))
+    #     print("20200825 {}.register_params {} {}".format(
+    #         cls, cls.parameters, cls.params_layout))
+
+
+def make_params_layout_handle(self):
+    # `self` is either an Action instance or an Actor class object
+    return self.params_layout.get_layout_handle()
