@@ -1,4 +1,4 @@
-# Copyright 2010-2020 Rumma & Ko Ltd
+# Copyright 2010-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """Utility functions used by :mod:`lino.modlib.extjs.views`.
 
@@ -12,6 +12,8 @@ from django.core import exceptions
 from lino.core import constants
 from lino.core import actors
 from lino.utils.jsgen import py2js
+from lino.utils.html import mark_safe, assert_safe, format_html
+from lino.utils.html import Grouper
 from lino.api import _
 
 
@@ -19,7 +21,7 @@ def json_response_kw(**kw):
     return json_response(kw)
 
 
-def json_response(x, content_type='application/json', status=200):
+def json_response(x, content_type="application/json", status=200):
     if True:
         s = py2js(x)
     else:
@@ -40,9 +42,9 @@ def json_response(x, content_type='application/json', status=200):
     # See 20120209.
 
     return http.HttpResponse(s, content_type=content_type, status=status)
-    #~ return HttpResponse(s, content_type='text/html')
-    #~ return HttpResponse(s, content_type='application/json')
-    #~ return HttpResponse(s, content_type='text/json')
+    # ~ return HttpResponse(s, content_type='text/html')
+    # ~ return HttpResponse(s, content_type='application/json')
+    # ~ return HttpResponse(s, content_type='text/json')
 
 
 def requested_actor(app_label, actor):
@@ -63,7 +65,7 @@ def requested_actor(app_label, actor):
     if issubclass(cl, models.Model):
         return cl.get_default_table()
     if not issubclass(cl, actors.Actor):
-        #~ raise http.Http404("%r is not an actor" % cl)
+        # ~ raise http.Http404("%r is not an actor" % cl)
         raise http.Http404("%r is not an actor" % cl)
     return cl
 
@@ -76,7 +78,8 @@ def action_request(app_label, actor, request, rqdata, is_list, **kw):
     if not action_name:
         # print("20210215", rqdata)
         if is_list:
-            action_name = rpt.default_list_action_name
+            a = rpt.default_action
+            # 20240404 action_name = rpt.default_list_action_name
         else:
             a = rpt.detail_action
             # action_name = rpt.default_elem_action_name
@@ -84,20 +87,22 @@ def action_request(app_label, actor, request, rqdata, is_list, **kw):
         a = rpt.get_url_action(action_name)
         if a is None:
             raise http.Http404(
-                "%s has no url action %r (possible values are %s)" % (
-                    rpt, action_name, rpt.get_url_action_names()))
+                "%s has no url action %r (possible values are %s)"
+                % (rpt, action_name, rpt.get_url_action_names())
+            )
     user = request.subst_user or request.user
     if True:  # False:  # 20130829
         if not a.get_view_permission(user.user_type):
             # print("20200110 {} {} {}".format(rpt, a.required, user.user_type.role.__class__))
             # raise Exception("20230124")
             raise exceptions.PermissionDenied(
-                "As %s you have no view permission for this action."
-                % user.user_type)
-                # The text of an Exception may not be
-                # internationalized because some error handling code
-                # may want to write it to a plain ascii stream.
-    ar = rpt.request(request=request, action=a, rqdata=rqdata, **kw)
+                "As %s you have no view permission for this action." % user.user_type
+            )
+            # The text of an Exception may not be
+            # internationalized because some error handling code
+            # may want to write it to a plain ascii stream.
+    kw.update(renderer=settings.SITE.kernel.default_renderer)
+    ar = rpt.create_request(request=request, action=a, rqdata=rqdata, **kw)
     # print("20210403b", a.action.__class__, rqdata)
     return ar
 
@@ -126,7 +131,11 @@ def choices_response(actor, request, qs, row2dict, emptyValue, field=None):
     wt = request.GET.get(constants.URL_PARAM_WINDOW_TYPE, None)
 
     if isinstance(qs, models.QuerySet):
-        qs = qs.filter(qs.model.quick_search_filter(quick_search)) if quick_search else qs
+        qs = (
+            qs.filter(qs.model.quick_search_filter(quick_search))
+            if quick_search
+            else qs
+        )
         count = qs.count()
 
         if offset:
@@ -135,7 +144,7 @@ def choices_response(actor, request, qs, row2dict, emptyValue, field=None):
 
         if limit:
             # ~ kw.update(limit=int(limit))
-            qs = qs[:int(limit)]
+            qs = qs[: int(limit)]
 
         rows = [row2dict(row, {}) for row in qs]
 
@@ -144,23 +153,30 @@ def choices_response(actor, request, qs, row2dict, emptyValue, field=None):
         if quick_search:
             txt = quick_search.lower()
 
-            rows = [row for row in rows
-                    if txt in row[constants.CHOICES_TEXT_FIELD].lower()]
+            rows = [
+                row for row in rows if txt in row[constants.CHOICES_TEXT_FIELD].lower()
+            ]
         count = len(rows)
         rows = rows[int(offset):] if offset else rows
-        rows = rows[:int(limit)] if limit else rows
+        rows = rows[: int(limit)] if limit else rows
 
     if wt == constants.WINDOW_TYPE_PARAMS and field and field.blank:
-        rows.insert(0, {
-            # constants.CHOICES_TEXT_FIELD: actor.get_blank_filter_text(),
-            constants.CHOICES_TEXT_FIELD: _("Blank"),
-            constants.CHOICES_VALUE_FIELD: constants.CHOICES_BLANK_FILTER_VALUE
-        })
-        rows.insert(1, {
-            # constants.CHOICES_TEXT_FIELD: actor.get_not_blank_filter_text(),
-            constants.CHOICES_TEXT_FIELD: _("Not Blank"),
-            constants.CHOICES_VALUE_FIELD: constants.CHOICES_NOT_BLANK_FILTER_VALUE
-        })
+        rows.insert(
+            0,
+            {
+                # constants.CHOICES_TEXT_FIELD: actor.get_blank_filter_text(),
+                constants.CHOICES_TEXT_FIELD: _("Blank"),
+                constants.CHOICES_VALUE_FIELD: constants.CHOICES_BLANK_FILTER_VALUE,
+            },
+        )
+        rows.insert(
+            1,
+            {
+                # constants.CHOICES_TEXT_FIELD: actor.get_not_blank_filter_text(),
+                constants.CHOICES_TEXT_FIELD: _("Not Blank"),
+                constants.CHOICES_VALUE_FIELD: constants.CHOICES_NOT_BLANK_FILTER_VALUE,
+            },
+        )
 
     # Add None choice
     if emptyValue is not None and not quick_search:
@@ -172,3 +188,49 @@ def choices_response(actor, request, qs, row2dict, emptyValue, field=None):
     return json_response_kw(count=count, rows=rows)
     # ~ return json_response_kw(count=len(rows),rows=rows)
     # ~ return json_response_kw(count=len(rows),rows=rows,title=_('Choices for %s') % fldname)
+
+
+def ar2html(ar, display_mode):
+    if display_mode == constants.DISPLAY_MODE_STORY:
+        html_text = mark_safe("")
+        for obj in ar.sliced_data_iterator:
+            chunk = obj.as_story_item(ar)
+            # assert_safe(chunk)
+            html_text += chunk
+    elif display_mode == constants.DISPLAY_MODE_TILES:
+        html_text = mark_safe("")
+        prev = None
+        for obj in ar.sliced_data_iterator:
+            chunk = obj.as_tile(ar, prev)
+            assert_safe(chunk)
+            html_text += chunk
+            prev = obj
+        html_text = format_html(
+            constants.TILES_CONTAINER_TEMPLATE, tiles=html_text)
+        # html_text += mark_safe("")
+
+    elif display_mode == constants.DISPLAY_MODE_LIST:
+        grp = Grouper(ar)
+        html_text = grp.begin()
+        for obj in ar.sliced_data_iterator:
+            par = ar.row_as_paragraph(obj)  # 20230207
+            # assert_safe(par)
+            # 20240506 par = ar.add_detail_link(obj, par)
+            # par = ar.obj2str(obj, par)
+            par = format_html('<p class="clearfix">{}</p>', par)
+            # url = ar.renderer.obj2url(ar, obj)
+            # if url is not None:
+            #     url = html.escape(url)
+            #     # par += ' <a href="{}">(Detail)</a>'.format(url)
+            #     par = '<a href="{}">{}</a>'.format(url, par)
+            html_text += grp.before_row(obj)
+            html_text += par
+            html_text += grp.after_row(obj)
+        html_text += grp.stop()
+    elif display_mode == constants.DISPLAY_MODE_SUMMARY:
+        html_text = ar.actor.get_table_summary(ar)
+    elif display_mode == constants.DISPLAY_MODE_HTML:
+        html_text = ar.actor.slave_as_html(ar.master_instance, ar)
+    else:
+        html_text = None
+    return html_text
