@@ -1,33 +1,39 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2023 Rumma & Ko Ltd.
+# Copyright 2009-2025 Rumma & Ko Ltd.
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
+from click import confirm
+from lino.api import dd
+from django.db import models
+from django.db import connections, transaction, DEFAULT_DB_ALIAS
+from django.core.management.color import no_style
+from django.db.utils import IntegrityError, OperationalError
+from django.core.management.base import BaseCommand, CommandError
+from django.core.management import call_command
+from django.conf import settings
 import os
 import shutil
 from pathlib import Path
 import tempfile
 
 import warnings
-warnings.filterwarnings(
-    "ignore", "No fixture named '.*' found.",
-    UserWarning, "django.core.management.commands.loaddata")
-warnings.filterwarnings(
-    "ignore", "No fixture data found for *",
-    RuntimeWarning, "django.core.management.commands.loaddata")
 
-from django.conf import settings
-from django.core.management import call_command
-from django.core.management.base import BaseCommand, CommandError
-from django.db.utils import IntegrityError, OperationalError
-from django.db.migrations.exceptions import CircularDependencyError
-from django.core.management.color import no_style
+warnings.filterwarnings(
+    "ignore",
+    "No fixture named '.*' found.",
+    UserWarning,
+    "django.core.management.commands.loaddata",
+)
+warnings.filterwarnings(
+    "ignore",
+    "No fixture data found for *",
+    RuntimeWarning,
+    "django.core.management.commands.loaddata",
+)
+
+
 # ~ from django.core.management.sql import sql_reset
-from django.db import connections, transaction, DEFAULT_DB_ALIAS
-from django.db import models
 
-from lino.api import dd
-
-from rstgen.utils import confirm
 
 USE_SQLDELETE = True
 
@@ -70,6 +76,7 @@ def remove_tree(pth):
     # pth.rename(tempdir)
     # pth.mkdir()
 
+
 # def really_remove(top):
 #     for pth in top.iterdir():
 #         if pth.is_dir():
@@ -83,26 +90,40 @@ def remove_tree(pth):
 
 
 class Command(BaseCommand):
-    """Flush the database and load the specified fixtures.
-
-    """
+    """Flush the database and load the specified fixtures."""
 
     def add_arguments(self, parser):
         super().add_arguments(parser)
-        parser.add_argument('fixtures', nargs='*', help='the fixtures to load')
-        parser.add_argument('--noinput', action='store_false',
-                            dest='interactive', default=True,
-                            help='Do not prompt for input of any kind.'),
+        parser.add_argument("fixtures", nargs="*", help="the fixtures to load")
+        (
+            parser.add_argument(
+                "--noinput",
+                action="store_false",
+                dest="interactive",
+                default=True,
+                help="Do not prompt for input of any kind.",
+            ),
+        )
         # parser.add_argument('--nobuildcache', action='store_false',
         #                     dest='buildcache', default=True,
         #                     help='Does nothing.'),
-        parser.add_argument('--removemedia', action='store_true',
-                            dest='removemedia', default=False,
-                            help='Remove all files in the settings.MEDIA_ROOT directory.'),
-        parser.add_argument('--database', action='store', dest='database',
-                            default=DEFAULT_DB_ALIAS,
-                            help='Nominates a database to reset. '
-                                 'Defaults to the "default" database.')
+        (
+            parser.add_argument(
+                "--removemedia",
+                action="store_true",
+                dest="removemedia",
+                default=False,
+                help="Remove all files in the settings.MEDIA_ROOT directory.",
+            ),
+        )
+        parser.add_argument(
+            "--database",
+            action="store",
+            dest="database",
+            default=DEFAULT_DB_ALIAS,
+            help="Nominates a database to reset. "
+            'Defaults to the "default" database.',
+        )
 
     def try_sql(self, conn, sql_list):
         hope = False
@@ -135,10 +156,15 @@ class Command(BaseCommand):
                 if p.app_name.startswith(prefix):
                     makemigrations = True
                     break
-            if p.app_name.startswith('lino') or makemigrations:
-                migrations_dir = Path(p.app_module.__file__).parent / 'migrations'
+            if p.app_name.startswith("lino") or makemigrations:
+                migrations_dir = Path(p.app_module.__file__).parent / "migrations"
                 migrations_dir.mkdir()
-                call_command('makemigrations', p.app_name.split('.')[-1], interactive=False, verbosity=verbosity)
+                call_command(
+                    "makemigrations",
+                    p.app_name.split(".")[-1],
+                    interactive=False,
+                    verbosity=verbosity,
+                )
 
     def removemigrations(self):
         for p in settings.SITE.installed_plugins:
@@ -147,20 +173,20 @@ class Command(BaseCommand):
                 if p.app_name.startswith(prefix):
                     rm_mig_dir = True
                     break
-            if p.app_name.startswith('lino') or rm_mig_dir:
-                migrations_dir = Path(p.app_module.__file__).parent / 'migrations'
+            if p.app_name.startswith("lino") or rm_mig_dir:
+                migrations_dir = Path(p.app_module.__file__).parent / "migrations"
                 if migrations_dir.exists():
                     shutil.rmtree(migrations_dir)
 
     def handle(self, *args, **options):
-        interactive = options.get('interactive')
-        verbosity = options.get('verbosity')
-        using = options.get('database', DEFAULT_DB_ALIAS)
-        dbname = settings.DATABASES[using]['NAME']
-        engine = settings.DATABASES[using]['ENGINE']
+        interactive = options.get("interactive")
+        verbosity = options.get("verbosity")
+        using = options.get("database", DEFAULT_DB_ALIAS)
+        dbname = settings.DATABASES[using]["NAME"]
+        engine = settings.DATABASES[using]["ENGINE"]
 
         # buildcache = options.pop('buildcache')
-        removemedia = options.pop('removemedia')
+        removemedia = options.pop("removemedia")
         assert str(settings.SITE.media_root) == settings.MEDIA_ROOT
         mroot = settings.SITE.media_root
         if not mroot.exists():
@@ -170,15 +196,15 @@ class Command(BaseCommand):
             msg = "We are going to flush your database ({})".format(dbname)
             if removemedia:
                 msg += "\nAND REMOVE ALL FILES BELOW {}".format(mroot)
-            msg += ".\nAre you sure (y/n) ?"
-            if not confirm(msg):
+            msg += ".\nAre you sure?"
+            if not confirm(msg, default=True):
                 raise CommandError("User abort.")
 
         # mroot = Path(settings.MEDIA_ROOT)
         if removemedia:
             remove_tree(mroot)
 
-        fixtures = options.pop('fixtures', args)  # don't keep this option
+        fixtures = options.pop("fixtures", args)  # don't keep this option
 
         # print(20160817, fixtures, options)
 
@@ -187,13 +213,14 @@ class Command(BaseCommand):
         # the following log message was useful on Travis 20150104
         if verbosity > 0:
             dd.logger.info(
-                "`initdb %s` started on database %s.", ' '.join(fixtures), dbname)
+                "`initdb %s` started on database %s.", " ".join(fixtures), dbname
+            )
 
-        if engine == 'django.db.backends.sqlite3':
-            if dbname != ':memory:' and os.path.isfile(dbname):
+        if engine == "django.db.backends.sqlite3":
+            if dbname != ":memory:" and os.path.isfile(dbname):
                 os.remove(dbname)
                 del connections[using]
-        elif engine == 'django.db.backends.mysql':
+        elif engine == "django.db.backends.mysql":
             conn = connections[using]
             cursor = conn.cursor()
             cursor.execute("DROP DATABASE %s;" % dbname)
@@ -207,7 +234,7 @@ class Command(BaseCommand):
             conn = connections[using]
             cursor = conn.cursor()
             cursor.execute("set foreign_key_checks=0;")
-        elif engine.startswith('django.db.backends.postgresql'):
+        elif engine.startswith("django.db.backends.postgresql"):
             foralltables(using, "DROP TABLE IF EXISTS {} CASCADE;")
             # cmd = """select 'DROP TABLE "' || tablename || '" IF EXISTS CASCADE;' from pg_tables where schemaname = 'public';"""
             # cursor.execute(cmd)
@@ -227,8 +254,9 @@ class Command(BaseCommand):
                 # sql_delete was removed in Django 1.9
                 # ~ sql_list = u'\n'.join(sql_reset(app, no_style(), conn)).encode('utf-8')
 
-                app_list = [models.get_app(p.app_label)
-                            for p in settings.SITE.installed_plugins]
+                app_list = [
+                    models.get_app(p.app_label) for p in settings.SITE.installed_plugins
+                ]
                 for app in app_list:
                     # app_label = app.__name__.split('.')[-2]
                     sql_list.extend(sql_delete(app, no_style(), conn))
@@ -247,18 +275,18 @@ class Command(BaseCommand):
 
             transaction.commit_unless_managed()
 
-        if engine.startswith('django.db.backends.postgresql'):
+        if engine.startswith("django.db.backends.postgresql"):
             try:
-                call_command('migrate', **options)
+                call_command("migrate", **options)
                 self.makemigrations(verbosity)
-                call_command('makemigrations', interactive=False, verbosity=verbosity)
-                call_command('migrate', '--run-syncdb', **options)
+                call_command("makemigrations", interactive=False, verbosity=verbosity)
+                call_command("migrate", "--run-syncdb", **options)
             finally:
                 self.removemigrations()
         else:
             # call_command('makemigrations', '--merge', interactive=False, verbosity=verbosity)
-            call_command('makemigrations', interactive=False, verbosity=verbosity)
-            call_command('migrate', '--run-syncdb', **options)
+            call_command("makemigrations", interactive=False, verbosity=verbosity)
+            call_command("migrate", "--run-syncdb", **options)
 
         for p in settings.SITE.installed_plugins:
             p.on_initdb(settings.SITE, verbosity=verbosity)
@@ -267,15 +295,12 @@ class Command(BaseCommand):
             # if engine == 'django.db.backends.postgresql':
             #     foralltables(using, "ALTER TABLE {} DISABLE TRIGGER ALL;")
 
-            options.pop('interactive')
-            call_command('loaddata', *fixtures, **options)
+            options.pop("interactive")
+            call_command("loaddata", *fixtures, **options)
 
             # if engine == 'django.db.backends.postgresql':
             #     foralltables(using, "ALTER TABLE {} ENABLE TRIGGER ALL;")
 
-        # if buildcache:  # why did we add this at all?
-        #     settings.SITE.build_site_cache(verbosity=verbosity)
-
         settings.SITE.clear_site_config()
 
         # dblogger.info("Lino initdb %s done on database %s.", args, dbname)
