@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2010-2021 Rumma & Ko Ltd
+# Copyright 2010-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 This package contains model mixins, some of which are heavily used
 by applications and the :ref:`xl`. But none of them is mandatory.
@@ -9,8 +8,9 @@ by applications and the :ref:`xl`. But none of them is mandatory.
 .. autosummary::
    :toctree:
 
-    duplicable
+
     dupable
+    clonable
     sequenced
     human
     periods
@@ -23,32 +23,53 @@ by applications and the :ref:`xl`. But none of them is mandatory.
 from django.db import models
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
-from django.utils.text import format_lazy
-from django.utils import timezone
+# from django.utils.html import format_html
+# from django.utils.text import format_lazy
+# from django.utils import timezone
 from django.contrib.humanize.templatetags.humanize import naturaltime
 
+# Note that reordering the imports here can cause the field ordering to change
+# in models like lino_voga.lib.courses.TeacherType, which inherits from
+# `(Referrable, BabelNamed, Printable)`. This can cause doctests like
+# docs/specs/voga/courses.rst to fail because the `ref` field then came after
+# the name field. The TeacherTypes table has no explicit `column_names`, so it
+# uses the "natural" field ordering, which is, as this observation shows, quite
+# unpredictable.
+
 from lino.core import actions
 from lino.core import fields
 from lino.core import model
-from lino.core.workflows import ChangeStateAction
+# from lino.core.workflows import ChangeStateAction
+# from lino.core.exceptions import ChangedAPI
 from lino.utils.mldbc.fields import LanguageField
-from lino.core.exceptions import ChangedAPI
+from lino.utils.html import E
+from lino.utils.mldbc.mixins import BabelNamed, BabelDesignated
+from lino.utils.mldbc.fields import BabelCharField, BabelTextField
+
+from .human import Human
+from .polymorphic import Polymorphic
+from .periods import ObservedDateRange, Yearly, Monthly, Today
+from .periods import DateRange
+from .sequenced import Sequenced, Hierarchical
+from .clonable import Clonable, CloneRow
+from .registrable import Registrable, RegistrableState
+from .ref import Referrable, StructuredReferrable
+
 
 class Contactable(model.Model):
     """
     Mixin for models that represent somebody who can be contacted by
     email.
     """
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
-    email = models.EmailField(_('e-mail address'), blank=True)
+    email = models.EmailField(_("e-mail address"), blank=True)
     language = LanguageField(default=models.NOT_PROVIDED, blank=True)
 
     def get_as_user(self):
-        """Return the user object representing this contactable.
-
-        """
+        """Return the user object representing this contactable."""
         raise NotImplementedError()
 
 
@@ -58,18 +79,18 @@ class Phonable(model.Model):
     phone.
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
-    url = models.URLField(_('URL'), blank=True)
-    phone = models.CharField(_('Phone'), max_length=200, blank=True)
-    gsm = models.CharField(_('Mobile'), max_length=200, blank=True)
-    fax = models.CharField(_('Fax'), max_length=200, blank=True)
+    url = models.URLField(_("URL"), blank=True)
+    phone = models.CharField(_("Phone"), max_length=200, blank=True)
+    gsm = models.CharField(_("Mobile"), max_length=200, blank=True)
+    fax = models.CharField(_("Fax"), max_length=200, blank=True)
 
 
 class Modified(model.Model):
     """
-    Adds a a timestamp field which holds the last modification time of
+    Adds a a timestamp field that holds the last modification time of
     every individual database object.
 
     .. attribute:: modified
@@ -85,7 +106,7 @@ class Modified(model.Model):
     you explicitly call :meth:`touch`.
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     modified = models.DateTimeField(_("Modified"), editable=False, null=True)
@@ -96,49 +117,48 @@ class Modified(model.Model):
         super().save(*args, **kwargs)
 
     def touch(self):
-        self.modified = timezone.now()
+        self.modified = settings.SITE.now()
 
 
 class Created(model.Model):
     """
-    Adds a timestamp field which holds the creation time of every
-    individual database object.
+    Adds a timestamp field that holds the creation time of every
+    individual :term:`database row`.
 
     .. attribute:: created
 
-        The time when this object was created.
+        The time when this :term:`database row` was created.
 
-        Does nut use Django's `auto_now` and `auto_now_add` features
+        Does not use Django's `auto_now` and `auto_now_add` features
         because their deserialization would be problematic.
     """
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
     created = models.DateTimeField(_("Created"), editable=False)
 
-    @fields.displayfield(_('Created'))
+    @fields.displayfield(_("Created"))
     def created_natural(self, ar):
         return naturaltime(self.created)
 
     def save(self, *args, **kwargs):
-        if self.created is None and not settings.SITE.loading_from_dump:
-            self.created = timezone.now()
+        if self.created is None:  # and not settings.SITE.loading_from_dump:
+            self.created = settings.SITE.now()
         super().save(*args, **kwargs)
 
 
 class CreatedModified(Created, Modified):
-
     """
     Adds two timestamp fields `created` and `modified`.
 
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
 
 class ProjectRelated(model.Model):
-
     """
     Mixin for models that are related to a "project",
     i.e. to an object of the type given by your `lino.core.site.Site.project_model`.
@@ -156,31 +176,36 @@ class ProjectRelated(model.Model):
         <lino.core.fields.DummyField>`.
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     project = fields.ForeignKey(
         settings.SITE.project_model,
-        blank=True, null=True,
-        related_name="%(app_label)s_%(class)s_set_by_project")
+        blank=True,
+        null=True,
+        related_name="%(app_label)s_%(class)s_set_by_project",
+    )
 
     def get_related_project(self):
-        if settings.SITE.project_model:
-            return self.project
+        # if settings.SITE.project_model:
+        # When project_model is None, project is a dummy field which always returns None
+        return self.project
 
     # def on_create(self, ar):
-    #     super(ProjectRelated, self).on_create(ar)
+    #     super().on_create(ar)
     #     print(20200327, ar.actor.master_key, ar.master_instance)
     #     if ar.actor.master_key and ar.actor.master_key == "project":
     #         self.project = ar.master_instance
 
-    def as_summary_row(self, ar, **kwargs):
-        s = super().as_summary_row(ar, **kwargs)
+    def as_summary_item(self, ar, text=None, **kwargs):
+        e = super().as_summary_item(ar, text, **kwargs)
         # s = [ar.obj2html(self)]
-        if settings.SITE.project_model:
+        if text is None and ar and settings.SITE.project_model:
             if self.project and not ar.is_obvious_field("project"):
-                s += " ({)}".format(ar.obj2htmls(self.project))
-        return s
+                e = E.span(e, " (", self.project.as_summary_item(ar), ")")
+                # s += " ({)}".format(ar.obj2htmls(self.project))
+                # s = format_html("{} ({})", s, self.project.as_summary_row(ar, **kwargs))
+        return e
 
     def update_owned_instance(self, controllable):
         """
@@ -190,27 +215,27 @@ class ProjectRelated(model.Model):
         """
         if isinstance(controllable, ProjectRelated):
             controllable.project = self.project
-        super(ProjectRelated, self).update_owned_instance(controllable)
+        super().update_owned_instance(controllable)
 
     def get_mailable_recipients(self):
         if isinstance(self.project, settings.SITE.models.contacts.Partner):
             if self.project.email:
-                yield ('to', self.project)
-        for r in super(ProjectRelated, self).get_mailable_recipients():
+                yield ("to", self.project)
+        for r in super().get_mailable_recipients():
             yield r
 
     def get_postable_recipients(self):
         if isinstance(self.project, settings.SITE.models.contacts.Partner):
             yield self.project
-        for p in super(ProjectRelated, self).get_postable_recipients():
+        for p in super().get_postable_recipients():
             yield p
 
     @classmethod
     def get_simple_parameters(cls):
-        for p in super(ProjectRelated, cls).get_simple_parameters():
+        for p in super().get_simple_parameters():
             yield p
         # if settings.SITE.project_model:
-        yield 'project'
+        yield "project"
 
     # @classmethod
     # def setup_parameters(cls, params):
@@ -231,18 +256,17 @@ class Story(model.Model):
     @fields.virtualfield(fields.HtmlBox())
     def body(self, ar):
         if ar is None:
-            return ''
+            return ""
         # ar.master_instance = self
-        html = ar.renderer.show_story(
-            ar, self.get_story(ar), header_level=1)
+        html = ar.renderer.show_story(ar, self.get_story(ar), header_level=1)
         return ar.html_text(html)
         # return ar.html_text(ar.story2html(
         #     self.get_story(ar), header_level=1))
 
     def as_appy_pod_xml(self, apr):
-        chunks = tuple(apr.story2odt(
-            self.get_story(apr.ar), master_instance=self))
-        return str('').join(chunks)  # must be utf8 encoded
+        chunks = tuple(apr.story2odt(self.get_story(apr.ar), master_instance=self))
+        return str("").join(chunks)  # must be utf8 encoded
+
 
 class DragAndDrop(actions.Action):
     select_rows = True
@@ -252,6 +276,7 @@ class DragAndDrop(actions.Action):
         obj = ar.selected_rows[0]
         obj.on_dropped(ar, **kw)
 
+
 class Draggable(model.Model):
     class Meta:
         abstract = True
@@ -260,26 +285,3 @@ class Draggable(model.Model):
 
     def on_dropped(self, ar, **kwargs):
         pass
-
-
-
-from .ref import Referrable, StructuredReferrable
-from .registrable import Registrable, RegistrableState
-from lino.mixins.duplicable import Duplicable, Duplicate
-from lino.mixins.sequenced import Sequenced, Hierarchical
-from lino.mixins.periods import DateRange
-from lino.mixins.periods import ObservedDateRange, Yearly, Monthly, Today
-from lino.mixins.polymorphic import Polymorphic
-
-# Observation: moving the following two lines to the top (to be together with
-# the import of LanguageField) caused the field ordering to change in models
-# like `lino_voga.lib.courses.TeacherType` which inherits from `(Referrable,
-# BabelNamed, Printable)`. Which caused docs/specs/voga/courses.rst to fail
-# because the `ref` field then came after the name field. The TeacherTypes table
-# has no explicit `column_names`, so it uses the "natural" field ordering, which
-# is, as this observation shows, quite unpredictable.
-
-from lino.utils.mldbc.fields import BabelCharField, BabelTextField
-from lino.utils.mldbc.mixins import BabelNamed, BabelDesignated
-
-from lino.mixins.human import Human
