@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2017 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the :class:`Polymorphic` model mixin.
 See :doc:`/dev/mti`.
 
@@ -11,7 +10,7 @@ from builtins import str
 
 from django.core.exceptions import ObjectDoesNotExist
 from django.utils.translation import gettext_lazy as _
-from etgen.html import E
+from lino.utils.html import E
 
 from lino.core import fields
 from lino.core import model
@@ -23,7 +22,6 @@ from lino.utils import mti
 
 
 class ChildAction(Action):
-
     readonly = False
     show_in_toolbar = False
 
@@ -41,7 +39,7 @@ class ChildAction(Action):
 
     def get_child(self, obj):
         """Returns the MTI child in `self.child_model` if it exists, otherwise
-return None.
+        return None.
 
         """
         try:
@@ -55,7 +53,7 @@ return None.
 
 
 class DeleteChild(ChildAction):
-    name_prefix = 'del_'
+    name_prefix = "del_"
 
     def get_action_permission(self, ar, obj, state):
         if self.get_child(obj) is None:
@@ -63,22 +61,19 @@ class DeleteChild(ChildAction):
         return super(DeleteChild, self).get_action_permission(ar, obj, state)
 
     def run_on_row(self, obj, ar):
-
         if ar is not None:
             pre_remove_child.send(
-                sender=obj, request=ar.request,
-                child=self.child_model)
+                sender=obj, request=ar.request, child=self.child_model
+            )
         mti.delete_child(obj, self.child_model, ar)
         ar.set_response(refresh=True)
 
 
 class InsertChild(ChildAction):
-
-    name_prefix = 'ins_'
+    name_prefix = "ins_"
 
     def get_action_permission(self, ar, obj, state):
-        parent_link_field = self.child_model._meta.parents.get(
-            obj.__class__, None)
+        parent_link_field = self.child_model._meta.parents.get(obj.__class__, None)
         if parent_link_field is None:
             return False
         if self.get_child(obj) is not None:
@@ -88,9 +83,7 @@ class InsertChild(ChildAction):
 
     def run_on_row(self, obj, ar):
         if ar is not None:
-            pre_add_child.send(
-                sender=obj, request=ar.request,
-                child=self.child_model)
+            pre_add_child.send(sender=obj, request=ar.request, child=self.child_model)
         mti.insert_child(obj, self.child_model, full_clean=True)
         ar.set_response(refresh=True)
 
@@ -105,7 +98,7 @@ class Polymorphic(model.Model):
     person and an employee at the same time.
 
     Note that not every usage of Multiple Table Inheritance means
-    polymorphism. For example `lino_xl.lib.ledger.models.Voucher`
+    polymorphism. For example `lino_xl.lib.accounting.models.Voucher`
     has a pointer to the journal which knows which specialization to
     use, so a given voucher has always exactly one specialization.
 
@@ -117,6 +110,7 @@ class Polymorphic(model.Model):
     Usage examples in :doc:`/dev/mti` and :doc:`/dev/lets/index`.
 
     """
+
     class Meta(object):
         abstract = True
 
@@ -165,7 +159,7 @@ class Polymorphic(model.Model):
                 return getattr(self, a)
             except ObjectDoesNotExist:
                 pass
-        #~ return self
+        # ~ return self
 
     def disable_delete(self, ar=None):
         """Overrides :meth:`lino.core.model.Model.disable_delete`.
@@ -188,8 +182,7 @@ class Polymorphic(model.Model):
                 if obj is not None:
                     ignore_models = set(self._mtinav_models)
                     ignore_models.remove(m)
-                    msg = m._lino_ddh.disable_delete_on_object(
-                        obj, ignore_models)
+                    msg = m._lino_ddh.disable_delete_on_object(obj, ignore_models)
                     if msg is not None:
                         return msg
 
@@ -198,7 +191,8 @@ class Polymorphic(model.Model):
         ignore_models = set(self._mtinav_models)
         ignore_models.remove(self.__class__)
         return self.__class__._lino_ddh.disable_delete_on_object(
-            self, ignore_models=ignore_models)
+            self, ignore_models=ignore_models
+        )
 
     def get_mti_buttons(self, ar):
         """"""
@@ -221,7 +215,7 @@ class Polymorphic(model.Model):
                         ba = ar.actor.get_action_by_name(k)
                         if ba and ba.get_row_permission(ar, self, None):
                             # btn = ar.row_action_button(self, ba, _("+"))
-                            btn = ar.row_action_button(self, ba, _(u"➕"))
+                            btn = ar.row_action_button(self, ba, _("➕"))
                             # Heavy Plus Sign U+2795
                             # btn = ar.row_action_button(self, ba,
                             #                            icon_name='add')
@@ -236,13 +230,13 @@ class Polymorphic(model.Model):
                             k = DeleteChild.name_prefix + m.__name__.lower()
                             ba = ar.actor.get_action_by_name(k)
                             if ba and ba.get_row_permission(ar, self, None):
-                                btn = ar.row_action_button(self, ba, _(u"❌"))
+                                btn = ar.row_action_button(self, ba, _("❌"))
                                 # Cross Mark U+274C
                                 item += [" [", btn, "]"]
 
             if item is not None:
                 if sep is None:
-                    sep = ', '
+                    sep = ", "
                 else:
                     elems.append(sep)
                 elems += item
@@ -253,15 +247,16 @@ class Polymorphic(model.Model):
         buttons = self.get_mti_buttons(ar)
         return E.span(*buttons)
 
-    def obj2href(self, ar, *args, **kwargs):
+    def as_summary_item(self, ar, text=None, **kwargs):
         # a = super(Polymorphic, self).get_detail_action(ar)
         a = self.get_detail_action(ar)
         if a is not None:
-            return super(Polymorphic, self).obj2href(ar, *args, **kwargs)
+            return super().as_summary_item(ar, text, **kwargs)
         for m in self._mtinav_models:
             if m is not self.__class__:
                 obj = mti.get_child(self, m)
                 if obj is not None:
                     if obj.get_detail_action(ar) is not None:
-                        return obj.obj2href(ar, *args, **kwargs)
-        return ''
+                        return obj.as_summary_item(ar, text, **kwargs)
+        return str(self)
+        # return "?!? {} has no detail_action for {}".format(self, ar)
