@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2021 Rumma & Ko Ltd
+# Copyright 2009-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Defines the model mixins :class:`Sequenced` and :class:`Hierarchical`.
@@ -20,11 +20,11 @@ from lino.core import actions
 from lino.core import fields
 from lino.core.inject import update_field
 from lino.core.utils import navinfo
-from etgen.html import E
+from lino.utils.html import E
 from lino.utils import AttrDict
 from lino.utils import join_elems
 
-from .duplicable import Duplicable, Duplicate
+from lino.mixins.clonable import Clonable, CloneRow
 
 
 class MoveByN(actions.Action):
@@ -33,7 +33,7 @@ class MoveByN(actions.Action):
     This action is available on any :class:`Sequenced` object as
     :attr:`Sequenced.move_by_n`.
 
-    It is currently only used by React to allow for drag and drop reording.
+    It is currently only used by React to allow for drag and drop reordering.
 
     """
 
@@ -44,14 +44,14 @@ class MoveByN(actions.Action):
     # label = "↑"  #
     custom_handler = True
     # icon_name = 'arrow_up'
-    #~ icon_file = 'arrow_up.png'
+    # ~ icon_file = 'arrow_up.png'
     readonly = False
     show_in_toolbar = False
 
     def get_action_permission(self, ar, obj, state):
         if ar.data_iterator is None:
             return False
-        if not super(MoveByN, self).get_action_permission(ar, obj, state):
+        if not super().get_action_permission(ar, obj, state):
             return False
         if ar.get_total_count() == 0:
             return False
@@ -59,7 +59,7 @@ class MoveByN(actions.Action):
 
     def run_from_ui(self, ar, **kw):
         obj = ar.selected_rows[0]
-        obj.seqno += int(ar.request.GET['seqno'])
+        obj.seqno += int(ar.request.GET["seqno"])
         obj.seqno_changed(ar)
         # obj.full_clean()
         obj.save()
@@ -69,7 +69,6 @@ class MoveByN(actions.Action):
         ar.success(**kw)
 
 
-
 class MoveUp(actions.Action):
     """Move this row one row upwards.
 
@@ -81,21 +80,26 @@ class MoveUp(actions.Action):
     # label = _("Up")
     # label = "\u2191" thin arrow up
     # label = "\u25b2" # triangular arrow up
-    label = "\u25B2"  # ▲ Black up-pointing triangle
-    # label = "↑"  #
+    label = _("Move up")
+    # button_text = "\u25B2"  # ▲ Black up-pointing triangle
+    button_text = "↑"
     custom_handler = True
+    callable_from = "t"
     # icon_name = 'arrow_up'
-    #~ icon_file = 'arrow_up.png'
+    # ~ icon_file = 'arrow_up.png'
     readonly = False
 
     def get_action_permission(self, ar, obj, state):
         if ar.data_iterator is None:
             return False
-        if not super(MoveUp, self).get_action_permission(ar, obj, state):
+        if not super().get_action_permission(ar, obj, state):
             return False
+        # if ar.order_by is None or "seqno" not in ar.order_by:
+        #     return False
         if ar.get_total_count() == 0:
             return False
         if ar.data_iterator[0] == obj:
+            # print(f"20250305 first of {ar.data_iterator}")
             return False
         # print("20161128", obj.seqno, ar.data_iterator.count())
         return True
@@ -119,28 +123,34 @@ class MoveDown(actions.Action):
     :attr:`Sequenced.move_down`.
 
     """
+
+    label = _("Move down")
     # label = _("Down")
-    label = "↓"
+    button_text = "↓"
     # label = "\u25bc" # triangular arrow down
     # label = "\u2193"
-    label = "\u25BC"  # ▼ Black down-pointing triangle
+    # button_text = "\u25BC"  # ▼ Black down-pointing triangle
     # icon_name = 'arrow_down'
     custom_handler = True
-    #~ icon_file = 'arrow_down.png'
+    # ~ icon_file = 'arrow_down.png'
+    callable_from = "t"
     readonly = False
 
     def get_action_permission(self, ar, obj, state):
         if ar.data_iterator is None:
             return False
-        if not super(MoveDown, self).get_action_permission(ar, obj, state):
+        if not super().get_action_permission(ar, obj, state):
             return False
+        # if ar.order_by is None or "seqno" not in ar.order_by:
+        #     return False
         n = ar.get_total_count()
         if n == 0:
             return False
         if ar.data_iterator[n - 1] == obj:
+            # print(f"20250305 last of {ar.data_iterator}")
             return False
-        #~ if obj.__class__.__name__=='Entry' and obj.seqno == 25:
-            #~ print 20130706, ar.data_iterator.count(), ar.data_iterator
+        # ~ if obj.__class__.__name__=='Entry' and obj.seqno == 25:
+        # ~ print 20130706, ar.data_iterator.count(), ar.data_iterator
         return True
 
     def run_from_ui(self, ar, **kw):
@@ -151,30 +161,29 @@ class MoveDown(actions.Action):
         # obj.full_clean()
         obj.save()
         kw = dict()
-        #~ kw.update(refresh=True)
+        # ~ kw.update(refresh=True)
         kw.update(refresh_all=True)
         kw.update(message=_("Moved down."))
         ar.success(**kw)
 
 
-class DuplicateSequenced(Duplicate):
-    """Duplicate this row."""
+class CloneSequenced(CloneRow):
+
     def run_from_code(self, ar, **kw):
         obj = ar.selected_rows[0]
 
-        #~ print '20120605 duplicate', self.seqno, self.account
+        # ~ print '20120605 duplicate', self.seqno, self.account
         seqno = obj.seqno
-        qs = obj.get_siblings().filter(seqno__gt=seqno).order_by('-seqno')
+        qs = obj.get_siblings().filter(seqno__gt=seqno).order_by("-seqno")
         for s in qs:
-            #~ print '20120605 duplicate inc', s.seqno, s.account
+            # ~ print '20120605 duplicate inc', s.seqno, s.account
             s.seqno += 1
             s.save()
-        kw.update(seqno=seqno+1)
-        return super(DuplicateSequenced, self).run_from_code(ar, **kw)
+        kw.update(seqno=seqno + 1)
+        return super().run_from_code(ar, **kw)
 
 
-
-class Sequenced(Duplicable):
+class Sequenced(Clonable):
     """Mixin for models that have a field :attr:`seqno` containing a
     "sequence number".
 
@@ -182,12 +191,11 @@ class Sequenced(Duplicable):
 
         The sequence number of this item with its parent.
 
-    .. method:: duplicate
+    .. method:: clone_row
 
-        Create a duplicate of this object and insert the new object
-        below this one.
+        Create a duplicate of this row and insert the new row below this one.
 
-        Implemented by :class:`DuplicateSequenced`
+        Implemented by :class:`CloneSequenced`
 
     .. attribute:: move_up
 
@@ -202,34 +210,33 @@ class Sequenced(Duplicable):
         Displays buttons for certain actions on this row:
 
         - :attr:`move_up` and :attr:`move_down`
-        - duplicate
+        - clone_row
 
     .. attribute:: move_by_n
 
     """
 
-    move_action_names = ('move_up', 'move_down', 'duplicate')
+    move_action_names = ("move_up", "move_down", "clone_row")
     """The names of the actions to display in the `move_buttons`
     column.
 
     Overridden by :class:`lino.modlib.dashboard.Widget` where the
-    duplicate button would be irritating.
+    clone_row button would be irritating.
 
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
-        ordering = ['seqno']
+        ordering = ["seqno"]
 
     seqno = models.IntegerField(_("No."), blank=True, null=False)
 
-    duplicate = DuplicateSequenced()
+    clone_row = CloneSequenced()
 
     move_up = MoveUp()
     move_down = MoveDown()
     move_by_n = MoveByN()
 
-
     def __str__(self):
         return str(_("Row # %s") % self.seqno)
 
@@ -263,27 +270,23 @@ class Sequenced(Duplicable):
         :class:`lino_welfare.modlib.debts.models.Entry`.
 
         """
-        return self.__class__.objects.order_by('seqno')
+        return self.__class__.objects.order_by("seqno")
 
     def set_seqno(self):
         """
         Initialize `seqno` to the `seqno` of eldest sibling + 1.
         """
-        qs = self.get_siblings().order_by('seqno')
-        if qs is None:  # TODO: remove this as it is no longer used (?)
-            self.seqno = 0
+        qs = self.get_siblings().order_by("seqno")
+        if (count := qs.count()) == 0:
+            self.seqno = 1
         else:
-            n = qs.count()
-            if n == 0:
-                self.seqno = 1
-            else:
-                last = qs[n - 1]
-                self.seqno = last.seqno + 1
+            last = qs[count - 1]
+            self.seqno = last.seqno + 1
 
     def full_clean(self, *args, **kw):
         if not self.seqno:
             self.set_seqno()
-        super(Sequenced, self).full_clean(*args, **kw)
+        super().full_clean(*args, **kw)
 
         # if hasattr(self, 'amount'):
         #     logger.info("20151117 Sequenced.full_clean a %s", self.amount)
@@ -293,9 +296,8 @@ class Sequenced(Duplicable):
 
     def seqno_changed(self, ar):
         """If the user manually assigns a seqno."""
-        #get siblings list
-        qs = self.get_siblings().order_by('seqno').exclude(
-            id=self.id)
+        # get siblings list
+        qs = self.get_siblings().order_by("seqno").exclude(id=self.id)
 
         # print("20170615 qs is", qs)
         # old_self = qs.get(id=self.id)
@@ -317,14 +319,22 @@ class Sequenced(Duplicable):
 
             seq_no += 1
 
-        ar.success(message=_("Renumbered {} of {} siblings.").format(
-            n, qs.count()))
+        ar.success(
+            message=_("Renumbered {} of {} siblings.").format(n, qs.count()))
         ar.set_response(refresh_all=True)
 
+    @fields.displayfield("⇵", wildcard_data_elem=True)
+    def dndreorder(self, ar):
+        """A place holder column for drag and drop row reorder on :term:`React front end`
+
+        CAUTION: Do NOT rename this field, for react works on checking the name as dndreorder.
+        """
+        return None
+
     @fields.displayfield(_("Move"))
     def move_buttons(obj, ar):
         if ar is None:
-            return ''
+            return ""
         actor = ar.actor
         l = []
         state = None  # TODO: support a possible state?
@@ -332,14 +342,15 @@ class Sequenced(Duplicable):
             ba = actor.get_action_by_name(n)
             if ba.get_row_permission(ar, obj, state):
                 l.append(ar.renderer.action_button(obj, ar, ba))
-                l.append(' ')
+                l.append(" ")
         return E.p(*l)
 
-Sequenced.set_widget_options('move_buttons', width=5)
-Sequenced.set_widget_options('seqno', hide_sum=True)
+
+Sequenced.set_widget_options("move_buttons", width=5)
+Sequenced.set_widget_options("seqno", hide_sum=True)
 
 
-class Hierarchical(Duplicable):
+class Hierarchical(Clonable):
     """Model mixin for things that have a "parent" and "siblings".
 
     Pronounciation: [hai'ra:kikl]
@@ -349,20 +360,20 @@ class Hierarchical(Duplicable):
         A comma-separated list of the children.
 
     """
+
     class Meta(object):
         abstract = True
 
-    parent = fields.ForeignKey('self',
-                               verbose_name=_("Parent"),
-                               null=True, blank=True,
-                               related_name='children')
+    parent = fields.ForeignKey(
+        "self", verbose_name=_("Parent"), null=True, blank=True, related_name="children"
+    )
 
     @fields.displayfield(_("Children"))
     def children_summary(self, ar):
         if ar is None:
-            return ''
+            return ""
         elems = [ar.obj2html(ch) for ch in self.children.all()]
-        elems = join_elems(elems, sep=', ')
+        elems = join_elems(elems, sep=", ")
         return E.p(*elems)
 
     def get_siblings(self):
@@ -370,15 +381,15 @@ class Hierarchical(Duplicable):
             return self.parent.children.all()
         return self.__class__.objects.filter(parent__isnull=True)
 
-    #~ def save(self, *args, **kwargs):
-        #~ super(Hierarchical, self).save(*args, **kwargs)
+    # ~ def save(self, *args, **kwargs):
+    # ~ super(Hierarchical, self).save(*args, **kwargs)
     def full_clean(self, *args, **kwargs):
         p = self.parent
         while p is not None:
             if p == self:
                 raise ValidationError("Cannot be your own ancestor")
             p = p.parent
-        super(Hierarchical, self).full_clean(*args, **kwargs)
+        super().full_clean(*args, **kwargs)
 
     def is_parented(self, other):
         if self == other:
@@ -421,9 +432,7 @@ class Hierarchical(Duplicable):
         return tree
 
     def whole_clan(self):
-        """Return a set of this instance and all children and grandchildren.
-
-        """
+        """Return a set of this instance and all children and grandchildren."""
         # TODO: go deeper but check for circular references
         clan = set([self])
         l1 = self.__class__.objects.filter(parent=self)
@@ -456,6 +465,7 @@ class Hierarchical(Duplicable):
         parent = self.get_ancestor()
         items.append(parent)
         child = parent.children.all()
+
         def append_to_tree(parent, child, tree):
             if child.count() == 0:
                 return
@@ -467,6 +477,7 @@ class Hierarchical(Duplicable):
                     items.append(c)
                     t = t + ((c, append_to_tree(c, c.children.all(), tree)),)
                 return t
+
         return (parent, append_to_tree(parent, child, tuple()))
 
     @fields.htmlbox()
@@ -479,22 +490,30 @@ class Hierarchical(Duplicable):
         tab = " ⸱ "  # U+2e31
 
         # folder_closed = "🗀 " # U+1F5C0
-        folder_closed = "⌾ " # U+233e
+        folder_closed = "⌾ "  # U+233e
         # folder_open = "🗁 " # U+1F5C1
-        folder_open = "⌾ " # U+233e
-        folder_with_child_closed = "📁 " # U+1F4C1 "🗂 "
-        folder_with_child_open = "📂 " # U+1F4C2
+        folder_open = "⌾ "  # U+233e
+        folder_with_child_closed = "📁 "  # U+1F4C1 "🗂 "
+        folder_with_child_open = "📂 "  # U+1F4C2
 
         et = E.div()
 
         def append_to_et(child, i=0):
             def get_text(child):
-                return (tab * i) + (
-                    folder_open if child in openned and child.children.count() == 0
-                    else folder_with_child_open if child in openned
-                    else folder_closed if child.children.count() == 0
-                    else folder_with_child_closed
-                    ) + str(child)
+                return (
+                    (tab * i)
+                    + (
+                        folder_open
+                        if child in openned and child.children.count() == 0
+                        else folder_with_child_open
+                        if child in openned
+                        else folder_closed
+                        if child.children.count() == 0
+                        else folder_with_child_closed
+                    )
+                    + str(child)
+                )
+
             et.append(ar.goto_pk(child.id, get_text(child)))
             if child.children.count() > 0:
                 i += 1
@@ -507,7 +526,9 @@ class Hierarchical(Duplicable):
                         et.append(ar.goto_pk(c.id, get_text(c)))
                 i -= 1
             if child == self:
-                sar = ar.actor.request(parent=ar, master_instance=self, is_on_main_actor=False)
+                sar = ar.actor.create_request(
+                    parent=ar, master_instance=self, is_on_main_actor=False
+                )
                 # sar = ar.spawn_request(master_instance=self, is_on_main_actor=False)
                 if ar.actor.insert_action is not None:
                     ir = ar.actor.insert_action.request_from(sar)
