@@ -1,6 +1,5 @@
 # Copyright 2008-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """The standard model library included with Lino.
 
 A collection of ready-to-use plugins (Django calls them "apps") for
@@ -31,7 +30,6 @@ Front ends
 .. autosummary::
    :toctree:
 
-    bootstrap3
     extjs
 
 Optional features
@@ -56,8 +54,6 @@ Communication
 
     comments
     notify
-    changes
-    linod
 
 Enterprise Resources
 ====================
@@ -69,7 +65,5 @@ Enterprise Resources
     languages
     office
     smtpd
-    uploads
-
 
 """
