@@ -1,15 +1,14 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2014-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the :class:`Dupable` model mixin and related functionality
 to assist users in finding unwanted duplicate database records.
 
-Don't mix up this module with :mod:`lino.mixins.duplicable`.  Models
-are "duplicable" if users may *want* to duplicate some instance
+Don't mix up this module with :mod:`lino.mixins.`.  Models
+are "" if users may *want* to duplicate some instance
 thereof, while "dupable" implies that the duplicates are *unwanted*.
 To dupe *somebody* means "to make a dupe of; deceive; delude; trick."
-(`reference.com <http://dictionary.reference.com/browse/dupe>`_), and
+(`reference.com <https://dictionary.reference.com/browse/dupe>`_), and
 to dupe *something* means to duplicate it (eventually in order to
 cheat somebody e.g. by making a cheap copy of a valuable object).
 
@@ -24,25 +23,22 @@ and the `Double Metaphone
 <https://en.wikipedia.org/wiki/Metaphone#Double_Metaphone>`_
 algorithm.  Read also Doug Hellmann about `Using Fuzzy Matching to
 Search by Sound with Python
-<http://www.informit.com/articles/article.aspx?p=1848528>`_
+<https://www.informit.com/articles/article.aspx?p=1848528>`_
 (2012-03-22).
 
 """
 
-from __future__ import unicode_literals
-import six
-from builtins import object
-
 from django.db import models
 
 from lino.api import dd, _
 from lino.modlib.gfks.mixins import Controllable
 from lino.utils import join_elems
-from etgen.html import E
+from lino.utils.html import E, tostring
 
 from .mixins import Dupable
 from lino.core import constants
 
+
 class PhoneticWord(Controllable):
     """Base class for the table of phonetic words of a given dupable
     model. For every (non-abstract) dupable model there must be a
@@ -53,11 +49,12 @@ class PhoneticWord(Controllable):
     of `PhoneticWordBase`.
 
     """
-    class Meta(object):
+
+    class Meta:
         verbose_name = _("Phonetic word")
         verbose_name_plural = _("Phonetic words")
 
-    allow_cascaded_delete = ['owner']
+    allow_cascaded_delete = ["owner"]
     # quick_search_fields = ['owner__' + dupable_words_field]
 
     word = models.CharField(max_length=100)
@@ -77,57 +74,47 @@ class PhoneticWord(Controllable):
     def reduce_word(cls, s):
         # from metaphone.word import Word
         import metaphone as fuzzy
+
         # fuzzy.DMetaphone does not work with unicode strings, see
         # https://bitbucket.org/yougov/fuzzy/issue/2/fuzzy-support-for-unicode-strings-with
         # dm = fuzzy.doublemetaphone(s.encode('utf8'))
         dm = fuzzy.doublemetaphone(s)
         dms = dm[0] or dm[1]
         if dms is None:
-            return ''
-        if isinstance(dms, six.binary_type):
-            dms = dms.decode('utf8')
+            return ""
+        if isinstance(dms, bytes):
+            dms = dms.decode("utf8")
         return dms
 
-    @classmethod
-    def get_simple_parameters(cls):
-        for p in super(PhoneticWord, cls).get_simple_parameters():
-            yield p
-        yield 'owner_id'
-        yield 'owner_type'
+    # @classmethod
+    # def get_simple_parameters(cls):
+    #     for p in super(PhoneticWord, cls).get_simple_parameters():
+    #         yield p
+    #     yield "owner_id"
+    #     yield "owner_type"
 
 
 class PhoneticWords(dd.Table):
-    model = 'dupable.PhoneticWord'
+    model = "dupable.PhoneticWord"
     required_roles = dd.login_required(dd.SiteAdmin)
 
 
 class WordsByOwner(PhoneticWords):
     "Show the phonetic words for this database record."
+
     required_roles = dd.login_required()
-    master_key = 'owner'
+    master_key = "owner"
     column_names = "word"
-    label = ' ♋ '  # 264B
+    label = " ♋ "  # 264B
 
 
 class SimilarObjects(dd.VirtualTable):
-    """Shows the other objects which are similar to this one."""
+    """Shows the other objects that are similar to this one."""
+
     label = _("Similar objects")
-    display_mode = ((None, constants.DISPLAY_MODE_SUMMARY), )
-    # master = dd.Model
+    default_display_modes = {None: constants.DISPLAY_MODE_SUMMARY}
     master = Dupable
 
-    # class Row:
-
-    #     def __init__(self, master, other):
-    #         self.master = master
-    #         self.other = other
-
-    #     def summary_row(self, ar):
-    #         yield ar.obj2html(self.other)
-
-    #     def __unicode__(self):
-    #         return unicode(self.other)
-
     @classmethod
     def get_data_rows(self, ar):
         mi = ar.master_instance
@@ -135,40 +122,21 @@ class SimilarObjects(dd.VirtualTable):
             return
 
         for o in mi.find_similar_instances(4):
-            # yield self.Row(mi, o)
             yield o
 
     @dd.displayfield(_("Similar record"))
     def similar_record(self, obj, ar):
-        # return ar.obj2html(obj.other)
         return ar.obj2html(obj)
 
     @classmethod
-    def get_table_summary(self, obj, ar):
+    def table_as_summary(self, ar):
         chunks = []
-        for other in ar.spawn(self, master_instance=obj):
+        for other in ar:
             chunks.append(ar.obj2html(other))
-
-        # sar = WordsByOwner.default_action.request_from(
-        #     ar, master_instance=obj)
-        # if sar.get_permission():
-        #     if len(chunks):
-        #         chunks.append(E.br())
-        #     s = getattr(obj, obj.dupable_words_field)
-        #     words = ' '.join(obj.get_dupable_words(s))
-        #     label = "({0})".format(words)
-        #     btn = sar.ar2button(label=label)
-        #     chunks.append(btn)
         if len(chunks):
             chunks.append(E.br())
+        obj = ar.master_instance
         s = getattr(obj, obj.dupable_words_field)
-        words = ' '.join(obj.get_dupable_words(s))
+        words = " ".join(obj.get_dupable_words(s))
         chunks.append("({0})".format(words))
-        return E.p(*join_elems(chunks))
-        # if len(chunks):
-        #     # s = getattr(obj, obj.dupable_words_field)
-        #     # words = ' '.join(obj.get_dupable_words(s))
-        #     # # chunks.append(_("Phonetic words: {0}").format(words))
-        #     # chunks.append(" ({0})".format(words))
-        #     return E.p(*join_elems(chunks))
-        # return ''
+        return tostring(join_elems(chunks))
