@@ -1,23 +1,14 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2019 Rumma & Ko Ltd
+# Copyright 2009-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
-"""Adds the default Lino user interface based on ExtJS.
+"""This is the ExtJS Lino front end for Lino.
 
 It is being automatically included by every Lino application unless
-you disable it (e.g. by overriding your :meth:`get_apps_modifiers
-<lino.core.site.Site.get_apps_modifiers>` or your
-:meth:`get_installed_apps <lino.core.site.Site.get_installed_apps>`
+you disable it (e.g. by overriding your :meth:`get_plugin_modifiers
+<lino.core.site.Site.get_plugin_modifiers>` or your
+:meth:`get_installed_plugins <lino.core.site.Site.get_installed_plugins>`
 method).
 
-When your Lino application uses the ExtJS user interface, then you may
-need a `commercial license from Sencha
-<https://www.sencha.com/store/extjs/>`__. Summary without warranty of
-correctness: you need a commercial license if (1) your application is
-not available under the GPL **and** (2) your site is used by other
-people than the empoyees of the company who is the copyright holder of
-your application.
-
 
 .. autosummary::
    :toctree:
@@ -27,21 +18,18 @@ your application.
 
 """
 
-from __future__ import unicode_literals
-from __future__ import print_function
-
 from lino.api.ad import Plugin
+from lino.core.constants import ICON_NAMES
 from django.utils.translation import gettext_lazy as _
 
 # raise Exception("20160528")
 
 
 class Plugin(Plugin):
-    """Extends :class:`lino.core.plugin.Plugin`.
+    """Extends :class:`lino.core.plugin.Plugin`."""
 
-    """
-
-    needs_plugins = ['lino.modlib.bootstrap3']
+    # needs_plugins = ["lino.modlib.bootstrap5"]
+    needs_plugins = ["lino.modlib.jinja"]
 
     enter_submits_form = False
     """Whether the :kbd:`ENTER` key (or :kbd:`CTRL+ENTER` when in a
@@ -76,7 +64,7 @@ class Plugin(Plugin):
 
     url_prefix = "ext"
 
-    media_name = 'ext-3.3.1'
+    media_name = "ext-3.3.1"
 
     # media_base_url = "http://extjs-public.googlecode.com/" + \
     #                  "svn/tags/extjs-3.3.1/release/"
@@ -98,17 +86,22 @@ class Plugin(Plugin):
 
     autorefresh_seconds = 0
     # autorefresh_seconds = 60
-    """Number of seconds to wait between two refreshes when autorefresh is
-    activated. Default is 60. Set this to 0 in order to deactivate
-    the autorefresh button.
+    """
+    Number of seconds to wait between two refreshes of the dashboard. Default is
+    0, which deactivates the feature.
+
+    Be careful when uing this feature on a production site because it causes
+    every connected client to refresh their dashboard every x seconds,
+    independently of whether the window is visible or not.
 
     """
 
-    ui_handle_attr_name = 'extjs_handle'
+    ui_handle_attr_name = "extjs_handle"
 
     def on_ui_init(self, kernel):
         # logger.info("20140227 extjs.Plugin.on_ui_init() a")
         from .ext_renderer import ExtRenderer
+
         self.renderer = ExtRenderer(self)
         kernel.extjs_renderer = self.renderer
 
@@ -120,8 +113,13 @@ class Plugin(Plugin):
 
     def get_row_edit_lines(self, e, panel):
         from lino.core.elems import (
-            GridElement, HtmlBoxElement, FieldElement, form_field_name)
+            GridElement,
+            HtmlBoxElement,
+            FieldElement,
+            form_field_name,
+        )
         from lino.core import constants
+
         master_field = panel.layout_handle.layout._datasource.master_field
         if isinstance(e, GridElement):
             yield "%s.on_master_changed();" % e.as_ext()
@@ -138,29 +136,33 @@ class Plugin(Plugin):
                 if master_field:
                     yield "var bp = this.get_base_params();"
                     yield "%s.setContextValue('%s',bp['%s']);" % (
-                        e.as_ext(), constants.URL_PARAM_MASTER_PK,
-                        constants.URL_PARAM_MASTER_PK)
+                        e.as_ext(),
+                        constants.URL_PARAM_MASTER_PK,
+                        constants.URL_PARAM_MASTER_PK,
+                    )
                     yield "%s.setContextValue('%s',bp['%s']);" % (
-                        e.as_ext(), constants.URL_PARAM_MASTER_TYPE,
-                        constants.URL_PARAM_MASTER_TYPE)
+                        e.as_ext(),
+                        constants.URL_PARAM_MASTER_TYPE,
+                        constants.URL_PARAM_MASTER_TYPE,
+                    )
                 if not master_field or master_field.name != f.name:
                     yield (
                         "%s.setContextValue('%s', record ? record."
-                        "data['%s'] : undefined);" % (
-                            e.as_ext(), f.name, form_field_name(f)))
+                        "data['%s'] : undefined);"
+                        % (e.as_ext(), f.name, form_field_name(f))
+                    )
 
     def get_css_includes(self, site):
-        yield self.build_lib_url('resources/css/ext-all.css')
+        yield self.build_lib_url("resources/css/ext-all.css")
 
     def get_js_includes(self, settings, language):
         return []
 
     def get_head_lines(self, site, request):
         yield "<style>"
-        from lino.core.constants import ICON_NAMES
         tpl = ".x-tbar-{0}{{ background-image: url({1}) !important; }}"
         for n in ICON_NAMES:
-            url = site.build_static_url('images', 'mjames', n + '.png')
+            url = site.build_static_url("images", "mjames", n + ".png")
             yield tpl.format(n, url)
 
         yield """
@@ -176,14 +178,14 @@ class Plugin(Plugin):
             onclick = "alert('ExtJS client version is ' + Ext.version);"
             tip = "Click to see ExtJS client version"
             text = "(version)"
-            version = html.a(text, href='#', onclick=onclick, title=tip)
+            version = html.a(text, href="#", onclick=onclick, title=tip)
             yield ("ExtJS", version, "http://www.sencha.com")
 
-            yield ("Silk Icons", '1.3',
-                   "http://www.famfamfam.com/lab/icons/silk/")
+            yield ("Silk Icons", "1.3", "http://www.famfamfam.com/lab/icons/silk/")
 
     def get_index_view(self):
         from . import views
+
         return views.AdminIndex.as_view()
 
     # def get_detail_url(self, ar, actor, pk, *args, **kw):
@@ -205,45 +207,55 @@ class Plugin(Plugin):
     #         #     self.site.server_url,
     #         #     actor.app_label, actor.__name__, pk)
 
-
     def get_patterns(self):
         # this is called after startup.
         # print("20221102 get_patterns()")
         from django.conf import settings
+
         # django.conf.urls.url() is deprecated in favor of django.urls.re_path
         # from django.conf.urls import url  # patterns
         from django.urls import re_path as url
         from . import views
 
-        # if self.site.developer_site_cache:
-        #     self.renderer.build_site_cache()
-
-        rx = '^'
+        rx = "^"
 
         urlpatterns = [
             # url(rx + '/?$', views.AdminIndex.as_view()),
-            url(rx + '$', views.AdminIndex.as_view()),
-            url(rx + r'api/main_html$', views.MainHtml.as_view()),
-            url('service-worker.js', views.SWView.as_view()),
+            url(rx + "$", views.AdminIndex.as_view()),
+            url(rx + r"api/main_html$", views.MainHtml.as_view()),
+            url("service-worker.js", views.SWView.as_view()),
             # url(rx + r'auth$', views.Authenticate.as_view()),
             # url(rx + r'grid_config/(?P<app_label>\w+)/(?P<actor>\w+)$',
             #     views.GridConfig.as_view()),
-            url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)$',
-                views.ApiList.as_view()),
-            url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>.+)$',
-                views.ApiElement.as_view()),
-            url(rx + r'restful/(?P<app_label>\w+)/(?P<actor>\w+)$',
-                views.Restful.as_view()),
-            url(rx + r'restful/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>.+)$',
-                views.Restful.as_view()),
-            url(rx + r'choices/(?P<app_label>\w+)/(?P<rptname>\w+)$',
-                views.Choices.as_view()),
-            url(rx + r'choices/(?P<app_label>\w+)/(?P<rptname>\w+)/'
-                '(?P<fldname>\w+)$',
-                views.Choices.as_view()),
-            url(rx + r'apchoices/(?P<app_label>\w+)/(?P<actor>\w+)/'
-                '(?P<an>\w+)/(?P<field>\w+)$',
-                views.ActionParamChoices.as_view()),
+            url(
+                rx + r"api/(?P<app_label>\w+)/(?P<actor>\w+)$", views.ApiList.as_view()
+            ),
+            url(
+                rx + r"api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>.+)$",
+                views.ApiElement.as_view(),
+            ),
+            url(
+                rx + r"restful/(?P<app_label>\w+)/(?P<actor>\w+)$",
+                views.Restful.as_view(),
+            ),
+            url(
+                rx + r"restful/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>.+)$",
+                views.Restful.as_view(),
+            ),
+            url(
+                rx + r"choices/(?P<app_label>\w+)/(?P<rptname>\w+)$",
+                views.Choices.as_view(),
+            ),
+            url(
+                rx + r"choices/(?P<app_label>\w+)/(?P<rptname>\w+)/"
+                r"(?P<fldname>\w+)$",
+                views.Choices.as_view(),
+            ),
+            url(
+                rx + r"apchoices/(?P<app_label>\w+)/(?P<actor>\w+)/"
+                r"(?P<an>\w+)/(?P<field>\w+)$",
+                views.ActionParamChoices.as_view(),
+            ),
             # the thread_id can be a negative number:
             # url(rx + r'callbacks/(?P<thread_id>[\-0-9a-zA-Z]+)/'
             #     '(?P<button_id>\w+)$',
@@ -251,9 +263,8 @@ class Plugin(Plugin):
         ]
         if settings.SITE.use_eid_applet:
             urlpatterns.append(
-                url(rx + r'eid-applet-service$',
-                    views.EidAppletService.as_view()))
+                url(rx + r"eid-applet-service$", views.EidAppletService.as_view())
+            )
         if settings.SITE.use_jasmine:
-            urlpatterns.append(
-                url(rx + r'run-jasmine$', views.RunJasmine.as_view()))
+            urlpatterns.append(url(rx + r"run-jasmine$", views.RunJasmine.as_view()))
         return urlpatterns
