@@ -14,9 +14,8 @@ from .fields import GenericForeignKey, GenericForeignKeyIdField
 
 
 class Controllable(dd.Model):
-
     # Translators: will also be concatenated with '(type)' '(object)'
-    owner_label = _('Controlled by')
+    owner_label = _("Controlled by")
 
     controller_is_optional = True
 
@@ -26,39 +25,38 @@ class Controllable(dd.Model):
     owner_type = dd.ForeignKey(
         ContentType,
         editable=True,
-        blank=True, null=True,
+        blank=True,
+        null=True,
         related_name="%(app_label)s_%(class)s_set",
-        verbose_name=format_lazy("{} {}", owner_label, _('(type)')))
+        verbose_name=format_lazy("{} {}", owner_label, _("(type)")),
+    )
 
     owner_id = GenericForeignKeyIdField(
         owner_type,
         editable=True,
-        blank=True, null=True,
-        verbose_name=format_lazy("{} {}", owner_label, _('(object)')))
+        blank=True,
+        null=True,
+        verbose_name=format_lazy("{} {}", owner_label, _("(object)")),
+    )
 
-    owner = GenericForeignKey(
-        'owner_type', 'owner_id',
-        verbose_name=owner_label)
+    owner = GenericForeignKey("owner_type", "owner_id", verbose_name=owner_label)
 
     @classmethod
     def update_controller_field(cls, verbose_name=None, **kwargs):
-
         if verbose_name is not None:
-            dd.update_field(cls, 'owner', verbose_name=verbose_name)
+            dd.update_field(cls, "owner", verbose_name=verbose_name)
             kwargs.update(
-                verbose_name=format_lazy(u"{} {}",
-                    verbose_name, _('(object)')))
-        dd.update_field(cls, 'owner_id', **kwargs)
+                verbose_name=format_lazy("{} {}", verbose_name, _("(object)"))
+            )
+        dd.update_field(cls, "owner_id", **kwargs)
         if verbose_name is not None:
-            kwargs.update(
-                verbose_name=format_lazy(u"{} {}",
-                    verbose_name, _('(type)')))
-        dd.update_field(cls, 'owner_type', **kwargs)
+            kwargs.update(verbose_name=format_lazy("{} {}", verbose_name, _("(type)")))
+        dd.update_field(cls, "owner_type", **kwargs)
 
     def update_owned_instance(self, controllable):
         if self.owner:
             self.owner.update_owned_instance(controllable)
-        super(Controllable, self).update_owned_instance(controllable)
+        super().update_owned_instance(controllable)
 
     def save(self, *args, **kw):
         if settings.SITE.loading_from_dump:
@@ -66,11 +64,18 @@ class Controllable(dd.Model):
         else:
             if self.owner:
                 self.owner.update_owned_instance(self)
-            super(Controllable, self).save(*args, **kw)
+            super().save(*args, **kw)
             if self.owner:
                 self.owner.after_update_owned_instance(self)
 
     def controlled_rows(self, model, **kwargs):
-        gfk = self._meta.get_field('owner')
+        gfk = self._meta.get_field("owner")
         kwargs = gfk2lookup(gfk, self, **kwargs)
         return model.objects.filter(**kwargs)
+
+    @classmethod
+    def get_simple_parameters(cls):
+        for p in super().get_simple_parameters():
+            yield p
+        yield "owner_type"
+        yield "owner_id"
