@@ -11,21 +11,21 @@ from django.contrib.contenttypes.models import ContentType, models
 
 from lino.core.roles import SiteStaff
 from lino.api import dd, rt
-from etgen.html import E
+from lino.utils.html import E
 from lino.utils import join_elems
 
 from lino.core.utils import get_models
 
-
 if VERSION[0] > 2:
     # restore Django 2 behaviour
     def old__ct_str(self):
         return self.name
+
     ContentType.__str__ = old__ct_str
 
 
 class ContentTypes(dd.Table):
-    model = 'contenttypes.ContentType'
+    model = "contenttypes.ContentType"
 
     required_roles = dd.login_required(SiteStaff)
 
@@ -44,20 +44,19 @@ class ContentTypes(dd.Table):
             for b in cl.__bases__:
                 add(b)
             # :
-            if issubclass(cl, dd.Model) and cl is not dd.Model \
-               and cl._meta.managed:
-                if getattr(cl, '_meta', False) and not cl._meta.abstract:
-                    #~ logger.info("20120205 adding(%r)",cl)
+            if issubclass(cl, dd.Model) and cl is not dd.Model and cl._meta.managed:
+                if getattr(cl, "_meta", False) and not cl._meta.abstract:
+                    # ~ logger.info("20120205 adding(%r)",cl)
                     ct = ContentType.objects.get_for_model(cl)
-                    chunks.append(
-                        ar.obj2html(ct, str(cl._meta.verbose_name)))
-        #~ add(obj.model_class())
+                    chunks.append(ar.obj2html(ct, str(cl._meta.verbose_name)))
+
+        # ~ add(obj.model_class())
         cl = obj.model_class()
         # e.g. if database is nor synchronized
         if cl is not None:
             for b in cl.__bases__:
                 add(b)
-        return E.p(*join_elems(chunks, sep=', '))
+        return E.p(*join_elems(chunks, sep=", "))
 
 
 # class HelpText(dd.Model):
@@ -123,6 +122,7 @@ class ContentTypes(dd.Table):
 #     master_key = 'content_type'
 #
 
+
 class BrokenGFKs(dd.VirtualTable):
     label = _("Broken GFKs")
     required_roles = dd.login_required(SiteStaff)
@@ -156,7 +156,7 @@ class BrokenGFKs(dd.VirtualTable):
 
 
 class BrokenGFKsByModel(BrokenGFKs):
-    master = 'contenttypes.ContentType'
+    master = "contenttypes.ContentType"
 
     column_names = "database_object message todo"
 
@@ -170,7 +170,7 @@ class BrokenGFKsByModel(BrokenGFKs):
 
     @classmethod
     def get_pk_field(self):
-        return rt.models.system.SiteConfig._meta.get_field('id')
+        return rt.models.system.SiteConfig._meta.get_field("id")
         # return settings.SITE.site_config._meta.get_field('id')
 
     @classmethod
@@ -208,7 +208,6 @@ class BrokenGFKsByModel(BrokenGFKs):
 #         dd.logger.debug("No help texts : %s", e)
 #         pass
 
-
 # cause `inv mm` to generate translatable strings from Django's
 # original module since those translations are not loaded.
 _("content type")
