@@ -1,35 +1,29 @@
 # -*- coding: utf-8 -*-
+# fmt: off
+
 from pathlib import Path
-import synodal
 
 docs_path = Path('../docs').resolve()
 
 templates_path = []  # will be populated by lino.sphinxcontrib.configure()
-
 intersphinx_mapping = {}
-for r in synodal.REPOS_LIST:
-    if r.public_url and r.nickname:
-        # if r.nickname == "welfare": continue
-        intersphinx_mapping[r.nickname] = (r.public_url, None)
 
 {% if makehelp.language.index == 0 -%}
 
-html_context = dict(public_url="{{settings.SITE.server_url}}media/cache/help")
+html_context = dict(public_url="{{settings.SITE.server_url}}/media/cache/help")
 
 from rstgen.sphinxconf import configure ; configure(globals())
 from lino.sphinxcontrib import configure ; configure(globals())
 
-# print("20230314", intersphinx_mapping)
-
 project = "{{settings.SITE.title}}"
 html_title = "{{settings.SITE.title}}"
 
-{% if settings.SITE.site_config.site_company %}
+{% if settings.SITE.plugins.contacts.site_owner %}
 import datetime
-copyright = "{} {{settings.SITE.site_config.site_company}}".format(
+copyright = "{} {{settings.SITE.plugins.contacts.site_owner}}".format(
     datetime.date.today())
 {% endif %}
-htmlhelp_basename = 'help'
+
 extensions += ['lino.sphinxcontrib.logo']
 
 {% else -%}{# elif makehelp.language.index == 0 -#}
