@@ -25,7 +25,8 @@ from django.apps import apps
 # from atelier.projects import Project
 
 import lino
-#from lino.core.utils import app_labels
+
+# from lino.core.utils import app_labels
 from lino.utils import curry
 import rstgen
 from rstgen.utils import cd
@@ -40,12 +41,15 @@ from lino.core.model import Model
 from lino.core.utils import model_class_path
 from lino.modlib.help.utils import HelpTextsLoader, simplify_name
 from lino.modlib.gfks.fields import GenericForeignKey
-from lino.api.dd import full_model_name
-from lino.api import doctest
+from lino.api import dd
+
+# removed import doctest because it caused "pytest not installed" during
+# makehelp on LF:
+# from lino.api import doctest
 
 use_dirhtml = False
 
-include_useless = settings.SITE.get_plugin_setting('help', 'include_useless')
+include_useless = settings.SITE.get_plugin_setting("help", "include_useless")
 
 LAYOUT_HTML = """
 {% extends "!layout.html" %}
@@ -65,6 +69,7 @@ LAYOUT_HTML = """
 {% block relbar2 %}{% endblock %}
 """
 
+
 def runcmd(cmd, **kw):
     """Run the specified command in a subprocess.
 
@@ -81,77 +86,98 @@ def runcmd(cmd, **kw):
     cp = subprocess.run(cmd, **kw)
     if cp.returncode != 0:
         # subprocess.run("sudo journalctl -xe", **kw)
-        raise Exception(
-        "{} ended with return code {}".format(cmd, cp.returncode))
-
+        raise Exception("{} ended with return code {}".format(cmd, cp.returncode))
 
 
 def field_ref(f):
     if isinstance(f.model, Model):
-        return ":ref:`{}.{}`".format(full_model_name(f.model), f.name)
+        return ":ref:`{}.{}`".format(dd.full_model_name(f.model), f.name)
     # parameter field
     return ":ref:`{}.{}`".format(str(f.model), f.name)
 
+
 def report_ref(rpt):
     return ":doc:`{}`".format(str(rpt))
     # return settings.SITE.server_url + '.' + str(rpt)
-    #~ return ":ref:`%s.%s`" % (settings.SITE.source_name,str(rpt))
+    # ~ return ":ref:`%s.%s`" % (settings.SITE.source_name,str(rpt))
+
 
 def model_ref(model):
-    return settings.SITE.source_name + '.' + model._meta.app_label + '.' + model.__name__
+    return (
+        settings.SITE.source_name + "." + model._meta.app_label + "." + model.__name__
+    )
+
 
 def verbose_name(f):
     return str(f.verbose_name or "(None)")
 
-def shortpar(name='', label='', text=''):
+
+def shortpar(name="", label="", text=""):
     label = str(label).strip() or _("(no label)")
     text = str(text).strip()
     name = str(name).strip()
     return doc2rst(f"**{label}** ({name}) : {text}")
 
+
 def rubric(s):
     return "\n\n.. rubric:: {}\n\n".format(s)
 
+
 def rptlist(l):
-    return ', '.join([report_ref(rpt) for rpt in sorted(l, key=str)])
-        # ":doc:`%s (%s) <%s>`" % (str(rpt),
-        #                          force_str(rpt.label), report_ref(rpt))
-        # for rpt in l])
+    return ", ".join([report_ref(rpt) for rpt in sorted(l, key=str)])
+    # ":doc:`%s (%s) <%s>`" % (str(rpt),
+    #                          force_str(rpt.label), report_ref(rpt))
+    # for rpt in l])
+
 
 def model_referenced_from(model):
-    #~ headers = ["name","description"]
-    #~ rows = []
+    # ~ headers = ["name","description"]
+    # ~ rows = []
     def ddhfmt(ddh):
-        return ', '.join(['{}.{}'.format(full_model_name(model), fk.name)
-                          for model, fk in ddh.fklist])
+        return ", ".join(
+            [
+                "{}.{}".format(dd.full_model_name(model), fk.name)
+                for model, fk in ddh.fklist
+            ]
+        )
+
     return ddhfmt(model._lino_ddh)
-    #~ rows.append(['_lino_ddh',ddhfmt(model._lino_ddh)])
-    #~ return rstgen.table(headers,rows)
+    # ~ rows.append(['_lino_ddh',ddhfmt(model._lino_ddh)])
+    # ~ return rstgen.table(headers,rows)
 
 
 class GeneratingCommand(BaseCommand):
-
     def add_arguments(self, parser):
-        parser.add_argument('-t', '--tmpdir', action='store',
-                            dest='tmpdir', default=None,
-                            help='Path for temporary files.')
-        parser.add_argument('-l', '--language', action='store',
-                            dest='language', default=None,
-                            help='Generate only the specified language.')
+        parser.add_argument(
+            "-t",
+            "--tmpdir",
+            action="store",
+            dest="tmpdir",
+            default=None,
+            help="Path for temporary files.",
+        )
+        parser.add_argument(
+            "-l",
+            "--language",
+            action="store",
+            dest="language",
+            default=None,
+            help="Generate only the specified language.",
+        )
 
     def handle(self, *args, **options):
-
         if settings.SITE.master_site:
             settings.SITE.logger.info(
-                "No need to `makehelp` on slave site '%s'.", settings.SETTINGS_MODULE)
+                "No need to `makehelp` on slave site '%s'.", settings.SETTINGS_MODULE
+            )
             return
 
         self.options = options
-        self.output_dir = settings.SITE.media_root / 'cache' / 'help'
+        self.output_dir = settings.SITE.media_root / "cache" / "help"
         self.generated_count = 0
         self.htl = HelpTextsLoader(settings.SITE)
 
-        tmpdir = options['tmpdir']
+        tmpdir = options["tmpdir"]
         if tmpdir:
             self.run_on_temp_dir(tmpdir)
         else:
@@ -159,44 +185,47 @@ class GeneratingCommand(BaseCommand):
                 self.run_on_temp_dir(tmpdirname)
 
     def run_on_temp_dir(self, temp_dir):
-        verbosity = self.options.get('verbosity', 0)
+        verbosity = self.options.get("verbosity", 0)
         self.temp_dir = temp_dir
         logger.info("Generating temporary Sphinx files to %s", self.temp_dir)
         self.generate_files()
-        logger.info("Generated %s temporary Sphinx files to %s", self.generated_count, self.temp_dir)
+        logger.info(
+            "Generated %s temporary Sphinx files to %s",
+            self.generated_count,
+            self.temp_dir,
+        )
         logger.info("Building site help html to %s", self.output_dir)
 
         def doit(lng):
             self.language = lng
             docs_dir = self.docspath()
-            builder = 'html'
+            builder = "html"
             if use_dirhtml:
-                builder = 'dirhtml'
-            args = ['sphinx-build', '-b', builder]
-            args += ['-T'] # show full traceback on exception
+                builder = "dirhtml"
+            args = ["sphinx-build", "-b", builder]
+            args += ["-T"]  # show full traceback on exception
             # ~ args += ['-a'] # all files, not only outdated
             # ~ args += ['-P'] # no postmortem
             if not verbosity:
-                args += ['-Q'] # no output
-            args += ['.']
+                args += ["-Q"]  # no output
+            args += ["."]
             if lng.index == 0:
                 args += [str(self.output_dir)]
             else:
                 args += [str(self.output_dir / lng.django_code)]
-            cmd = ' '.join(args)
+            cmd = " ".join(args)
             print("Run `cd {} && {}`".format(docs_dir, cmd))
             with cd(docs_dir):
                 runcmd(cmd)
 
         self.with_languages(doit)
 
-
     def docspath(self, output_file=None):
         parts = [self.temp_dir]
         if self.language.index == 0:
-            parts.append('docs')
+            parts.append("docs")
         else:
-            parts.append(self.language.django_code + 'docs')
+            parts.append(self.language.django_code + "docs")
         if output_file:
             parts.append(output_file)
         return os.path.join(*parts)
@@ -204,21 +233,20 @@ class GeneratingCommand(BaseCommand):
     def generate(self, tplname, output_file, **context):
         output_file = self.docspath(output_file)
         logger.info("Generating %s", output_file)
-        #~ logger.info("Generating %s from %s",fn,tpl_filename)
+        # ~ logger.info("Generating %s from %s",fn,tpl_filename)
         env = settings.SITE.plugins.jinja.renderer.jinja_env
         template = env.get_template(tplname)
         context.update(self.context)
         content = template.render(**context)
-        open(output_file, 'wt').write(content)
+        open(output_file, "wt").write(content)
         self.generated_count += 1
-        return ''
+        return ""
 
 
 class Command(GeneratingCommand):
     help = "Generate the local help pages for this Lino site."
 
     def generate_files(self):
-
         self.context = dict(
             header=rstgen.header,
             h1=curry(rstgen.header, 1),
@@ -227,17 +255,19 @@ class Command(GeneratingCommand):
             models=models,
             abstract=abstract,
             refto=self.refto,
+            repr=repr,
             settings=settings,
             actors=actors,
             # actors_list=[a for a in actors.actors_list if not a.abstract],
-            doctest=doctest,
+            # doctest=doctest,
             translation=translation,
             use_dirhtml=use_dirhtml,
             include_useless=include_useless,
-            #~ py2rst=rstgen.py2rst,
+            # ~ py2rst=rstgen.py2rst,
             languages=[lng.django_code for lng in settings.SITE.languages],
             get_models=apps.get_models,
-            full_model_name=full_model_name,
+            full_model_name=dd.full_model_name,
+            dd=dd,
             model_overview=self.model_overview,
             actor2par=self.actor2par,
             actors2table=self.actors2table,
@@ -247,7 +277,7 @@ class Command(GeneratingCommand):
             model_ref=model_ref,
             makehelp=self,
         )
-        self.context['_'] = _
+        self.context["_"] = _
 
         # from synodal import REPOS_DICT, REPOS_LIST
         # myrepo = None
@@ -258,24 +288,24 @@ class Command(GeneratingCommand):
         def doit(lng):
             self.language = lng
 
-            tplpath = Path(self.docspath(), '.templates')
+            tplpath = Path(self.docspath(), ".templates")
             tplpath.mkdir(parents=True, exist_ok=True)
-            tplpath /= 'layout.html'
+            tplpath /= "layout.html"
             tplpath.write_text(LAYOUT_HTML)
             self.generated_count += 1
 
-            self.generate('makehelp/conf.tpl.py', 'conf.py')
-            self.generate('makehelp/index.tpl.rst', 'index.rst')
-            self.generate('makehelp/copyright.tpl.rst', 'copyright.rst')
-            self.generate('makehelp/actors.tpl.rst', 'actors.rst')
+            self.generate("makehelp/conf.tpl.py", "conf.py")
+            self.generate("makehelp/index.tpl.rst", "index.rst")
+            self.generate("makehelp/copyright.tpl.rst", "copyright.rst")
+            self.generate("makehelp/actors.tpl.rst", "actors.rst")
             if include_useless:
-                self.generate('makehelp/models.tpl.rst', 'models.rst')
-                self.generate('makehelp/plugins.tpl.rst', 'plugins.rst')
+                self.generate("makehelp/models.tpl.rst", "models.rst")
+                self.generate("makehelp/plugins.tpl.rst", "plugins.rst")
 
         self.with_languages(doit)
 
     def with_languages(self, doit):
-        lng = self.options.get('language', None)
+        lng = self.options.get("language", None)
         if lng is None:
             for lng in settings.SITE.languages:
                 doit(lng)
@@ -285,34 +315,38 @@ class Command(GeneratingCommand):
 
     def refto(self, x, text=None):
         if x is None:
-            return '`None`'
+            return "`None`"
         if isinstance(x, type):
             if issubclass(x, actors.Actor):
-                if text is None: text = str(x)
+                if text is None:
+                    text = str(x)
                 ref = self.get_help_text_ref(x)
                 if ref:
-                    return ':class:`{} <{}>`'.format(text, ref)
+                    return ":class:`{} <{}>`".format(text, ref)
             elif issubclass(x, models.Model):
-                if text is None: text = x.__name__
-                return ':doc:`' + text + ' <' + full_model_name(x) + '>`'
+                if text is None:
+                    text = x.__name__
+                return ":doc:`" + text + " <" + dd.full_model_name(x) + ">`"
         if isinstance(x, BoundAction):
-            if text is None: text = x.action_name
-            return ':doc:`{} <{}>`'.format(text, x.actor)
+            if text is None:
+                text = x.action_name
+            return ":doc:`{} <{}>`".format(text, x.actor)
         if isinstance(x, (models.Field, fields.FakeField, GenericForeignKey)):
-            if text is None: text = x.verbose_name or x.name
+            if text is None:
+                text = x.verbose_name or x.name
             ref = self.get_help_text_ref(x)
             if ref:
-                return ':attr:`{} <{}>`'.format(text, ref)
-            if getattr(x, 'model', None):
+                return ":attr:`{} <{}>`".format(text, ref)
+            if getattr(x, "model", None):
                 ref = simplify_name(model_class_path(x.model))
-                return ':attr:`{} <{}.{}>`'.format(text, ref, x.name)
-            return ':attr:`{} <oops>`'.format(x.name)
+                return ":attr:`{} <{}.{}>`".format(text, ref, x.name)
+            return ":attr:`{} <oops>`".format(x.name)
         return "(unformatted: ``{}``)".format(repr(x))
-        #~ if isinstance(x,Field):
+        # ~ if isinstance(x,Field):
         # return ':ref:`' + x.verbose_name + ' <' + full_model_name(x.model) + '.' + x.name + '>`'
 
     def get_help_text_ref(self, x):
-        ref = getattr(x, '_lino_help_ref', None)
+        ref = getattr(x, "_lino_help_ref", None)
         if ref:
             return ref
         if not isinstance(x, type):
@@ -350,14 +384,17 @@ class Command(GeneratingCommand):
         if isinstance(e, elems.FieldElement):
             e = e.field
         if isinstance(e, (models.Field, fields.FakeField, GenericForeignKey)):
-            return shortpar(self.refto(e, e.name), e.verbose_name, self.get_help_text_from_field(e))
+            return shortpar(
+                self.refto(e, e.name), e.verbose_name, self.get_help_text_from_field(e)
+            )
         return self.refto(e)
 
     def actors2table(self):
         rows = []
         headers = [_("Name"), _("Description")]
         for a in actors.actors_list:
-            if a.abstract: continue
+            if a.abstract:
+                continue
             ref, text = self.htl.get_help_text_for_class(a)
             label = ":doc:`{} <{}>`".format((a.label or "???").strip(), a)
             name = ":class:`{} <{}>`".format(a, ref)
@@ -373,19 +410,23 @@ class Command(GeneratingCommand):
 
     def action2par(self, a):
         if isinstance(a, BoundAction):
-            a = a.action
+            # a = a.action
+            return shortpar(a.action.action_name, a.action.label,
+                            a.get_help_text() or _("See {}.").format(
+                                self.refto(a.action)))
         return shortpar(a.action_name, a.label, self.get_help_text_from_field(a))
 
     def collect_refs(self, cls, role):
         refs = []
-        if cls is None: return refs
+        if cls is None:
+            return refs
         for i, b in enumerate(cls.mro()):
             if i == 0:
                 continue
             ref = self.get_help_text_ref(b)
             if ref and not ref in refs:
                 refs.append(ref)
-        return [':{}:`{}`'.format(role, ref) for ref in refs]
+        return [":{}:`{}`".format(role, ref) for ref in refs]
 
     def get_intro(self, cls):
         s = ""
@@ -394,8 +435,8 @@ class Command(GeneratingCommand):
         if txt:
             s += "\n\n" + str(txt)
             s += " :class:`{} <{}>`".format(_("(more)"), ref)
-        else:
-            s += "\n\n" + _("No help_text for {}").format(cls)
+        # else:
+        #     s += "\n\n" + _("No help_text for {}").format(cls)
 
         refs = self.collect_refs(cls, "class")
         # refs += self.collect_refs(rpt.model, "class")
@@ -412,7 +453,8 @@ class Command(GeneratingCommand):
             row_ref = self.get_help_text_ref(m)
             if issubclass(m, models.Model):
                 row_label = _("a :class:`{} <{}>`").format(
-                    m._meta.verbose_name, row_ref)
+                    m._meta.verbose_name, row_ref
+                )
                 # s += "\n\n**{}** : {}".format(
                 #     _("Database model"), self.refto(m, full_model_name(m)))
             else:
@@ -421,7 +463,7 @@ class Command(GeneratingCommand):
                 )
                 # s += "\n\n**{}** : {}".format(
                 #     _("Row model"), self.refto(m))
-            msg = _("Every row of this :term:`data view` is {}.")
+            msg = _("Every row of this :term:`data table` is {}.")
             s += "\n\n" + msg.format(row_label, row_ref)
 
         msg = _("User Guide: :ref:`ug.plugins.{}`.")
@@ -447,13 +489,12 @@ class Command(GeneratingCommand):
                 # s += show(_("Columns"), rpt.get_handle().get_columns())
                 s += show(_("Columns"), visible)
             # s += rubric(_("Columns"))
-            # s += rstgen.ul([elem2par(e) for e in rpt.list_layout.walk()])
+            # s += rstgen.ul([elem2par(e) for e in rpt.grid_layout.walk()])
             # s += rstgen.ul([elem2par(f) for f in rpt.wildcard_data_elems()])
 
         # s += show(_("Other fields"),
         #     [e for e in rpt.wildcard_data_elems() if not e in seen])
 
-
         if rpt.detail_action:
             s += show(_("Detail layout"), rpt.get_detail_elems())
             # s += rubric(_("Detail fields"))
@@ -461,7 +502,13 @@ class Command(GeneratingCommand):
 
         s += rubric(_("Toolbar actions"))
         # s += rstgen.ul([action2par(a) for a in rpt.get_toolbar_actions()])
-        s += rstgen.ul([self.action2par(ba) for ba in rpt._actions_list if ba.action.show_in_toolbar])
+        s += rstgen.ul(
+            [
+                self.action2par(ba)
+                for ba in rpt._actions_list
+                if ba.action.show_in_toolbar
+            ]
+        )
         if rpt.parameters:
             s += show(_("Filter parameters"), rpt.parameters.values())
             # s += rubric(_("Filter parameters"))
@@ -475,7 +522,7 @@ class Command(GeneratingCommand):
         masters = [r for r in kernel.master_tables if r.model is model]
         if masters:
             s += "\n\n**{}** : {}".format(_("Master tables"), rptlist(masters))
-        slaves = getattr(model, '_lino_slaves', None)
+        slaves = getattr(model, "_lino_slaves", None)
         if slaves:
             s += "\n\n**{}** : {}".format(_("Slave tables"), rptlist(slaves.values()))
 
@@ -483,7 +530,7 @@ class Command(GeneratingCommand):
         for fm, fk in model._lino_ddh.fklist:
             pointers.append(self.refto(fk))
         if len(pointers):
-            s += "\n\n**{}** : {}".format(_("Pointers"), ', '.join(pointers))
+            s += "\n\n**{}** : {}".format(_("Pointers"), ", ".join(pointers))
 
         # s += rubric("Database fields:")
         # s += rstgen.ul([self.field2par(f) for f in model._meta.fields])
