@@ -1,12 +1,10 @@
 # Copyright 2010-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
 from importlib import import_module
 
+simplify_parts = set(["models", "desktop", "ui", "choicelists", "actions", "mixins"])
 
-simplify_parts = set(
-    ['models', 'desktop', 'ui', 'choicelists', 'actions', 'mixins'])
 
 def simplify_name(name):
     """
@@ -20,15 +18,14 @@ def simplify_name(name):
     """
     if name.startswith("lino.mixins."):
         return name
-    parts = name.split('.')
+    parts = name.split(".")
     for e in simplify_parts:
         if e in parts:
             parts.remove(e)
-    return '.'.join(parts)
+    return ".".join(parts)
 
 
 class HelpTextsLoader:
-
     _help_texts = dict()
 
     def __init__(self, site):
@@ -37,7 +34,7 @@ class HelpTextsLoader:
     def load_help_texts(self, site):
         """Collect :xfile:`help_texts.py` modules"""
         for p in site.installed_plugins:
-            mn = p.app_name + '.help_texts'
+            mn = p.app_name + ".help_texts"
             try:
                 m = import_module(mn)
                 # print("20160725 Loading help texts from", mn)
@@ -46,11 +43,11 @@ class HelpTextsLoader:
                 pass
 
     def get_help_text_for_class(self, m, attrname=None):
-        k = m.__module__ + '.' + m.__name__
+        k = m.__module__ + "." + m.__name__
         k = simplify_name(k)
         # debug = k.startswith('users')
         if attrname:
-            k += '.' + attrname
+            k += "." + attrname
         txt = self._help_texts.get(k, None)
         return (k, txt)
 
@@ -70,7 +67,7 @@ class HelpTextsLoader:
         #     raise Exception("{} : {}".format(repr(fld), e))
         # debug = cls.__name__ == "User"
         # debug = True
-        if not hasattr(fld, 'help_text'):
+        if not hasattr(fld, "help_text"):
             # e.g. models or plugins don't have a help_text attribute
             if debug:
                 print("20170824 {!r} has no help_text".format(fld))
@@ -82,12 +79,17 @@ class HelpTextsLoader:
             # if m in self.unhelpful_classes:
             #     continue
             k, txt = self.get_help_text_for_class(m, attrname)
-            # if attrname == "update_missing_rates":
-            #     print("20181004 {} {} {}".format(cls, k, txt))
+            # if attrname == "update_guests":
+            #     # if str(cls) == "cal.Events":
+            #     # if k == "lino_xl.lib.cal.Events.update_guests":
+            #     print(f"20250622 {hash(fld)} {cls} {k} {fld.help_text} {txt}")
             if txt is None:
                 if debug:
-                    print("20170824 {}.{} : no help_text using {!r}".format(
-                        cls, attrname, k))
+                    print(
+                        "20170824 {}.{} : no help_text using {!r}".format(
+                            cls, attrname, k
+                        )
+                    )
                 if fld.help_text:
                     # hard-coded help text gets overridden only if docs
                     # provide a more specific help text.
@@ -97,11 +99,20 @@ class HelpTextsLoader:
                 if debug:
                     # from lino.api import dd
                     # dd.logger.info("20200818 site.py %s", fld.__hash__())
-                    print("20170824 {}.{}.help_text {!r} found using {} --> {}".format(
-                        cls, attrname, txt, k, fld))
+                    print(
+                        "20170824 {}.{}.help_text {!r} found using {} --> {}".format(
+                            cls, attrname, txt, k, fld
+                        )
+                    )
                 fld.help_text = txt
+                # fld._found = True
+                # try:
+                #     fld.help_text = txt
+                # except AttributeError as e:
+                #     raise AttributeError("20240329 {} {}".format(fld, e))
                 fld._lino_help_ref = k  # for makehelp
+                # if attrname == "update_guests":
+                #     print(f"20250622 {fld} {cls} {k} {txt}")
                 return
         if debug:
-            print("20170824 {}.{} : no help_text".format(
-                cls, attrname))
+            print("20170824 {}.{} : no help_text".format(cls, attrname))
