@@ -1,6 +1,5 @@
 # Copyright 2008-2015 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Import filters
 """
@@ -12,15 +11,16 @@ from django.utils.translation import gettext_lazy as _
 
 class Plugin(ad.Plugin):
     "See :doc:`/dev/plugins`."
+
     verbose_name = _("Import filters")
 
-    def setup_config_menu(config, site, user_type, m):
+    def setup_config_menu(config, site, user_type, m, ar=None):
         p = site.plugins.importfilters
-        m = m.add_menu('filters', p.verbose_name)
-        m.add_action('importfilters.Filters')
-        m.add_action('importfilters.Import')
+        m = m.add_menu("filters", p.verbose_name)
+        m.add_action("importfilters.Filters")
+        m.add_action("importfilters.Import")
 
-    def setup_explorer_menu(config, site, user_type, m):
+    def setup_explorer_menu(config, site, user_type, m, ar=None):
         p = site.plugins.importfilters
-        m = m.add_menu('filters', p.verbose_name)
-        m.add_action('importfilters.Filters')
+        m = m.add_menu("filters", p.verbose_name)
+        m.add_action("importfilters.Filters")
