@@ -1,6 +1,5 @@
 # Copyright 2009-2015 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Database models for `lino.modlib.importfilter`.
 
 """
@@ -12,11 +11,9 @@ from django.contrib.contenttypes import models as contenttypes
 from django.contrib.contenttypes.models import ContentType
 from django.utils.encoding import force_str
 
-
 from django.db import models
 from django.utils.translation import gettext_lazy as _
 
-
 from lino.api import dd, rt
 
 from lino.utils.instantiator import Instantiator
@@ -28,11 +25,9 @@ class FieldSeparators(dd.ChoiceList):
 
 class Filter(dd.Model):
     name = models.CharField(_("Name"), max_length=200)
-    content_type = dd.ForeignKey(contenttypes.ContentType,
-                                 verbose_name=_("Model"))
+    content_type = dd.ForeignKey(contenttypes.ContentType, verbose_name=_("Model"))
     field_sep = models.CharField(_("Field separator"), max_length=10)
-    help_text = dd.RichTextField(_("HelpText"),
-                                 blank=True, null=True, format='plain')
+    help_text = dd.RichTextField(_("HelpText"), blank=True, null=True, format="plain")
 
 
 class Filters(dd.Table):
@@ -46,7 +41,6 @@ class Filters(dd.Table):
 
 
 class Item(mixins.Sequenced):
-
     class Meta(object):
         verbose_name = _("Import Filter Item")
         verbose_name_plural = _("Import Filter Items")
@@ -55,8 +49,7 @@ class Item(mixins.Sequenced):
     field = models.CharField(_("Field"), max_length=200)
     column = models.IntegerField(_("Column"))
 
-    help_text = dd.RichTextField(_("HelpText"),
-                                 blank=True, null=True, format='plain')
+    help_text = dd.RichTextField(_("HelpText"), blank=True, null=True, format="plain")
 
     @dd.chooser(simple_values=True)
     def field_choices(cls, filter):
@@ -65,7 +58,7 @@ class Item(mixins.Sequenced):
             model = filter.content_type.model_class()
             meta = model._meta
             for f in meta.fields:
-                if not getattr(f, '_lino_babel_field', False):
+                if not getattr(f, "_lino_babel_field", False):
                     l.append(f.name)
             l.sort()
         return l
@@ -77,17 +70,18 @@ class Items(dd.Table):
 
 
 class ItemsByFilter(Items):
-    master_key = 'filter'
+    master_key = "filter"
     column_names = "column field help_text workflow_buttons *"
 
 
 class Import(dd.VirtualTable):
-    column_names = 'description obj2unicode'
+    column_names = "description obj2unicode"
     parameters = dict(
         filter=dd.ForeignKey(Filter),
-        data=dd.RichTextField(_("Data to import"),
-                              blank=True, null=True,
-                              format='plain'))
+        data=dd.RichTextField(
+            _("Data to import"), blank=True, null=True, format="plain"
+        ),
+    )
 
     @classmethod
     def get_data_rows(self, ar):
@@ -100,7 +94,7 @@ class Import(dd.VirtualTable):
                 cells = flt.field_sep.split(ln)
                 for item in flt.item_set.all():
                     if item.column:
-                        kw[item.field] = cells[item.column-1]
+                        kw[item.field] = cells[item.column - 1]
                 yield build(**kw)
 
     @dd.displayfield(_("Description"))
