@@ -6,6 +6,7 @@ from datetime import timedelta
 from lino.utils import get_client_ip_address
 from lino.api import ad
 
+
 class IPRecord(object):
     def __init__(self, addr):
         self.addr = addr
@@ -16,14 +17,12 @@ class IPRecord(object):
         # self.last_request = None
 
     def __repr__(self):  # just for debugging
-        return "{} {} {}".format(
-            self.addr, self.login_failures, self.blacklisted_since)
+        return "{} {} {}".format(self.addr, self.login_failures, self.blacklisted_since)
 
 
 class Plugin(ad.Plugin):
-
     max_blacklist_time = timedelta(minutes=1)
-    max_failed_auth_per_ip = 4 # Should be set in settings.SITE?
+    max_failed_auth_per_ip = 4  # Should be set in settings.SITE?
     ip_records = {}
 
     def get_ip_record(self, request):
