@@ -2,7 +2,6 @@
 # Copyright 2016-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
 import os
 from copy import copy
 
@@ -14,15 +13,13 @@ from lino.modlib.printing.choicelists import DjangoBuildMethod
 
 
 class JinjaBuildMethod(DjangoBuildMethod):
-
     def build(self, ar, action, elem):
         filename = action.before_build(self, elem)
         if filename is None:
             return
         tpl = self.get_template(action, elem)
-        lang = str(elem.get_print_language()
-                   or translation.get_language())
-                   # or settings.SITE.DEFAULT_LANGUAGE.django_code)
+        lang = str(elem.get_print_language() or translation.get_language())
+        # or settings.SITE.DEFAULT_LANGUAGE.django_code)
         ar = copy(ar)
         ar.renderer = settings.SITE.plugins.jinja.renderer
         # ar.tableattrs = dict()
@@ -32,11 +29,16 @@ class JinjaBuildMethod(DjangoBuildMethod):
             cmd_options = elem.get_build_options(self)
             dd.logger.info(
                 "%s render %s -> %s (%r, %s)",
-                self.name, tpl.lino_template_names, filename, lang, cmd_options)
+                self.name,
+                tpl.lino_template_names,
+                filename,
+                lang,
+                cmd_options,
+            )
             context = elem.get_printable_context(ar)
             html = tpl.render(context)
-            self.html2file(html, filename)
+            self.html2file(html, filename, context)
             return os.path.getmtime(filename)
 
-    def html2file(self, html, filename):
-        raise NotImplementedError()
+    def html2file(self, html, filename, context):
+        open(filename, "w").write(html)
