@@ -1,6 +1,5 @@
 # Copyright 2008-2015 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines a list of languages (the :class:`Language
 <lino.modlib.langguages.models.Language>` model).
 
@@ -28,9 +27,9 @@ from lino import ad, _
 class Plugin(ad.Plugin):
     "See :doc:`/dev/plugins`."
 
-    verbose_name = _("Boards")
+    verbose_name = _("Languages")
 
-    def setup_config_menu(self, site, user_type, m):
+    def setup_config_menu(self, site, user_type, m, ar=None):
         p = self.get_menu_group()
         m = m.add_menu(p.app_label, p.verbose_name)
-        m.add_action('languages.Languages')
+        m.add_action("languages.Languages")
