@@ -4,4 +4,9 @@
 
 from django.conf import settings
 
-CHANNEL_NAME = "linod_" + settings.SITE.site_dir.name
+suffix = "_" + settings.SITE.site_dir.name
+
+CHANNEL_NAME = "linod" + suffix
+BROADCAST_CHANNEL = "broadcast" + suffix
+
+get_channel_name = lambda name: str(name) + suffix
