@@ -1,6 +1,5 @@
 # Copyright 2015-2019 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Adds printing functionality to a Lino application.
 See :doc:`/specs/printing`.
 
@@ -11,9 +10,9 @@ from django.utils.translation import gettext_lazy as _
 
 
 class Plugin(ad.Plugin):
-    "See :doc:`/dev/plugins`."
-
     verbose_name = _("Printing")
+    print_demo_objects = 1
+    with_pdf = False
 
     # needs_plugins = ['lino_xl.lib.appypod']
     # needs_plugins = ['lino.modlib.checkdata']
@@ -21,7 +20,10 @@ class Plugin(ad.Plugin):
     # we don't specify checkdata as needed plugin because checkdata would
     # require users
 
-    def post_site_startup(self, site):
-        super(Plugin, self).post_site_startup(site)
-        site.models.printing.BuildMethods.sort()
+    def get_requirements(self, site):
+        if self.with_pdf:
+            yield "pypdf"
 
+    def post_site_startup(self, site):
+        super().post_site_startup(site)
+        site.models.printing.BuildMethods.sort()
