@@ -1,9 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2023 Rumma & Ko Ltd
+# Copyright 2009-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
-import os
 import datetime
 
 from django.db import models
@@ -12,17 +10,22 @@ from django.utils.translation import gettext_lazy as _
 from django.utils import translation
 from django.utils.timezone import make_aware
 
-from lino.api import rt
+from lino.api import dd, rt
 from lino.modlib.checkdata.choicelists import Checker
 
 from lino.utils.choosers import chooser
 from lino.core.model import Model
-from lino.mixins.duplicable import Duplicable
-
+from lino.mixins.clonable import Clonable
 
 from .choicelists import BuildMethods
-from .actions import (DirectPrintAction, CachedPrintAction,
-                      ClearCacheAction, EditTemplate)
+from .actions import (
+    DirectPrintAction,
+    CachedPrintAction,
+    ClearCache,
+    EditTemplate,
+)
+
+NEEDSPDF_MSG = _("Please set a PDF print method for {}")
 
 
 def weekdays(d):
@@ -30,6 +33,7 @@ def weekdays(d):
         yield d
         d += datetime.timedelta(days=1)
 
+
 def register_build_method(bm):
     # not used.
     BuildMethods.add_item_instance(bm)
@@ -40,10 +44,9 @@ def build_method_choices():
 
 
 class PrintableType(Model):
-
     templates_group = None
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     build_method = BuildMethods.field(blank=True, null=True)
@@ -59,9 +62,7 @@ class PrintableType(Model):
 
     @chooser(simple_values=True)
     def template_choices(cls, build_method):
-        return cls.get_template_choices(
-            build_method,
-            cls.get_template_groups())
+        return cls.get_template_choices(build_method, cls.get_template_groups())
 
     @classmethod
     def get_template_choices(cls, build_method, template_groups):
@@ -70,28 +71,34 @@ class PrintableType(Model):
         if not build_method:
             return []
         return rt.find_template_config_files(
-            build_method.template_ext, *template_groups)
+            build_method.template_ext, *template_groups
+        )
 
 
 class Printable(Model):
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
-    @classmethod
-    def get_printable_demo_objects(cls):
+    print_demo_objects = None
 
-        qs = cls.objects.all()
-        if qs.count() > 0:
-            yield qs[0]
+    @classmethod
+    def get_printable_demo_objects(cls, excerpt_type):
+        limit = cls.print_demo_objects or dd.plugins.printing.print_demo_objects
+        if limit is None:
+            return cls.objects.all()
+        return cls.objects.all()[:limit]
+        # qs = cls.objects.all()
+        # if qs.count() > 0:
+        #     yield qs[0]
 
     @classmethod
     def get_template_group(cls):
         # used by excerpts and printable
-        return cls._meta.app_label + '/' + cls.__name__
+        return cls._meta.app_label + "/" + cls.__name__
 
     def get_body_template(self):
-        return ''
+        return ""
 
     # def get_excerpt_type(self):
     #     "Return the primary ExcerptType for the given model."
@@ -131,90 +138,125 @@ class Printable(Model):
             kw = ar.get_printable_context(**kw)
         kw.update(this=self)  # for backward compatibility
         kw.update(obj=self)  # preferred in new templates
-        kw.update(language=self.get_print_language() or \
-                    translation.get_language())
-                  # settings.SITE.DEFAULT_LANGUAGE.django_code)
+        kw.update(language=self.get_print_language()
+                  or translation.get_language())
+        # settings.SITE.DEFAULT_LANGUAGE.django_code)
         kw.update(site=settings.SITE)
         kw.update(weekdays=weekdays)
         return kw
 
-    def before_printable_build(self, bm):
-        pass
+    def must_build_printable(self, bm):
+        return True
 
 
-class CachedPrintable(Duplicable, Printable):
+class CachedPrintable(Clonable, Printable):
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     do_print = CachedPrintAction()
-    do_clear_cache = ClearCacheAction()
+    do_clear_cache = ClearCache()
     edit_template = EditTemplate()
 
     build_time = models.DateTimeField(
         _("build time"), null=True, editable=False)
-
     build_method = BuildMethods.field(blank=True, null=True)
 
     def full_clean(self, *args, **kwargs):
         if not self.build_method:
             self.build_method = self.get_default_build_method()
-        super(CachedPrintable, self).full_clean(*args, **kwargs)
+        super().full_clean(*args, **kwargs)
 
     def on_duplicate(self, ar, master):
-        super(CachedPrintable, self).on_duplicate(ar, master)
+        super().on_duplicate(ar, master)
         self.build_time = None
         self.build_method = None
 
-    def get_target_name(self):
-        if self.build_time:
-            return self.get_build_method().get_target_name(
-                self.do_print, self)
+    def must_build_printable(self, bm):
+        return self.build_time is None
+
+    # def get_target_name(self):
+    #     if self.build_time:
+    #         return self.get_build_method().get_target_name(self.do_print, self)
 
     def get_build_method(self):
         return self.build_method or self.get_default_build_method()
 
     def get_target_url(self):
-        return self.build_method.get_target_url(
-            self.do_print, self)
+        return self.build_method.get_target_url(self.do_print, self)
+
+    def get_target_file(self):
+        return self.build_method.get_target_file(self.do_print, self)
 
     def get_cache_mtime(self):
         """Return the modification time (a `datetime`) of the generated cache
         file, or `None` if no such file exists.
 
         """
-        filename = self.get_target_name()
+        filename = self.get_target_file().path
         if not filename:
             return None
-        try:
-            t = os.path.getmtime(filename)
-        except OSError:
+        if not filename.exists():
             return None
-        return datetime.datetime.fromtimestamp(t)
+        mtime = datetime.datetime.fromtimestamp(filename.lstat().st_mtime)
+        if settings.USE_TZ:
+            mtime = make_aware(mtime)
+        return mtime
 
     def clear_cache(self):
         self.build_time = None
         self.save()
 
-    def build_target(elem, ar):
-        bm = elem.get_build_method()
-        t = bm.build(ar, elem.__class__.do_print, elem)
+    def build_target(self, ar):
+        bm = self.get_build_method()
+        t = bm.build(ar, self.__class__.do_print, self)
         if t is None:
-            raise Exception("%s : build() returned None?!")
+            raise Exception(f"{self} : build() returned None?!")
+        self.touch_from_timestamp(t)
+
+    def touch_from_timestamp(self, t):
         # t is a file timestamp as returned by os.path.getmtime()
         # expressend as the number of seconds since the epoch.
         t = datetime.datetime.fromtimestamp(t)
         if settings.USE_TZ:
             t = make_aware(t)
-        elem.build_time = t
-        elem.save()
+        self.build_time = t
+        self.save()
+
+
+class MultiCachedPrintable(CachedPrintable):
+
+    class Meta:
+        abstract = True
+
+    def get_printable_children(self):
+        raise NotImplementedError
+
+    def build_target(self, ar):
+        from pypdf import PdfWriter
+        target = self.get_target_file()
+        children = []
+        for obj in self.get_printable_children():
+            bm = obj.get_build_method()
+            if bm.target_ext != ".pdf":
+                raise Warning(NEEDSPDF_MSG.format(obj))
+            if obj.must_build_printable(bm):
+                obj.build_target(ar)
+            children.append(obj.get_target_file().path)
+        if len(children) == 0:
+            raise Warning(_("No content to print for {}").format(self))
+        merger = PdfWriter()
+        for pdf in children:
+            merger.append(pdf)
+        merger.write(target.path)
+        merger.close()
+        self.touch_from_timestamp(target.path.stat().st_mtime)
 
 
 class TypedPrintable(CachedPrintable):
-
     type = None
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     def get_printable_type(self):
@@ -223,28 +265,28 @@ class TypedPrintable(CachedPrintable):
     def get_template_groups(self):
         ptype = self.get_printable_type()
         if ptype is None:
-            return super(TypedPrintable, self).get_template_groups()
+            return super().get_template_groups()
         return ptype.get_template_groups()
 
     def get_default_build_method(self):
         ptype = self.get_printable_type()
         if ptype and ptype.build_method:
             return ptype.build_method
-        return super(TypedPrintable, self).get_default_build_method()
+        return super().get_default_build_method()
 
     # def get_build_method(self):
     #     if not self.build_method:
     #         return self.get_default_build_method()
     #     return self.build_method
-        # ptype = self.get_printable_type()
-        # if ptype and ptype.build_method:
-        #     return ptype.build_method
-        # return super(TypedPrintable, self).get_build_method()
+    # ptype = self.get_printable_type()
+    # if ptype and ptype.build_method:
+    #     return ptype.build_method
+    # return super().get_build_method()
 
     def get_print_templates(self, bm, action):
         ptype = self.get_printable_type()
         if ptype is None:
-            return super(TypedPrintable, self).get_print_templates(bm, action)
+            return super().get_print_templates(bm, action)
 
         if ptype.template:
             return [ptype.template]
@@ -255,17 +297,17 @@ class CachedPrintableChecker(Checker):
     model = CachedPrintable
     verbose_name = _("Check for missing target files")
 
-    def get_checkdata_problems(self, obj, fix=False):
-        if obj.build_time is not None:
-            t = obj.get_cache_mtime()
-            if t is None:
-                msg = _("Seems to have been built ({bt}), "
-                        "but cache file is missing .")
-                params = dict(bt=obj.build_time)
-                yield (False, msg.format(**params))
-                # if fix:
-                #     obj.build_time = None
-                #     obj.full_clean()
-                #     obj.save()
+    def get_checkdata_problems(self, ar, obj, fix=False):
+        if obj.build_method is None:
+            return
+        if (mtime := obj.get_cache_mtime()) != obj.build_time:
+            msg = _("Build time is {btime} but should be {mtime}.")
+            params = dict(btime=obj.build_time, mtime=mtime)
+            yield (True, msg.format(**params))
+            if fix:
+                obj.build_time = mtime
+                obj.full_clean()
+                obj.save()
+
 
 CachedPrintableChecker.activate()
