@@ -5,6 +5,7 @@
 from django.db.models import ForeignKey
 from lino.core.choicelists import ChoiceListField, ChoiceList
 
+
 def get_serializer_for_model(Model, options={}):
     from rest_framework import serializers
     from .fields import ChoiceField
@@ -18,21 +19,21 @@ def get_serializer_for_model(Model, options={}):
             clfs.append(fld)
 
     meta_attrs = {
-        'model': Model,
+        "model": Model,
     }
 
-    if options.get('fields', None) is not None:
-        meta_attrs['fields'] = options['fields']
+    if options.get("fields", None) is not None:
+        meta_attrs["fields"] = options["fields"]
     else:
-        meta_attrs['fields'] = '__all__'
+        meta_attrs["fields"] = "__all__"
 
-    attrs_dict = dict(Meta=type('Meta', (type, ), meta_attrs))
+    attrs_dict = dict(Meta=type("Meta", (type,), meta_attrs))
 
-    if options.get('extra_fields', None) is not None:
-        attrs_dict.update(options['extra_fields'])
+    if options.get("extra_fields", None) is not None:
+        attrs_dict.update(options["extra_fields"])
 
     for fld in clfs:
         attrs_dict[fld.name] = ChoiceField(choicelist=fld.choicelist)
 
-    Serializer = type('Serializer', (serializers.ModelSerializer, ), attrs_dict)
+    Serializer = type("Serializer", (serializers.ModelSerializer,), attrs_dict)
     return Serializer
