@@ -4,14 +4,15 @@
 
 from lino.api import ad, _
 
+
 class Plugin(ad.Plugin):
     verbose_name = _("Summaries")
     start_year = None
     end_year = None
     duration_max_length = 6
 
-    def on_init(self):
+    def pre_site_startup(self, site):
         if self.end_year is None:
-            self.end_year = self.site.today().year
+            self.end_year = site.today().year
         if self.start_year is None:
             self.start_year = self.end_year - 2
