@@ -2,7 +2,28 @@
 # Copyright 2009-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-from etgen.html import E
+from lino.utils.report import EmptyTable
+from lino.core.signals import testcase_setup
+from django.test.signals import setting_changed
+from .mixins import Lockable
+from .choicelists import (
+    YesNo,
+    Genders,
+    PeriodEvents,
+    DurationUnits,
+    Recurrences,
+    Weekdays,
+    DisplayColors
+)
+from lino.modlib.checkdata.choicelists import Checker
+from lino.modlib.printing.choicelists import BuildMethods
+from lino.core.actors import resolve_action
+from lino.core.roles import SiteStaff
+from lino.core.utils import is_devserver
+from lino.core.utils import full_model_name
+from lino.core import actions
+from lino.api import dd, rt
+from lino.utils.html import E
 from django.conf import settings
 from django.utils.encoding import force_str
 
@@ -10,21 +31,12 @@ from django.db import models
 from django.db.utils import DatabaseError
 from django.utils.translation import gettext_lazy as _
 
-from django.apps import apps ; get_models = apps.get_models
+from django.apps import apps
 
-from lino.api import dd, rt
-from lino.core import actions
-from lino.core.utils import full_model_name
-from lino.core.utils import is_devserver
-from lino.core.roles import SiteStaff
-from lino.core.actors import resolve_action
+get_models = apps.get_models
 
-from lino.modlib.printing.choicelists import BuildMethods
-from lino.modlib.checkdata.choicelists import Checker
 
 # import them here to have them on rt.models.system:
-from .choicelists import YesNo, Genders, PeriodEvents, DashboardLayouts, DurationUnits, Recurrences, Weekdays
-from .mixins import Lockable
 
 
 class BuildSiteCache(dd.Action):
@@ -40,21 +52,21 @@ Seems that it worked. Refresh your browser.
 Note that other users might experience side effects because
 of the unexpected .js update, but there are no known problems so far.
 Please report any anomalies.""",
-            alert=_("Success"), clear_site_cache=True)
+            alert=_("Success"),
+            clear_site_cache=True,
+        )
 
 
 if False:
-    class SiteConfigManager(models.Manager):
 
+    class SiteConfigManager(models.Manager):
         def get(self, *args, **kwargs):
             return settings.SITE.site_config
 
 
-
 class SiteConfig(dd.Model):
-
     class Meta(object):
-        abstract = dd.is_abstract_model(__name__, 'SiteConfig')
+        abstract = dd.is_abstract_model(__name__, "SiteConfig")
         verbose_name = _("Site configuration")
 
     config_id = 1
@@ -66,30 +78,22 @@ class SiteConfig(dd.Model):
     This is Lino's equivalent of Django's :setting:`SITE_ID` setting.
     Lino applications don't need ``django.contrib.sites`` (`The
     "sites" framework
-    <https://docs.djangoproject.com/en/5.0/ref/contrib/sites/>`_)
+    <https://docs.djangoproject.com/en/5.2/ref/contrib/sites/>`_)
     because an analog functionality is provided by
     :mod:`lino.modlib.system`.
     """
 
-
-
     if False:
         objects = SiteConfigManager()
         real_objects = models.Manager()
 
     default_build_method = BuildMethods.field(
-        verbose_name=_("Default build method"),
-        blank=True, null=True)
+        verbose_name=_("Default build method"), blank=True, null=True
+    )
 
     simulate_today = models.DateField(
         _("Simulated date"), blank=True, null=True)
 
-    site_company = dd.ForeignKey(
-        "contacts.Company",
-        blank=True, null=True,
-        verbose_name=_("Site owner"),
-        related_name='site_company_sites')
-
     _site_config = None
 
     @classmethod
@@ -98,7 +102,6 @@ class SiteConfig(dd.Model):
 
     @classmethod
     def get_site_config(cls):
-
         site = settings.SITE
 
         if not site._startup_done:
@@ -106,9 +109,9 @@ class SiteConfig(dd.Model):
             return None
 
         if cls._site_config is None:
-            #~ raise Exception(20130301)
-            #~ print '20130320 create _site_config'
-            #~ from lino.core.utils import resolve_model
+            # ~ raise Exception(20130301)
+            # ~ print '20130320 create _site_config'
+            # ~ from lino.core.utils import resolve_model
             # from lino.core.utils import obj2str
             SiteConfig = cls
             try:
@@ -117,10 +120,10 @@ class SiteConfig(dd.Model):
                 #       obj2str(self._site_config, True))
             except (SiteConfig.DoesNotExist, DatabaseError):
                 # e.g. during migrate the SiteConfig maybe doesn't yet exist
-            # except SiteConfig.DoesNotExist:
-            #~ except Exception,e:
+                # except SiteConfig.DoesNotExist:
+                # ~ except Exception,e:
                 kw = dict(id=cls.config_id)
-                #~ kw.update(settings.SITE.site_config_defaults)
+                # ~ kw.update(settings.SITE.site_config_defaults)
                 kw.update(site.site_config_defaults)
                 site_config = SiteConfig(**kw)
                 site_config.full_clean()
@@ -129,15 +132,15 @@ class SiteConfig(dd.Model):
                 # print("20180502 Created SiteConfig {}".format(
                 #     obj2str(self._site_config, True)))
                 # 20120725
-                # polls_tutorial menu selection `Config --> Site Parameters`
+                # polls_tutorial menu selection `Config --> Site configuration`
                 # said "SiteConfig 1 does not exist"
                 # cannot save the instance here because the db table possibly doesn't yet exit.
-                #~ self._site_config.save()
-            cls._site_config.on_startup()
+                # ~ self._site_config.save()
+            # cls._site_config.on_startup()
         return cls._site_config
 
     def __str__(self):
-        return force_str(_("Site Parameters"))
+        return force_str(_("Site configuration"))
 
     def update(self, **kw):
         """
@@ -160,14 +163,16 @@ class SiteConfig(dd.Model):
     #             kw[fld.attname] = getattr(other, fld.attname)
     #     self.update(**kw)
 
-    def on_startup(self):
-        # if not self.site_company:
-        #     raise Exception("20230423")
-        if self.site_company:
-            # print("20230423", self.site_company)
-            settings.SITE.copyright_name = str(self.site_company)
-            if self.site_company.url:
-                settings.SITE.copyright_url = self.site_company.url
+    # def on_startup(self):
+    #     # if not self.site_company:
+    #     #     raise Exception("20230423")
+    #     # if self.site_company:
+    #     site = settings.SITE
+    #     if (owner := site.get_plugin_setting('contacts', 'site_owner')) is not None:
+    #         # print("20230423", self.site_company)
+    #         site.copyright_name = str(owner)
+    #         if owner.url:
+    #             site.copyright_url = owner.url
 
     # def full_clean(self, *args, **kw):
     #     super().full_clean(*args, **kw)
@@ -185,7 +190,10 @@ class SiteConfig(dd.Model):
                     diffs.append(
                         "{}: {} -> {}".format(fld.attname, oldval, newval))
             if len(diffs):
-                print("20220824 Overriding SiteConfig instance (diffs={})".format(diffs))
+                print(
+                    "20220824 Overriding SiteConfig instance (diffs={})".format(
+                        diffs)
+                )
             cls._site_config = self
             # cls._site_config.update_from(self)
             # return
@@ -193,42 +201,42 @@ class SiteConfig(dd.Model):
         super().save(*args, **kw)
         # settings.SITE.clear_site_config()
 
+    @property
+    def site_company(self):
+        # Backwards compatibility after 20250617
+        return settings.SITE.plugins.contacts.site_owner
+
 
 def my_handler(sender, **kw):
     # print("20180502 {} my_handler calls clear_site_config()".format(
     #     settings.SITE))
     settings.SITE.clear_site_config()
-    #~ kw.update(sender=sender)
+    # ~ kw.update(sender=sender)
     # dd.database_connected.send(sender)
-    #~ dd.database_connected.send(sender,**kw)
+    # ~ dd.database_connected.send(sender,**kw)
+
 
-from django.test.signals import setting_changed
-from lino.core.signals import testcase_setup
 setting_changed.connect(my_handler)
 testcase_setup.connect(my_handler)
 dd.connection_created.connect(my_handler)
 models.signals.post_migrate.connect(my_handler)
 
 
-# def dashboard_layouts(cls, k, layout_class, **options):
-#     assert k == "detail_layout"
-#     yield cls.detail_layout
-#     for i in DashboardLayouts.get_list_items():
-#         yield i.main
-
-
 class SiteConfigs(dd.Table):
-
-    model = 'system.SiteConfig'
+    model = "system.SiteConfig"
     required_roles = dd.login_required(SiteStaff)
     hide_navigator = True
     allow_delete = False
     hide_top_toolbar = True
 
-    detail_layout = dd.DetailLayout("""
+    detail_layout = dd.DetailLayout(
+        """
     default_build_method
+    simulate_today
     # lino.ModelsBySite
-    """, window_size=(60, 'auto'))
+    """,
+        window_size=(60, "auto"),
+    )
 
     @classmethod
     def get_default_action(cls):
@@ -236,11 +244,8 @@ class SiteConfigs(dd.Table):
 
     do_build = BuildSiteCache()
 
-from lino.utils.report import EmptyTable
-
 
 class Dashboard(EmptyTable):
-
     # label = _("D")
     hide_navigator = True
     required_roles = set()
@@ -252,31 +257,20 @@ class Dashboard(EmptyTable):
     # notify.MyMessages
     # """
 
-    @classmethod
-    def get_extra_layouts(cls):
-        # count = 0
-        for i in DashboardLayouts.get_list_items():
-            yield i.value, i.main
-            # print("20210530", i.value, repr(i.main))
-        #     count += 1
-        # assert count > 0
-
     @classmethod
     def get_default_action(cls):
         # raise Exception("20210530")
         # return None
         # return actions.ShowExtraDetail(None)
         # cls._bind_action("extra_"+name, a, False)
-        ba = cls.get_action_by_name('extra_default')
+        ba = cls.get_action_by_name("extra_default")
         # assert ba is not None
         return ba
 
     @classmethod
     def get_detail_action(self, ar):
         u = ar.get_user()
-        if u.dashboard_layout:
-            return cls.get_action_by_name('extra_' + u.dashboard_layout.name)
-        return cls.get_action_by_name('extra_default')
+        return cls.get_action_by_name("extra_default")
 
     @dd.htmlbox()
     def welcome_messages(cls, obj, ar=None):
@@ -313,32 +307,31 @@ class Dashboard(EmptyTable):
 #             ba = resolve_action(ql)
 #             Dashboard.define_action(ba.action.action_name=ba)
 
-    # for ql in settings.SITE.get_quicklinks()
-    # for m in get_checkable_models().keys():
-    #     if m is None:
-    #         continue
-    #     assert m is not Problem
-    #     m.define_action(check_data=UpdateMessagesByController(m))
-    #     m.define_action(fix_problems=FixMessagesByController(m))
+# for ql in settings.SITE.get_quicklinks()
+# for m in get_checkable_models().keys():
+#     if m is None:
+#         continue
+#     assert m is not Problem
+#     m.define_action(check_data=UpdateMessagesByController(m))
+#     m.define_action(fix_problems=FixMessagesByController(m))
 
 
 class BleachChecker(Checker):
-
     verbose_name = _("Find unbleached html content")
     model = dd.Model
 
     def get_checkable_models(self):
-
-        for m in super(BleachChecker, self).get_checkable_models():
+        for m in super().get_checkable_models():
             if len(m._bleached_fields):
                 yield m
 
-    def get_checkdata_problems(self, obj, fix=False):
-        t = tuple(obj.fields_to_bleach())
+    def get_checkdata_problems(self, ar, obj, fix=False):
+        t = tuple(obj.fields_to_bleach(save=False))
         if len(t):
-            fldnames = ', '.join([f.name for f, old, new in t])
+            fldnames = tuple([f.name for f, old, new in t])
             yield (True, _("Fields {} have unbleached content.").format(fldnames))
             if fix:
+                # obj.before_ui_save(ar, None)
                 obj.before_ui_save(None, None)
                 obj.full_clean()
                 obj.save()
