@@ -1,18 +1,15 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2015-2018 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """See ;doc:`/specs/tinymce`.
 
 """
 
 from lino.api import ad
 
-
-#TINYMCE_VERSION = '3.4.8'
-#TINYMCE_VERSION = '4.1.10'
-TINYMCE_VERSION = '3.5.11'
-
+# TINYMCE_VERSION = '3.4.8'
+# TINYMCE_VERSION = '4.1.10'
+TINYMCE_VERSION = "3.5.11"
 """Which version of TinyMCE to use.
 
 With 4.1.10, windows containing a TextField don't open, and the JS
@@ -32,12 +29,12 @@ def javascript(url):
 class Plugin(ad.Plugin):
     "See :doc:`/dev/plugins`."
 
-    needs_plugins = ['lino.modlib.office']
+    needs_plugins = ["lino.modlib.office"]
     # needs_plugins = ['lino.modlib.office', 'lino.modlib.extjs']
 
-    site_js_snippets = ['tinymce/tinymce.js']
+    site_js_snippets = ["tinymce/tinymce.js"]
 
-    url_prefix = 'tinymce'
+    url_prefix = "tinymce"
 
     # window_width = 600
     # window_height = 500
@@ -59,26 +56,30 @@ class Plugin(ad.Plugin):
 
     field_buttons = (
         "bold,italic,underline,|,justifyleft,justifycenter,justifyright,|,"
-        "bullist,numlist,|,outdent,indent,|,undo,redo,|,removeformat,template")
+        "bullist,numlist,|,outdent,indent,|,undo,redo,|,removeformat,template"
+    )
     """The toolbar buttons when editing a field inside a detail form."""
 
     window_buttons1 = (
         "save,cancel,|,bold,italic,underline,|,justifyleft,justifycenter,"
         "justifyright,fontselect,fontsizeselect,formatselect,|,"
-        "search,replace")
+        "search,replace"
+    )
     """The first row of toolbar buttons when editing in own window."""
     window_buttons2 = (
         "cut,copy,paste,template,|,bullist,numlist,|,outdent,indent,|,"
         "undo,redo,|,link,unlink,anchor,image,|,code,preview,|,forecolor,"
-        "backcolor")
+        "backcolor"
+    )
     """The second row of toolbar buttons when editing in own window."""
 
     window_buttons3 = (
         "insertdate,inserttime,|,spellchecker,advhr,,removeformat,|,"
-        "sub,sup,|,charmap,emotions,|,tablecontrols")
+        "sub,sup,|,charmap,emotions,|,tablecontrols"
+    )
     """The third row of toolbar buttons when editing in own window."""
 
-    media_name = 'tinymce-' + TINYMCE_VERSION
+    media_name = "tinymce-" + TINYMCE_VERSION
     """Lino currently includes three versions of TinyMCE, but for
     production sites we still use the eldest version 3.4.8.
 
@@ -95,48 +96,57 @@ class Plugin(ad.Plugin):
     def get_used_libs(self, html=False):
         if html is not None:
             yield ("TinyMCE", TINYMCE_VERSION, "http://www.tinymce.com/")
-            yield ("Ext.ux.TinyMCE", '0.8.4', "http://www.byte-force.com")
+            yield ("Ext.ux.TinyMCE", "0.8.4", "http://www.byte-force.com")
 
     def get_js_includes(self, settings, language):
         if settings.SITE.kernel.default_renderer.extjs_version:
-            if TINYMCE_VERSION.startswith('3'):
-                yield self.build_lib_url('tiny_mce.js')
+            if TINYMCE_VERSION.startswith("3"):
+                yield self.build_lib_url("tiny_mce.js")
             else:
-                yield self.build_lib_url('tinymce.min.js')
+                yield self.build_lib_url("tinymce.min.js")
             yield settings.SITE.build_static_url("byteforce", "Ext.ux.TinyMCE.js")
 
     def get_patterns(self):
         from django.urls import re_path as url
+
         # from django.conf.urls import url
         from . import views
 
-        rx = '^'
+        rx = "^"
 
         urlpatterns = [
-            url(rx + r'templates/(?P<app_label>\w+)/'
-                + r'(?P<actor>\w+)/(?P<pk>\w+)/(?P<fldname>\w+)$',
-                views.Templates.as_view()),
-            url(rx + r'templates/(?P<app_label>\w+)/'
-                + r'(?P<actor>\w+)/(?P<pk>\w+)/(?P<fldname>\w+)/'
-                + r'(?P<tplname>\w+)$',
-                views.Templates.as_view())]
+            url(
+                rx
+                + r"templates/(?P<app_label>\w+)/"
+                + r"(?P<actor>\w+)/(?P<pk>\w+)/(?P<fldname>\w+)$",
+                views.Templates.as_view(),
+            ),
+            url(
+                rx
+                + r"templates/(?P<app_label>\w+)/"
+                + r"(?P<actor>\w+)/(?P<pk>\w+)/(?P<fldname>\w+)/"
+                + r"(?P<tplname>\w+)$",
+                views.Templates.as_view(),
+            ),
+        ]
 
         return urlpatterns
 
     def get_row_edit_lines(self, e, panel):
         from lino.core.elems import TextFieldElement
+
         if isinstance(e, TextFieldElement):
-            if e.format == 'html':
+            if e.format == "html":
                 yield "%s.refresh();" % e.as_ext()
 
-    def setup_config_menu(self, site, user_type, m):
+    def setup_config_menu(self, site, user_type, m, ar=None):
         if site.user_model is not None:
             mg = site.plugins.office
             m = m.add_menu(mg.app_label, mg.verbose_name)
-            m.add_action('tinymce.MyTextFieldTemplates')
+            m.add_action("tinymce.MyTextFieldTemplates")
 
-    def setup_explorer_menu(self, site, user_type, m):
+    def setup_explorer_menu(self, site, user_type, m, ar=None):
         if site.user_model is not None:
             mg = site.plugins.office
             m = m.add_menu(mg.app_label, mg.verbose_name)
-            m.add_action('tinymce.TextFieldTemplates')
+            m.add_action("tinymce.TextFieldTemplates")
