@@ -1,7 +1,9 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2023 Rumma & Ko Ltd
+# Copyright 2011-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
+import random
+import string
 from datetime import timedelta
 from django.db import models
 from django.db.models import Q
@@ -9,38 +11,47 @@ from django.conf import settings
 from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
 from django.utils import timezone
 
-from etgen.html import E
-from lino.api import dd, rt, _
+from lino.api import dd, rt, _, gettext
 from lino.core import userprefs
-# from lino.core.fields import NullCharField
+from lino.core.roles import Supervisor
 from lino.core.roles import SiteAdmin
-
+# from lino.core.fields import NullCharField
 from lino.mixins import CreatedModified, Contactable
 from lino.mixins import DateRange
+from lino.modlib.about.choicelists import TimeZones, DateFormats
+from lino.modlib.printing.mixins import Printable
+# from lino.modlib.publisher.mixins import Publishable
+from lino.modlib.about.models import About
+from lino.utils.html import tostring, format_html, mark_safe
 
 from .choicelists import UserTypes
-from .mixins import UserAuthored  #, TimezoneHolder
-from .actions import ChangePassword, SignOut, VerifyUser, CheckedSubmitInsert
-from .actions import SendWelcomeMail, SignIn, CreateAccount, ConnectAccount
-from .actions import MySettings
-# from .actions import SignIn
-from lino.modlib.about.choicelists import TimeZones, DateFormats
-from lino.modlib.system.choicelists import DashboardLayouts
-from lino.modlib.publisher.mixins import Publishable
+from .mixins import UserAuthored  # , TimezoneHolder
+from .actions import ChangePassword, SignOut, CheckedSubmitInsert
+from .actions import SendWelcomeMail, SignIn, ConnectAccount
+from .actions import CreateAccount, ResetPassword, VerifyUser, VerifyMe
 
-from lino.core.roles import Supervisor
+from .ui import *
+
+
+if multi_ledger := dd.is_installed("ledgers"):
+    from lino_xl.lib.ledgers.actions import SubscribeToLedger
 
-import random
-import string
 
 def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
     # thanks to https://stackoverflow.com/questions/2257441/random-string-generation-with-upper-case-letters-and-digits-in-python
-    return ''.join(random.SystemRandom().choice(chars) for _ in range(size))
+    # dd.logger.info("20240418 Gonna call random.SystemRandom()")
+    return "".join(random.SystemRandom().choice(chars) for _ in range(size))
 
 
 partner_model = dd.plugins.users.partner_model
 
-SYNCHRONIZED_FIELDS = ('first_name', 'last_name', 'email', 'language', 'remarks')
+SYNCHRONIZED_FIELDS = ("first_name", "last_name", "email", "language", "remarks")
+
+# Setting this to True would be more consistent in some regards but causes about
+# 25 doctests in welfare to fail, and it's not sure whether  the welfare end
+# users would like it:
+
+UPPERCASE_LAST_NAME_FOR_USERS = False
 
 
 class UserManager(BaseUserManager):
@@ -51,7 +62,7 @@ class UserManager(BaseUserManager):
         Creates and saves a User with the given username, email and password.
         """
         if not username:
-            raise ValueError('The given username must be set')
+            raise ValueError("The given username must be set")
         email = self.normalize_email(email)
         username = self.model.normalize_username(username)
         user = self.model(username=username, email=email, **extra_fields)
@@ -60,95 +71,121 @@ class UserManager(BaseUserManager):
         return user
 
     def create_user(self, username, email=None, password=None, **extra_fields):
-        extra_fields.setdefault('user_type', UserTypes.user)
+        extra_fields.setdefault("user_type", UserTypes.user)
         return self._create_user(username, email, password, **extra_fields)
 
     def create_superuser(self, username, email, password, **extra_fields):
-        extra_fields.setdefault('user_type', UserTypes.admin)
+        extra_fields.setdefault("user_type", UserTypes.admin)
         return self._create_user(username, email, password, **extra_fields)
 
 
-class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRange):
-    class Meta(object):
-        app_label = 'users'
-        verbose_name = _('User')
-        verbose_name_plural = _('Users')
-        abstract = dd.is_abstract_model(__name__, 'User')
-        ordering = ['last_name', 'first_name', 'username']
+class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange):
 
-    USERNAME_FIELD = 'username'
+    class Meta:
+        app_label = "users"
+        verbose_name = _("User")
+        verbose_name_plural = _("Users")
+        abstract = dd.is_abstract_model(__name__, "User")
+        ordering = ["last_name", "first_name", "username"]
+
+    USERNAME_FIELD = "username"
     _anon_user = None
     objects = UserManager()
 
     preferred_foreignkey_width = 15
-    hidden_columns = 'password remarks'
+    hidden_columns = "password remarks"
     # authenticated = True
-    quick_search_fields = 'username user_type first_name last_name remarks'
+    quick_search_fields = "username user_type first_name last_name remarks"
     allow_merge_action = True
 
-    # seems that Django doesn't like nullable username
-    # username = dd.NullCharField(_('Username'), max_length=30, unique=True)
-    username = models.CharField(_('Username'), max_length=30, unique=True)
+    username = models.CharField(
+        _("Username"), max_length=30, unique=True, null=True, blank=True)
     user_type = UserTypes.field(blank=True)
-    initials = models.CharField(_('Initials'), max_length=10, blank=True)
-    first_name = models.CharField(_('First name'), max_length=30, blank=True)
-    last_name = models.CharField(_('Last name'), max_length=30, blank=True)
-    remarks = models.TextField(_("Remarks"), blank=True)  # ,null=True)
-    partner = dd.ForeignKey(partner_model,
-                blank=True, null=True,
-                related_name='users_by_partner')
-
-    if dd.plugins.users.allow_online_registration:
-        verification_code = models.CharField(max_length=200, blank=True)
-        verification_code_sent_on = models.DateTimeField(null=True, blank=True)
-        verify = VerifyUser()
+    initials = models.CharField(_("Initials"), max_length=10, blank=True)
+    if dd.plugins.users.with_nickname:
+        nickname = models.CharField(_("Nickname"), max_length=50, blank=True)
     else:
-        verify = None
-        verification_code = dd.DummyField()
-        verification_code_sent_on = dd.DummyField()
+        nickname = dd.DummyField("")
+    first_name = models.CharField(_("First name"), max_length=30, blank=True)
+    last_name = models.CharField(_("Last name"), max_length=30, blank=True)
+    remarks = models.TextField(_("Remarks"), blank=True)  # ,null=True)
+    partner = dd.ForeignKey(
+        partner_model, blank=True, null=True, related_name="users_by_partner"
+    )
+
+    verification_password = models.CharField(max_length=200, blank=True)
+    verification_code = models.CharField(max_length=200, blank=True, default="!")
+    verification_code_sent_on = models.DateTimeField(null=True, blank=True)
 
     if settings.USE_TZ:
-        time_zone = TimeZones.field(default='default')
+        time_zone = TimeZones.field(default="default")
     else:
         time_zone = dd.DummyField()
 
-    date_format = DateFormats.field(default='default')
-    dashboard_layout = DashboardLayouts.field(blank=True, null=True)
+    date_format = DateFormats.field(default="default")
+
+    ledger = dd.ForeignKey("ledgers.Ledger", null=True, blank=True)
+
+    if multi_ledger:
+        ledger_subscribe__user = SubscribeToLedger(
+            params_layout="""
+        company role
+        ledger
+        """
+        )
 
     submit_insert = CheckedSubmitInsert()
+
+    verify_me = VerifyMe()
     send_welcome_email = SendWelcomeMail()
     change_password = ChangePassword()
     # sign_in = SignIn()
     sign_out = SignOut()
-    my_settings = MySettings()
+
+    if dd.get_plugin_setting("users", "third_party_authentication"):
+        connect_account = ConnectAccount()
+
+    # if settings.SITE.default_ui == "lino_react.react":
+    #     my_settings = MySettings()
 
     def __str__(self):
-        return self.get_full_name()
+        return self.nickname or self.get_full_name()
+
+    @classmethod
+    def get_simple_parameters(cls):
+        for p in super().get_simple_parameters():
+            yield p
+        yield "user_type"
 
     @property
     def is_active(self):
         # if not self.has_usable_password():
-        #     return False
+        if not self.user_type or not self.username:
+            return False
         if self.start_date and self.start_date > dd.today():
             return False
         if self.end_date and self.end_date < dd.today():
             return False
         return True
 
+    def must_verify(self):
+        self.verification_code = id_generator(12)
+        self.verification_code_sent_on = dd.now()
+
     def is_verified(self):
-        if dd.plugins.users.allow_online_registration:
-            return not self.verification_code
-        return False
+        return not self.verification_code
 
     def mark_verified(self):
-        self.verification_code = ''
+        self.verification_code = ""
 
     def is_verification_code_expired(self):
         if self.verification_code_sent_on is None:
             return False
-        return (self.verification_code_sent_on + timedelta(
-            minutes=dd.plugins.users.verification_code_expires
-        ) < timezone.now())
+        return (
+            self.verification_code_sent_on
+            + timedelta(minutes=dd.plugins.users.verification_code_expires)
+            < dd.now()
+        )
 
     def get_as_user(self):
         """
@@ -159,13 +196,21 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
     def get_full_name(self):
         if not self.first_name and not self.last_name:
             return self.initials or self.username or str(self.pk)
-        return '{} {}'.format(self.first_name, self.last_name).strip()
+        if UPPERCASE_LAST_NAME_FOR_USERS and settings.SITE.uppercase_last_name:
+            last_name = self.last_name.upper()
+        else:
+            last_name = self.last_name
+        if settings.SITE.last_name_first:
+            return f"{last_name}, {self.first_name}".strip()
+        return f"{self.first_name} {last_name}".strip()
 
-    # name_column is now defined in core.model.Model
-    # @dd.displayfield(_("Name"), max_length=15)
-    # def name_column(self, ar):
-    #     # return join_words(self.last_name.upper(),self.first_name)
-    #     return str(self)
+    @dd.displayfield(_("Status"))
+    def status(self, ar):
+        s = _("Active") if self.is_active else _("Inactive")
+        s += ". " + (_("Verified") if self.is_verified() else _("Not verified"))
+        if self.verification_password:
+            s += ". " + _("Password reset")
+        return s + "."
 
     # @dd.displayfield(_("Other authentication providers"))
     # def social_auth_links(self, ar=None):
@@ -175,17 +220,17 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
     #     # elems.append(E.a("foo"))
     #     # return E.p(elems)
 
-    @dd.virtualfield(dd.ForeignKey('contacts.Person'))
+    @dd.virtualfield(dd.ForeignKey("contacts.Person"))
     def person(self, ar):
         if self.partner:
-            return self.partner.get_mti_child('person')
+            return self.partner.get_mti_child("person")
 
     # person = property(get_person)
 
-    @dd.virtualfield(dd.ForeignKey('contacts.Company'))
+    @dd.virtualfield(dd.ForeignKey("contacts.Company"))
     def company(self, ar):
         if self.partner:
-            return self.partner.get_mti_child('company')
+            return self.partner.get_mti_child("company")
 
     def is_editable_by_all(self):
         return False
@@ -201,24 +246,26 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
                 if not user.user_type.has_required_roles([SiteAdmin]):
                     if not self.is_editable_by_all():
                         return False
-        return super(User, self).get_row_permission(ar, state, ba)
-        #~ return False
+        return super().get_row_permission(ar, state, ba)
+        # ~ return False
 
     def disabled_fields(self, ar):
         """
         Only System admins may change the `user_type` of users.
         See also :meth:`Users.get_row_permission`.
         """
-        rv = super(User, self).disabled_fields(ar)
+        rv = super().disabled_fields(ar)
         user = ar.get_user()
+        if self.is_verified():
+            rv.add("verify_me")
         if not user.user_type.has_required_roles([SiteAdmin]):
-            rv.add('user_type')
-            rv.add('partner')
-            rv.add('send_email')
+            rv.add("user_type")
+            rv.add("partner")
+            rv.add("send_email")
             # merging two users is only for SiteAdmin, even when you are Expert
-            rv.add('merge_row')
+            rv.add("merge_row")
             if user != self:
-                rv.add('change_password')
+                rv.add("change_password")
         return rv
 
     def full_clean(self, *args, **kw):
@@ -227,7 +274,7 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
         # if not self.initials:
         #     if self.first_name and self.last_name:
         #         self.initials = self.first_name[0] + self.last_name[0]
-        super(User, self).full_clean(*args, **kw)
+        super().full_clean(*args, **kw)
         self.sync_partner()
         if not self.language:
             self.language = settings.SITE.get_default_language()
@@ -235,7 +282,8 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
     def sync_partner(self, **kw):
         if partner_model is None:
             return
-        if self.user_type and isinstance(self.user_type.role, SiteAdmin):
+        if self.user_type and self.user_type.has_required_roles([SiteAdmin]):
+            # if self.user_type and isinstance(self.user_type.role, SiteAdmin):
             # site admins are not automatically linked to a partner
             return
         p = self.partner
@@ -254,26 +302,24 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
                 p.save()
                 self.partner = p
         else:
-            p = p.get_mti_child('person')
+            p = p.get_mti_child("person")
             if p is not None:
                 for k in SYNCHRONIZED_FIELDS:
                     if not getattr(self, k):
                         setattr(self, k, getattr(p, k))
 
     def on_create(self, ar):
-        self.email_changed(ar)
+        self.must_verify()
         self.start_date = dd.today()
-        return super(User, self).on_create(ar)
+        return super().on_create(ar)
 
     def email_changed(self, ar):
-        if self.email and dd.plugins.users.allow_online_registration:
-            self.verification_code = id_generator(12)
-            self.verification_code_sent_on = timezone.now()
+        self.must_verify()
 
     def get_received_mandates(self):
-        #~ return [ [u.id,_("as %s")%u] for u in self.__class__.objects.all()]
+        # ~ return [ [u.id,_("as %s")%u] for u in self.__class__.objects.all()]
         return [[u.id, str(u)] for u in self.__class__.objects.all()]
-        #~ return self.__class__.objects.all()
+        # ~ return self.__class__.objects.all()
 
     # @dd.htmlbox(_("Welcome"))
     # def welcome_email_body(self, ar):
@@ -281,7 +327,7 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
     #     return self.get_welcome_email_body(ar)
 
     def get_welcome_email_body(self, ar):
-        template = rt.get_template('users/welcome_email.eml')
+        template = rt.get_template("users/welcome_email.eml")
         # sar = ar.spawn_request(permalink_uris=True)
         ar.permalink_uris = True
         context = self.get_printable_context(ar)
@@ -290,15 +336,20 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
 
     @classmethod
     def get_active_users(cls, required_roles=[], unwanted_roles=None, **kwargs):
-        user_types = [t for t in UserTypes.get_list_items()
-              if t.has_required_roles(required_roles)]
+        user_types = [
+            t
+            for t in UserTypes.get_list_items()
+            if t.has_required_roles(required_roles)
+        ]
         if unwanted_roles is not None:
-            user_types = [t for t in user_types
-                if not t.has_required_roles(unwanted_roles)]
+            user_types = [
+                t for t in user_types if not t.has_required_roles(unwanted_roles)
+            ]
         qs = cls.objects.filter(
-            Q(end_date__isnull=True)|Q(end_date__gt=dd.today()),
-            Q(start_date__isnull=True)|Q(start_date__lt=dd.today()),
-            user_type__in=user_types)
+            Q(end_date__isnull=True) | Q(end_date__gt=dd.today()),
+            Q(start_date__isnull=True) | Q(start_date__lt=dd.today()),
+            user_type__in=user_types,
+        )
         if kwargs:
             qs = qs.filter(**kwargs)
         return qs
@@ -315,8 +366,8 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
         except cls.DoesNotExist:
             if default is models.NOT_PROVIDED:
                 raise cls.DoesNotExist(
-                    "No %s with username %r" % (
-                        str(cls._meta.verbose_name), username))
+                    "No %s with username %r" % (str(cls._meta.verbose_name), username)
+                )
             return default
 
     def get_preferences(self):
@@ -331,13 +382,15 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
         return settings.SITE.get_anonymous_user()
 
     @classmethod
-    def filter_active_users(self, qs, today, prefix=''):
+    def filter_active_users(self, qs, today, prefix=""):
         qs = qs.filter(
-            Q(**{prefix+"start_date__isnull":True}) |
-            Q(**{prefix+"start_date__lte": today}))
+            Q(**{prefix + "start_date__isnull": True})
+            | Q(**{prefix + "start_date__lte": today})
+        )
         qs = qs.filter(
-            Q(**{prefix+"end_date__isnull":True}) |
-            Q(**{prefix+"end_date__gte": today}))
+            Q(**{prefix + "end_date__isnull": True})
+            | Q(**{prefix + "end_date__gte": today})
+        )
         return qs
 
     # @dd.action(label=_("Send e-mail"),
@@ -348,19 +401,21 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
 
     def usertext(self):
         return "{0} {1}, {3} ({2})".format(
-            self.last_name, self.first_name, self.username, self.user_type)
+            self.last_name, self.first_name, self.username, self.user_type
+        )
 
     def get_authorities(self):
-
         if self.has_required_roles([Supervisor]):
-            users = settings.SITE.user_model.objects.exclude(
-                    user_type='').exclude(id=self.id)
+            users = settings.SITE.user_model.objects.exclude(user_type="").exclude(
+                id=self.id
+            )
         else:
-            qs = rt.models.users.Authority.objects.filter(
-                authorized=self).exclude(user__user_type='').select_related("user")
-            qs = qs.order_by(
-                'user__last_name', 'user__first_name',
-                'user__username')
+            qs = (
+                rt.models.users.Authority.objects.filter(authorized=self)
+                .exclude(user__user_type="")
+                .select_related("user")
+            )
+            qs = qs.order_by("user__last_name", "user__first_name", "user__username")
             users = [a.user for a in qs]
         return [(u.id, u.usertext()) for u in users]
 
@@ -368,11 +423,25 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Publishable, DateRang
     # def get_default_table(cls):
     #     return rt.models.users.MySettings
 
-settings.AUTH_USER_MODEL = 'users.User'
+    def dt_astimezone(self, dt):
+        """
+        Convert datetime to user timezone if time_zone is not None otherwise to default timezone.
+        Used in template notify/summary.eml
+        """
+        default_tz = rt.models.about.TimeZones.default
+        aware_dt = dt.astimezone(default_tz.tzinfo)
+        if self.time_zone:
+            aware_dt = aware_dt.astimezone(self.time_zone.tzinfo)
+        return aware_dt
+
+
+settings.AUTH_USER_MODEL = "users.User"
+
 
 class Authority(UserAuthored):
-    class Meta(object):
-        app_label = 'users'
+
+    class Meta:
+        app_label = "users"
         verbose_name = _("Authority")
         verbose_name_plural = _("Authorities")
 
@@ -380,15 +449,15 @@ class Authority(UserAuthored):
 
     @dd.chooser()
     def authorized_choices(cls, user):
-        qs = settings.SITE.user_model.objects.exclude(
-            user_type=None)
-            #~ user_type=UserTypes.blank_item) 20120829
+        qs = settings.SITE.user_model.objects.exclude(user_type=None)
+        # ~ user_type=UserTypes.blank_item) 20120829
         if user is not None:
             qs = qs.exclude(id=user.id)
-            #~ .exclude(level__gte=UserLevels.admin)
+            # ~ .exclude(level__gte=UserLevels.admin)
         return qs
 
-dd.update_field(Authority, 'user', null=False)
+
+dd.update_field(Authority, "user", null=False)
 
 # @dd.receiver(dd.pre_startup)
 # def inject_partner_field(sender=None, **kwargs):
@@ -409,32 +478,58 @@ dd.update_field(Authority, 'user', null=False)
 
 
 class Permission(dd.Model):
-    class Meta(object):
-        app_label = 'users'
+
+    class Meta:
+        app_label = "users"
         abstract = True
 
 
-from lino.modlib.about.models import About
 About.sign_in = SignIn()
+About.reset_password = ResetPassword()
+About.verify_user = VerifyUser()
+
 if dd.plugins.users.allow_online_registration:
     About.create_account = CreateAccount()
 
-@dd.receiver(dd.post_startup)
-def setup_memo_commands(sender=None, **kwargs):
-    # See :doc:`/specs/memo`
 
-    if not sender.is_installed('memo'):
-        return
+if dd.is_installed("memo"):
 
-    mp = sender.plugins.memo.parser
-    mp.add_suggester(
-        "@",
-        sender.models.users.User.objects.filter(
-            username__isnull=False).order_by('username'),
-        'username')
+    @dd.receiver(dd.post_startup)
+    def setup_memo_commands(sender=None, **kwargs):
+        # See :doc:`/specs/memo`
+
+        # if not sender.is_installed("memo"):
+        #     return
+
+        mp = sender.plugins.memo.parser
+        mp.add_suggester(
+            "@",
+            sender.models.users.User.objects.filter(
+                username__isnull=False).order_by("username"),
+            "username")
 
 
-from .ui import *
+if dd.plugins.users.allow_online_registration:
 
-if dd.get_plugin_setting('users', 'third_party_authentication'):
-    Me.connect_account = ConnectAccount()
+    def welcome_messages(ar):
+        me = ar.get_user()
+        if not me.is_verified():
+            # sar = rt.models.users.Me.create_request(parent=ar)
+            sar = ar
+            if me.email:
+                # verify_me =
+                msg = format_html(
+                    _("Your email address ({email}) is not verified, "
+                      "please check your mailbox and {verify} or {resend}."),
+                    email=me.email,
+                    verify=tostring(sar.instance_action_button(
+                        me.verify_me, _("verify now"))),
+                    resend=tostring(sar.instance_action_button(
+                        me.send_welcome_email, _("re-send our welcome email"))))
+            else:
+                msg = format_html(
+                    _("You have no email address, please {edit}."),
+                    edit=tostring(sar.obj2html(me, _("edit your user settings"))))
+            yield mark_safe(msg)
+
+    dd.add_welcome_handler(welcome_messages)
