@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2013-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 A Sphinx extension used to write multilingual user documentation
 for a Lino application.
@@ -12,7 +11,7 @@ for a Lino application.
   source code.  This is like :rst:dir:`py2rst` but with the following
   names defined:
 
-  :settings: The Django settings module which is active while building
+  :settings: The Django settings file which is active while building
              the docs.
 
   :dd: The :mod:`lino.api.dd` module.
@@ -54,10 +53,10 @@ The visible text will be automatically in the right language in
 multilingual userdocs.
 """
 
-
 from .base import menuselection_text
 
 from docutils.parsers.rst import Directive
+
 # from sphinx.util.compat import Directive
 from sphinx.roles import XRefRole
 from sphinx.roles import MenuSelection
@@ -87,7 +86,6 @@ from lino.core.actors import resolve_action
 from lino.modlib.users.choicelists import UserTypes
 
 
-
 def actor_name(a):
     return fully_normalize_name(settings.SITE.userdocs_prefix + str(a))
 
@@ -97,36 +95,36 @@ def model_name(m):
 
 
 def app_name(a):
-    assert a.__name__.endswith('.models')
-    parts = a.__name__.split('.')
+    assert a.__name__.endswith(".models")
+    parts = a.__name__.split(".")
     return settings.SITE.userdocs_prefix + parts[-2]
 
 
 def actor_ref(rpt, text=None):
     if text is None:
         text = force_str(rpt.label or rpt.title or str(rpt))
-    return ':ddref:`%s <%s>`' % (text, rpt)
+    return ":ddref:`%s <%s>`" % (text, rpt)
 
 
 def model_ref(m, text=None):
     if text is None:
         text = force_str(m._meta.verbose_name)
-    return ':ref:`%s <%s>`' % (text, model_name(m))
+    return ":ref:`%s <%s>`" % (text, model_name(m))
 
 
 def rptlist(l):
-    return ', '.join([actor_ref(a) for a in l])
+    return ", ".join([actor_ref(a) for a in l])
 
 
 def typeref(cls):
     text = cls.__name__
-    target = cls.__module__ + '.' + cls.__name__
+    target = cls.__module__ + "." + cls.__name__
     return ":class:`%s <%s>`" % (text, target)
 
 
 def old_fieldtype(f):
     if isinstance(f, models.ForeignKey):
-        #~ return f.__class__.__name__ + " to " + refto(f.rel.model)
+        # ~ return f.__class__.__name__ + " to " + refto(f.rel.model)
         return f.__class__.__name__ + " to " + model_ref(f.remote_field.model)
     return f.__class__.__name__
 
@@ -135,11 +133,13 @@ def fieldtype(f):
     s = typeref(f.__class__)
     if isinstance(f, models.ForeignKey):
         s = _("%(classref)s to %(model)s") % dict(
-            classref=s, model=model_ref(f.remote_field.model))
-        #~ print(20130908, s)
+            classref=s, model=model_ref(f.remote_field.model)
+        )
+        # ~ print(20130908, s)
     if isinstance(f, choicelists.ChoiceListField):
         s = _("%(classref)s to %(model)s") % dict(
-            classref=s, model=actor_ref(f.choicelist))
+            classref=s, model=actor_ref(f.choicelist)
+        )
     return s
 
 
@@ -157,13 +157,13 @@ def fields_ul(fields):
 
     items = []
     for f in fields:
-        if not hasattr(f, '_lino_babel_field'):
+        if not hasattr(f, "_lino_babel_field"):
             s = field2li(f)
             if s:
                 items.append(s)
-    #~ items = [ field2li(f) for f in fields if not hasattr(f,'_lino_babel_field')]
+    # ~ items = [ field2li(f) for f in fields if not hasattr(f,'_lino_babel_field')]
     if len(helpless):
-        s = ', '.join(helpless)
+        s = ", ".join(helpless)
         if len(items):
             s = _("... and %s") % s
         items.append(s)
@@ -172,26 +172,22 @@ def fields_ul(fields):
 
 def fields_table(fields):
     headers = ["name", "type"]
-    #~ formatters = [
-      #~ lambda f: f.name,
-      #~ lambda f: f.__class__.__name__,
-    #~ ]
+    # ~ formatters = [
+    # ~ lambda f: f.name,
+    # ~ lambda f: f.__class__.__name__,
+    # ~ ]
     headers.append("verbose name")
     headers.append("help text")
 
     def rowfmt(f):
-        cells = [
-            f.name,
-            fieldtype(f),
-            f.verbose_name,
-            f.help_text
-        ]
-        #~ for lng in babel.AVAILABLE_LANGUAGES:
-            #~ babel.set_language(lng)
-            #~ cells.append(force_str(_(f.verbose_name)))
-        #~ cells.append(f.help_text)
+        cells = [f.name, fieldtype(f), f.verbose_name, f.help_text]
+        # ~ for lng in babel.AVAILABLE_LANGUAGES:
+        # ~ babel.set_language(lng)
+        # ~ cells.append(force_str(_(f.verbose_name)))
+        # ~ cells.append(f.help_text)
         return cells
-    rows = [rowfmt(f) for f in fields if not hasattr(f, '_lino_babel_field')]
+
+    rows = [rowfmt(f) for f in fields if not hasattr(f, "_lino_babel_field")]
     return rstgen.table(headers, rows)
 
 
@@ -203,36 +199,41 @@ def get_actor_description(self):
     if self.help_text:
         body += unindent(force_str(self.help_text).strip()) + "\n\n"
 
-    #~ ll = self.get_handle().list_layout
-    #~ if ll is not None:
-        #~ body += fields_table([ e.field for e in ll.main.columns] )
+    # ~ ll = self.get_handle().grid_layout
+    # ~ if ll is not None:
+    # ~ body += fields_table([ e.field for e in ll.main.columns] )
 
-    #~ model_reports = [r for r in kernel.master_tables if r.model is self.model]
-    #~ if model_reports:
-        #~ body += '\n\nMaster tables: %s\n\n' % rptlist(model_reports)
-    #~ if getattr(model,'_lino_slaves',None):
-        #~ body += '\n\nSlave tables: %s\n\n' % rptlist(model._lino_slaves.values())
+    # ~ model_reports = [r for r in kernel.master_tables if r.model is self.model]
+    # ~ if model_reports:
+    # ~ body += '\n\nMaster tables: %s\n\n' % rptlist(model_reports)
+    # ~ if getattr(model,'_lino_slaves',None):
+    # ~ body += '\n\nSlave tables: %s\n\n' % rptlist(model._lino_slaves.values())
 
     return body
 
-#~ def get_model_description(self):
-    #~ """
-    #~ `self` is the actor
-    #~ """
-    #~ body = "\n\n"
-    #~ help_text = getattr(self,'help_text',None)
-    #~ if help_text:
-        #~ body += unindent(force_str(help_text).strip()) + "\n\n"
-#~
-    #~ body += fields_table(self._meta.fields)
-    #~
-    #~ return body
 
+# ~ def get_model_description(self):
+# ~ """
+# ~ `self` is the actor
+# ~ """
+# ~ body = "\n\n"
+# ~ help_text = getattr(self,'help_text',None)
+# ~ if help_text:
+# ~ body += unindent(force_str(help_text).strip()) + "\n\n"
+# ~
+# ~ body += fields_table(self._meta.fields)
+# ~
+# ~ return body
+
+IGNORED_ACTIONS = (
+    actions.ShowTable,
+    actions.SubmitDetail,
+    actions.ShowDetail,
+    actions.DeleteSelected,
+    actions.ShowInsert,
+    actions.SubmitInsert,
+)
 
-IGNORED_ACTIONS = (actions.ShowTable, actions.SubmitDetail,
-                   actions.ShowDetail,
-                   actions.DeleteSelected,
-                   actions.ShowInsert, actions.SubmitInsert)
 
 def menuselection(mi):
     return ":menuselection:`%s`" % menuselection_text(mi)
@@ -252,23 +253,24 @@ def actions_ul(action_list):
         items.append(desc)
     return rstgen.ul(items)
 
+
 # from lino.core.menus import find_menu_item
 
 
 def actors_overview_ul(model_reports):
-    user_type = UserTypes.get_by_value('900')
+    user_type = UserTypes.get_by_value("900")
     # deprecated
     items = []
     for tb in model_reports:
         desc = actor_ref(tb)
-        #~ label = str(tb.title or tb.label)
-        #~ desc += " (%s)" % str(tb)
+        # ~ label = str(tb.title or tb.label)
+        # ~ desc += " (%s)" % str(tb)
         desc += " (%s)" % typeref(tb)
         # mi = find_menu_item(tb.default_action)
         mi = user_type.find_menu_item(tb.default_action)
         if mi is not None:
             desc += _(" (Menu %s)") % menuselection(mi)
-            #~ print(unicode(mi.label).strip())
+            # ~ print(unicode(mi.label).strip())
         if tb.help_text:
             desc += " -- " + str(tb.help_text).strip()
 
@@ -277,12 +279,12 @@ def actors_overview_ul(model_reports):
 
 
 def resolve_name(name):
-    l = name.split('.')
+    l = name.split(".")
     if len(l) == 1:
         return 1, settings.SITE.plugins.get(name)
         # return 1, dd.resolve_app(name)
     if len(l) == 3:
-        model = settings.SITE.models.resolve(l[0] + '.' + l[1])
+        model = settings.SITE.models.resolve(l[0] + "." + l[1])
         if model is None:
             raise Warning("Unkown name %s" % name)
         return 3, model.get_data_elem(l[2])
@@ -290,22 +292,20 @@ def resolve_name(name):
 
 
 def form_lines():
-    yield '<script >'
+    yield "<script >"
 
 
 class ddrefRole(XRefRole):
-
     nodeclass = addnodes.pending_xref
     innernodeclass = nodes.emphasis
 
-    def __call__(self, typ, rawtext, text, lineno, inliner,
-                 options={}, content=[]):
-
-        typ = 'std:ref'
+    def __call__(self, typ, rawtext, text, lineno, inliner, options={}, content=[]):
+        typ = "std:ref"
         self._reporter = inliner.document.reporter
         self._lineno = lineno
-        return XRefRole.__call__(self, typ, rawtext, text, lineno,
-                                 inliner, options, content)
+        return XRefRole.__call__(
+            self, typ, rawtext, text, lineno, inliner, options, content
+        )
 
     def process_link(self, env, refnode, has_explicit_title, title, target):
         """Called after parsing title and target text, and creating the
@@ -314,10 +314,10 @@ class ddrefRole(XRefRole):
         tuple.
         """
 
-        #~ print(20130901, refnode, has_explicit_title, title, target)
-        #~ 20130901 <pending_xref refdomain="" refexplicit="False" reftype="ddref"/> False cal.Event cal.Event
+        # ~ print(20130901, refnode, has_explicit_title, title, target)
+        # ~ 20130901 <pending_xref refdomain="" refexplicit="False" reftype="ddref"/> False cal.Event cal.Event
 
-        target = ws_re.sub(' ', target)  # replace newlines or tabs by spaces
+        target = ws_re.sub(" ", target)  # replace newlines or tabs by spaces
         # ~ target = ' '.join(target.split()) # replace newlines or tabs by spaces
 
         level, x = resolve_name(target)
@@ -330,7 +330,7 @@ class ddrefRole(XRefRole):
         with translation.override(lng):
             if isinstance(x, models.Field):
                 text = utils.unescape(str(x.verbose_name))
-                target = model_name(x.model) + '.' + x.name
+                target = model_name(x.model) + "." + x.name
                 # print(target)
             elif isinstance(x, Plugin):
                 text = utils.unescape(str(x.verbose_name))
@@ -350,16 +350,16 @@ class ddrefRole(XRefRole):
 
         if not has_explicit_title:
             # avoid replacing title by the heading text
-            refnode['refexplicit'] = True
+            refnode["refexplicit"] = True
             title = text
 
-        refnode['refwarn'] = False  # never warn
+        refnode["refwarn"] = False  # never warn
 
-        #~ refnode['reftype'] = 'ref'
+        # ~ refnode['reftype'] = 'ref'
 
-        #~ title = "[%s]" % title
-        #~ if target == 'welfare.reception.waitingvisitors':
-        #~ print("20130907 ddref to %s : title=%r" % (target,title))
+        # ~ title = "[%s]" % title
+        # ~ if target == 'welfare.reception.waitingvisitors':
+        # ~ print("20130907 ddref to %s : title=%r" % (target,title))
 
         return title, target
 
@@ -378,14 +378,12 @@ class TempDataDirective(Directive):
 
     @classmethod
     def get_current_value(cls, env):
-        return env.temp_data.get(
-            cls.temp_data_key,
-            cls.get_default_value(env))
+        return env.temp_data.get(cls.temp_data_key, cls.get_default_value(env))
 
     def run(self):
         env = self.state.document.settings.env
         v = self.arguments[0].strip()
-        if v == 'None':
+        if v == "None":
             del env.temp_data[self.temp_data_key]
         else:
             env.temp_data[self.temp_data_key] = v
@@ -397,7 +395,8 @@ class CurrentLanguage(TempDataDirective):
     this document.
 
     """
-    temp_data_key = 'language'
+
+    temp_data_key = "language"
 
     @classmethod
     def get_default_value(cls, env):
@@ -409,7 +408,8 @@ class CurrentProject(TempDataDirective):
     this document.
 
     """
-    temp_data_key = 'lino_project'
+
+    temp_data_key = "lino_project"
 
 
 class Lino2rstDirective(Py2rstDirective):
@@ -417,6 +417,7 @@ class Lino2rstDirective(Py2rstDirective):
 
     def get_context(self):
         from django.conf import settings
+
         context = super(Lino2rstDirective, self).get_context()
         context.update(settings=settings)
         context.update(settings.SITE.models)
@@ -426,17 +427,17 @@ class Lino2rstDirective(Py2rstDirective):
 
     def output_from_exec(self, code):
         from django.utils import translation
+
         with translation.override(self.language):
             return super(Lino2rstDirective, self).output_from_exec(code)
 
 
 class ActorsOverviewDirective(Lino2rstDirective):
-
     def get_rst(self):
-        user_type = UserTypes.get_by_value('900')
+        user_type = UserTypes.get_by_value("900")
         with translation.override(self.language):
-            #~ set_language(lng)
-            actor_names = ' '.join(self.content).split()
+            # ~ set_language(lng)
+            actor_names = " ".join(self.content).split()
             items = []
             for an in actor_names:
                 cls = settings.SITE.models.resolve(an)
@@ -445,12 +446,12 @@ class ActorsOverviewDirective(Lino2rstDirective):
                 desc = "**{0}** (:class:`{1} <{2}>`)".format(
                     force_str(cls.label),
                     cls.__name__,
-                    cls.__module__ + '.' + cls.__name__
+                    cls.__module__ + "." + cls.__name__,
                 )
                 mi = user_type.find_menu_item(cls.default_action)
                 if mi is not None:
                     desc += _(" (Menu %s)") % menuselection(mi)
-                    #~ print(str(mi.label).strip())
+                    # ~ print(str(mi.label).strip())
                 if cls.help_text:
                     desc += "  : " + force_str(cls.help_text).strip()
 
@@ -460,62 +461,62 @@ class ActorsOverviewDirective(Lino2rstDirective):
 
 
 class ShowFieldsDirective(Lino2rstDirective):
-
     def get_rst(self):
         with translation.override(self.language):
-            names = ' '.join(self.content).split()
+            names = " ".join(self.content).split()
             if len(names) > 1:
-                field_names = ' '.join(names[1:])
+                field_names = " ".join(names[1:])
             else:
                 field_names = None
-            return analyzer.show_fields(
-                names[0], field_names=field_names)
+            return analyzer.show_fields(names[0], field_names=field_names)
 
 
 class FormDirective(Lino2rstDirective):
-
     def get_rst(self):
         level, cls = resolve_name(self.content[0])
-        s = ''
+        s = ""
         with translation.override(self.language):
-            s = '\n'.join(list(form_lines()))
+            s = "\n".join(list(form_lines()))
         return s
 
 
 class ActorDirective(Lino2rstDirective):
-    #~ has_content = False
+    # ~ has_content = False
     titles_allowed = True
-    #~ debug = True
+
+    # ~ debug = True
 
     def get_rst(self):
-        #~ from actordoc import get_actor_description
-        #~ from django.conf import settings
-        #~ from djangosite.dbutils import set_language
+        # ~ from actordoc import get_actor_description
+        # ~ from django.conf import settings
+        # ~ from djangosite.dbutils import set_language
         with translation.override(self.language):
             level, cls = resolve_name(self.content[0])
             if isinstance(cls, models.Field):
                 fld = cls
-                s = ''
-                name = str(fld.model) + '.' + fld.name
+                s = ""
+                name = str(fld.model) + "." + fld.name
                 title = force_str(fld.verbose_name).strip()
 
                 s += "\n.. index::\n   single: "
-                s += str(_('%(field)s (field in %(model)s)') % dict(
-                    field=title, model=model_ref(fld.model)))
-                s += '\n\n'
+                s += str(
+                    _("%(field)s (field in %(model)s)")
+                    % dict(field=title, model=model_ref(fld.model))
+                )
+                s += "\n\n"
                 s += rstgen.header(level, _("%s (field)") % title)
                 if len(self.content) > 1:
-                    s += '\n'.join(self.content[1:])
-                    s += '\n\n'
+                    s += "\n".join(self.content[1:])
+                    s += "\n\n"
                 return s
 
             if isinstance(cls, Plugin):
-                s = ''
+                s = ""
                 title = str(cls.verbose_name)
                 s += "\n.. index::\n   single: "
-                s += str(_('%s (app)') % title)
-                s += '\n\n.. _' + name + ':\n'
-                s += '\n'
+                s += str(_("%s (app)") % title)
+                s += "\n\n.. _" + name + ":\n"
+                s += "\n"
                 s += rstgen.header(level, _("%s (app)") % title)
                 return s
 
@@ -525,90 +526,90 @@ class ActorDirective(Lino2rstDirective):
             if issubclass(cls, models.Model):
                 model = cls
 
-                s = ''
+                s = ""
                 name = model_name(model).lower()
                 title = force_str(model._meta.verbose_name)
                 s += "\n.. index::\n   single: "
-                s += str(_('%(model)s (model in %(app)s)') % dict(
-                    model=title, app=model._meta.app_label))
+                s += str(
+                    _("%(model)s (model in %(app)s)")
+                    % dict(model=title, app=model._meta.app_label)
+                )
 
-                s += '\n\n'
+                s += "\n\n"
 
-                s += '\n\n.. _' + name + ':\n'
+                s += "\n\n.. _" + name + ":\n"
 
-                s += '\n'
+                s += "\n"
                 s += rstgen.header(level, _("%s (model)") % title)
 
-                s += '\n'
-                s += '\n:Internal name: ``%s``\n' % full_model_name(cls)
-                s += '\n:Implemented by: %s\n' % typeref(cls)
-                s += '\n'
+                s += "\n"
+                s += "\n:Internal name: ``%s``\n" % full_model_name(cls)
+                s += "\n:Implemented by: %s\n" % typeref(cls)
+                s += "\n"
 
                 if len(self.content) > 1:
-                    s += '\n'.join(self.content[1:])
-                    s += '\n\n'
+                    s += "\n".join(self.content[1:])
+                    s += "\n\n"
 
-                model_reports = [
-                    r for r in kernel.master_tables if r.model is cls]
-                model_reports += [r for r in kernel.slave_tables
-                                  if r.model is cls]
-                s += rstgen.boldheader(_("Views on %s") %
-                                       cls._meta.verbose_name)
+                model_reports = [r for r in kernel.master_tables if r.model is cls]
+                model_reports += [r for r in kernel.slave_tables if r.model is cls]
+                s += rstgen.boldheader(_("Views on %s") % cls._meta.verbose_name)
                 s += actors_overview_ul(model_reports)
 
-                s += rstgen.boldheader(_("Fields in %s") %
-                                       cls._meta.verbose_name)
+                s += rstgen.boldheader(_("Fields in %s") % cls._meta.verbose_name)
                 s += fields_ul(cls._meta.fields)
 
                 action_list = cls.get_default_table().get_actions()
                 action_list = [
-                    ba for ba in action_list
-                    if not isinstance(ba.action, IGNORED_ACTIONS)]
+                    ba
+                    for ba in action_list
+                    if not isinstance(ba.action, IGNORED_ACTIONS)
+                ]
                 if action_list:
-                    s += '\n'
-                    s += rstgen.boldheader(_("Actions on %s") %
-                                           cls._meta.verbose_name)
+                    s += "\n"
+                    s += rstgen.boldheader(_("Actions on %s") % cls._meta.verbose_name)
                     s += actions_ul(action_list)
 
-                slave_tables = getattr(cls, '_lino_slaves', {}).values()
+                slave_tables = getattr(cls, "_lino_slaves", {}).values()
                 if slave_tables:
-                    s += rstgen.boldheader(_("Tables referring to %s") %
-                                           cls._meta.verbose_name)
+                    s += rstgen.boldheader(
+                        _("Tables referring to %s") % cls._meta.verbose_name
+                    )
                     s += actors_overview_ul(slave_tables)
 
                 return s
 
             if issubclass(cls, actors.Actor):
-
                 title = force_str(cls.label or cls.title)
-                indextext = _('%(actor)s (view in %(app)s)') % dict(
-                    actor=title, app=cls.app_label)
+                indextext = _("%(actor)s (view in %(app)s)") % dict(
+                    actor=title, app=cls.app_label
+                )
                 name = actor_name(cls)
-                #~ if name == 'welfare.reception.waitingvisitors':
-                    #~ self.debug = True
-                #~ print(20130907, name)
-                self.index_entries.append(('single', indextext, name, ''))
-                #~ self.add_ref_target(name,name)
+                # ~ if name == 'welfare.reception.waitingvisitors':
+                # ~ self.debug = True
+                # ~ print(20130907, name)
+                self.index_entries.append(("single", indextext, name, ""))
+                # ~ self.add_ref_target(name,name)
 
-                s = ''
-                s += '\n\n.. _%s:\n\n' % name
+                s = ""
+                s += "\n\n.. _%s:\n\n" % name
                 s += rstgen.header(level, _("%s (view)") % title)
-                s += '\n:Internal name: ``%s`` (%s)\n' % (cls, typeref(cls))
+                s += "\n:Internal name: ``%s`` (%s)\n" % (cls, typeref(cls))
 
                 if len(self.content) > 1:
-                    s += '\n'.join(self.content[1:])
-                    s += '\n\n'
+                    s += "\n".join(self.content[1:])
+                    s += "\n\n"
 
-                s += '\n\n'
+                s += "\n\n"
                 s += get_actor_description(cls)
-                s += '\n\n'
+                s += "\n\n"
                 return s
             raise Exception("Cannot handle actor %r." % cls)
 
     def run(self):
         self.index_entries = []
-        #~ index_entries is a list of 4-tuples of
-        #~ ``(entrytype, entryname, target, ignored)``
+        # ~ index_entries is a list of 4-tuples of
+        # ~ ``(entrytype, entryname, target, ignored)``
         content = super(ActorDirective, self).run()
         indexnode = addnodes.index(entries=self.index_entries)
         return [indexnode] + content
@@ -616,24 +617,23 @@ class ActorDirective(Lino2rstDirective):
 
 def menupath_role(typ, rawtext, text, *args, **kwargs):
     a = resolve_action(text)
-    user_type = UserTypes.get_by_value('900')
+    user_type = UserTypes.get_by_value("900")
     mi = user_type.find_menu_item(a)
     if mi is None:
         raise Exception("Unknown menu descriptor %s" % text)
     text = menuselection_text(mi)
     rawtext = menuselection(mi)
-    return MenuSelection('menuselection', rawtext, text, *args, **kwargs)
+    return MenuSelection("menuselection", rawtext, text, *args, **kwargs)
 
 
 def setup(app):
-
-    app.add_directive('form', FormDirective)
-    app.add_directive('actor', ActorDirective)
-    app.add_directive('actors_overview', ActorsOverviewDirective)
-    app.add_directive('fields_list', ShowFieldsDirective)
-    app.add_role('ddref', ddrefRole())
-    app.add_role('menupath', menupath_role)
-    app.add_directive('currentlanguage', CurrentLanguage)
-    app.add_directive('currentproject', CurrentProject)
-    app.add_directive('django2rst', Lino2rstDirective)  # backward compat
-    app.add_directive('lino2rst', Lino2rstDirective)
+    app.add_directive("form", FormDirective)
+    app.add_directive("actor", ActorDirective)
+    app.add_directive("actors_overview", ActorsOverviewDirective)
+    app.add_directive("fields_list", ShowFieldsDirective)
+    app.add_role("ddref", ddrefRole())
+    app.add_role("menupath", menupath_role)
+    app.add_directive("currentlanguage", CurrentLanguage)
+    app.add_directive("currentproject", CurrentProject)
+    app.add_directive("django2rst", Lino2rstDirective)  # backward compat
+    app.add_directive("lino2rst", Lino2rstDirective)
