@@ -15,6 +15,7 @@ Adds Lino-specific Sphinx setup.
 
 """
 
+
 def my_escape(s):
     s = s.replace("\u25b6 ", "")
     return s
@@ -30,16 +31,22 @@ def menuselection_text(mi):
     return s
 
 
-
-
 def setup(app):
-
-    app.add_object_type('tcname', 'tcname',
-                        objname='template context name',
-                        indextemplate='pair: %s; template context name')
-    app.add_object_type(directivename='fixture', rolename='fixture',
-                        objname='demo fixture name',
-                        indextemplate='pair: %s; demo fixture name')
-    app.add_object_type(directivename='feature', rolename='feature',
-                        objname='site feature name',
-                        indextemplate='pair: %s; site feature name')
+    app.add_object_type(
+        "tcname",
+        "tcname",
+        objname="template context name",
+        indextemplate="pair: %s; template context name",
+    )
+    app.add_object_type(
+        directivename="fixture",
+        rolename="fixture",
+        objname="demo fixture name",
+        indextemplate="pair: %s; demo fixture name",
+    )
+    app.add_object_type(
+        directivename="feature",
+        rolename="feature",
+        objname="site feature name",
+        indextemplate="pair: %s; site feature name",
+    )
