@@ -1,6 +1,5 @@
 # Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Based on http://www.djangosnippets.org/snippets/102/
 
@@ -14,14 +13,16 @@ Added by LS:
 """
 import subprocess
 from subprocess import call, PIPE
-#from os import remove, rename
+
+# from os import remove, rename
 import os
-#from os.path import dirname
+
+# from os.path import dirname
 import tempfile  # from tempfile import NamedTemporaryFile
 from django.template import loader, Context
 
 
-def process_latex(template, context={}, outtype='pdf', outfile=None):
+def process_latex(template, context={}, outtype="pdf", outfile=None):
     """
     Processes a template as a LaTeX source file.
     Output is either being returned or stored in outfile.
@@ -38,24 +39,26 @@ def process_latex(template, context={}, outtype='pdf', outfile=None):
     tex = os.fdopen(fd)
     tex.write(r)
     # tex.flush()
-    #base = tex.name
+    # base = tex.name
     # print "base:",base
     tex.close()
     items = "log aux pdf dvi png".split()
-    names = dict((x, '%s.%s' % (base, x)) for x in items)
+    names = dict((x, "%s.%s" % (base, x)) for x in items)
     output = names[outtype]
 
-    if outtype == 'pdf' or outtype == 'dvi':
+    if outtype == "pdf" or outtype == "dvi":
         pdflatex(base, outtype)
-    elif outtype == 'png':
-        pdflatex(base, 'dvi')
-        call(['dvipng', '-bg', '-transparent',
-              names['dvi'], '-o', names['png']],
-             cwd=os.path.dirname(base),
-             stdout=PIPE, stderr=PIPE)
+    elif outtype == "png":
+        pdflatex(base, "dvi")
+        call(
+            ["dvipng", "-bg", "-transparent", names["dvi"], "-o", names["png"]],
+            cwd=os.path.dirname(base),
+            stdout=PIPE,
+            stderr=PIPE,
+        )
 
-    os.remove(names['log'])
-    os.remove(names['aux'])
+    os.remove(names["log"])
+    os.remove(names["aux"])
     os.remove(base)
 
     if not outfile:
@@ -66,12 +69,11 @@ def process_latex(template, context={}, outtype='pdf', outfile=None):
         os.rename(output, outfile)
 
 
-def pdflatex(outfile, outtype='pdf'):
+def pdflatex(outfile, outtype="pdf"):
     dirname, filename = os.path.split(outfile)
     print("dirname:", dirname)
     print("filename:", filename)
     subprocess.check_call(
-        ['pdflatex', '-interaction=nonstopmode',
-         '-output-format', outtype,
-         filename],
-        cwd=dirname)  # , stdout=PIPE, stderr=PIPE)
+        ["pdflatex", "-interaction=nonstopmode", "-output-format", outtype, filename],
+        cwd=dirname,
+    )  # , stdout=PIPE, stderr=PIPE)
