@@ -1,6 +1,5 @@
 # Copyright 2010-2015 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Generic support for :ref:`mldbc`.
 
 See usage example in :ref:`mldbc_tutorial`.
@@ -28,15 +27,14 @@ Multilingual database content:
 
 from django.conf import settings
 
+
 def babel_named(model, name, **kwargs):
     """Instantiate the given subclass of BabelNamed"""
-    kwargs = settings.SITE.str2kw('name', name, **kwargs)
+    kwargs = settings.SITE.str2kw("name", name, **kwargs)
     return model(**kwargs)
 
 
 def babeld(model, desig, **kwargs):
     """Instantiate the given subclass of BabelDesignated"""
-    kwargs = settings.SITE.str2kw('designation', desig, **kwargs)
+    kwargs = settings.SITE.str2kw("designation", desig, **kwargs)
     return model(**kwargs)
-
-
