@@ -1,6 +1,5 @@
 # Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 A collection of tools around multi-table inheritance.
 
@@ -13,12 +12,12 @@ from lino import logger
 
 from django.db import models
 from django.db import router
+
 # from django.db.models.deletion import Collector
 from django.db.models.deletion import Collector, DO_NOTHING
 from django.core.exceptions import ValidationError
 from django.utils.translation import gettext_lazy as _
 
-
 from lino.core.utils import resolve_model
 from lino.core.fields import VirtualField
 from lino.core.signals import pre_remove_child, pre_add_child  # , on_add_child
@@ -30,8 +29,15 @@ class ChildCollector(Collector):
     A Collector that does not delete the MTI parents.
     """
 
-    def collect(self, objs, source=None, nullable=False, collect_related=True,
-                source_attr=None, reverse_dependency=False):
+    def collect(
+        self,
+        objs,
+        source=None,
+        nullable=False,
+        collect_related=True,
+        source_attr=None,
+        reverse_dependency=False,
+    ):
         # modified copy of django.db.models.deletion.Collector.collect()
         # changes:
         # DOES NOT Recursively collect concrete model's parent models.
@@ -40,19 +46,21 @@ class ChildCollector(Collector):
         if self.can_fast_delete(objs):
             self.fast_deletes.append(objs)
             return
-        new_objs = self.add(objs, source, nullable,
-                            reverse_dependency=reverse_dependency)
+        new_objs = self.add(
+            objs, source, nullable, reverse_dependency=reverse_dependency
+        )
         if not new_objs:
             return
 
         model = new_objs[0].__class__
 
         if collect_related:
-
             related_objects = [
-                f for f in model._meta.get_fields(include_hidden=True)
+                f
+                for f in model._meta.get_fields(include_hidden=True)
                 if (f.one_to_many or f.one_to_one)
-                and f.auto_created and not f.concrete
+                and f.auto_created
+                and not f.concrete
                 and f.model is model  ## local_only
             ]
 
@@ -71,13 +79,15 @@ class ChildCollector(Collector):
                 elif sub_objs:
                     field.remote_field.on_delete(self, field, sub_objs, self.using)
             for field in model._meta.private_fields:
-                if hasattr(field, 'bulk_related_objects'):
+                if hasattr(field, "bulk_related_objects"):
                     # Its something like generic foreign key.
                     sub_objs = field.bulk_related_objects(new_objs, self.using)
-                    self.collect(sub_objs,
-                                 source=model,
-                                 source_attr=field.remote_field.related_name,
-                                 nullable=True)
+                    self.collect(
+                        sub_objs,
+                        source=model,
+                        source_attr=field.remote_field.related_name,
+                        nullable=True,
+                    )
 
 
 def get_child(obj, child_model):
@@ -106,8 +116,7 @@ def delete_child(obj, child_model, ar=None, using=None):
     ignore_models = set()
     # for m in models_by_base(obj.__class__):
     #     ignore_models.remove(child_model)
-    msg = child._lino_ddh.disable_delete_on_object(
-        obj, ignore_models)
+    msg = child._lino_ddh.disable_delete_on_object(obj, ignore_models)
     if msg:
         raise ValidationError(msg)
     # logger.debug(u"Delete child %s from %s",child_model.__name__,obj)
@@ -137,14 +146,17 @@ def delete_child(obj, child_model, ar=None, using=None):
 
     else:
         collector = Collector(using=using)
-        collector.collect([child], source=obj.__class__,
-                          nullable=True,
-                          keep_parents=True,
-                          collect_related=True)
+        collector.collect(
+            [child],
+            source=obj.__class__,
+            nullable=True,
+            keep_parents=True,
+            collect_related=True,
+        )
     collector.delete()
 
-    #~ setattr(obj,child_model.__name__.lower(),None)
-    #~ delattr(obj,child_model.__name__.lower())
+    # ~ setattr(obj,child_model.__name__.lower(),None)
+    # ~ delattr(obj,child_model.__name__.lower())
 
     # TODO: unchecking e.g. Company.is_courseprovider deletes the
     # child when saving the form, but the response to the PUT returns
@@ -162,19 +174,23 @@ def insert_child(obj, child_model, full_clean=False, **attrs):
     behaviour.
 
     """
-    #~ assert child_model != obj.__class__
-    #~ if child_model == obj.__class__:
-        #~ raise ValidationError(
-            #~ "A %s cannot be parent for itself" %
-            #~ obj.__class__.__name__)
+    # ~ assert child_model != obj.__class__
+    # ~ if child_model == obj.__class__:
+    # ~ raise ValidationError(
+    # ~ "A %s cannot be parent for itself" %
+    # ~ obj.__class__.__name__)
     parent_link_field = child_model._meta.parents.get(obj.__class__, None)
     if parent_link_field is None:
-        raise ValidationError(str("A %s cannot be parent for a %s" % (
-            obj.__class__.__name__, child_model.__name__)))
+        raise ValidationError(
+            str(
+                "A %s cannot be parent for a %s"
+                % (obj.__class__.__name__, child_model.__name__)
+            )
+        )
     attrs[parent_link_field.name] = obj
     # ~ for pm,pf in child_model._meta.parents.items(): # pm : parent model, pf : parent link field
-        #~ attrs[pf.name] = obj
-    #~ attrs["%s_ptr" % obj.__class__.__name__.lower()] = obj
+    # ~ attrs[pf.name] = obj
+    # ~ attrs["%s_ptr" % obj.__class__.__name__.lower()] = obj
     if AFTER17:
         fields_list = obj._meta.concrete_fields
     else:
@@ -183,7 +199,7 @@ def insert_child(obj, child_model, full_clean=False, **attrs):
         if not DJANGO2 or DJANGO2 and not field.primary_key:
             attrs[field.name] = getattr(obj, field.name)
     new_obj = child_model(**attrs)
-    #~ logger.info("20120830 insert_child %s",obj2str(new_obj))
+    # ~ logger.info("20120830 insert_child %s",obj2str(new_obj))
 
     new_obj.save()
     # on_add_child.send(sender=obj, child=new_obj)
@@ -194,11 +210,16 @@ def insert_child(obj, child_model, full_clean=False, **attrs):
         except ValidationError as e:
             msg = obj.error2str(e)
             raise ValidationError(
-                _("Problem while inserting %(child)s "
-                  "child of %(parent)s: %(message)s") %
-                dict(child=child_model.__name__,
-                     parent=obj.__class__.__name__,
-                     message=msg))
+                _(
+                    "Problem while inserting %(child)s "
+                    "child of %(parent)s: %(message)s"
+                )
+                % dict(
+                    child=child_model.__name__,
+                    parent=obj.__class__.__name__,
+                    message=msg,
+                )
+            )
     return new_obj
 
 
@@ -214,13 +235,14 @@ def mtichild(p, model, **kw):
     c.save()
     return model.objects.get(pk=p.pk)
 
-#~ def insert_child_and_save(obj,child_model,**attrs):
-    #~ """
-    #~ Insert (create) and save a `child_model` instance of existing `obj`.
-    #~ """
-    #~ obj = insert_child(obj,child_model,**attrs)
-    #~ obj.save()
-    #~ return obj
+
+# ~ def insert_child_and_save(obj,child_model,**attrs):
+# ~ """
+# ~ Insert (create) and save a `child_model` instance of existing `obj`.
+# ~ """
+# ~ obj = insert_child(obj,child_model,**attrs)
+# ~ obj.save()
+# ~ return obj
 
 
 class EnableChild(VirtualField):
@@ -232,11 +254,13 @@ class EnableChild(VirtualField):
     """
 
     editable = True
-    #~ default = models.NOT_PROVIDED
+
+    # ~ default = models.NOT_PROVIDED
 
     def __init__(self, child_model, **kw):
-        raise Exception("No longer recommended. "
-                        "Use `lino.mixins.polymorphic` instead.")
+        raise Exception(
+            "No longer recommended. " "Use `lino.mixins.polymorphic` instead."
+        )
         kw.update(default=False)
         self.child_model = child_model
         VirtualField.__init__(self, models.BooleanField(**kw), self.has_child)
@@ -246,12 +270,12 @@ class EnableChild(VirtualField):
         must be disabled.
 
         """
-        return lh.layout._datasource.model != self.child_model \
-            and issubclass(self.child_model, lh.layout._datasource.model)
+        return lh.layout._datasource.model != self.child_model and issubclass(
+            self.child_model, lh.layout._datasource.model
+        )
 
     def attach_to_model(self, model, name):
-        self.child_model = resolve_model(
-            self.child_model, model._meta.app_label)
+        self.child_model = resolve_model(self.child_model, model._meta.app_label)
         VirtualField.attach_to_model(self, model, name)
 
     def has_child(self, obj, request=None):
@@ -262,34 +286,34 @@ class EnableChild(VirtualField):
         """
         try:
             getattr(obj, self.child_model.__name__.lower())
-            #~ child = getattr(obj,self.child_model.__name__.lower())
-            #~ if child is None: return False
-            #~ print 20120531, repr(child)
-            #~ self.child_model.objects.get(pk=obj.pk)
+            # ~ child = getattr(obj,self.child_model.__name__.lower())
+            # ~ if child is None: return False
+            # ~ print 20120531, repr(child)
+            # ~ self.child_model.objects.get(pk=obj.pk)
         except self.child_model.DoesNotExist:
             return False
         return True
 
     def set_value_in_object(self, ar, obj, v):
         if self.has_child(obj):
-            #~ logger.debug('set_value_in_object : %s has child %s',
-                #~ obj.__class__.__name__,self.child_model.__name__)
+            # ~ logger.debug('set_value_in_object : %s has child %s',
+            # ~ obj.__class__.__name__,self.child_model.__name__)
             # child exists, delete it if it may not
             if not v:
                 if ar is not None:
                     pre_remove_child.send(
-                        sender=obj, request=ar.request,
-                        child=self.child_model)
+                        sender=obj, request=ar.request, child=self.child_model
+                    )
                 delete_child(obj, self.child_model, ar)
         else:
-            #~ logger.debug('set_value_in_object : %s has no child %s',
-                #~ obj.__class__.__name__,self.child_model.__name__)
+            # ~ logger.debug('set_value_in_object : %s has no child %s',
+            # ~ obj.__class__.__name__,self.child_model.__name__)
             if v:
                 # child doesn't exist. insert if it should
                 if ar is not None:
                     pre_add_child.send(
-                        sender=obj, request=ar.request,
-                        child=self.child_model)
+                        sender=obj, request=ar.request, child=self.child_model
+                    )
                 insert_child(obj, self.child_model, full_clean=True)
 
 
