@@ -65,6 +65,7 @@ class Emitter(object):
     "master".
 
     """
+
     created_tpl = None
     updated_tpl = None
     deleted_tpl = None
@@ -73,12 +74,16 @@ class Emitter(object):
     deleted_subject = _("Deleted: {obj}")
     master_field = None
 
-    def __init__(self, model=None, watched_fields=None,
-                 created_tpl=None, updated_tpl=None, deleted_tpl=None,
-                 master_field=None):
-        """
-
-        """
+    def __init__(
+        self,
+        model=None,
+        watched_fields=None,
+        created_tpl=None,
+        updated_tpl=None,
+        deleted_tpl=None,
+        master_field=None,
+    ):
+        """ """
         if model:
             self.model = model
         if watched_fields:
@@ -88,7 +93,7 @@ class Emitter(object):
         if master_field:
             self.master_field = master_field
 
-        for k in ('created_tpl', 'updated_tpl', 'deleted_tpl'):
+        for k in ("created_tpl", "updated_tpl", "deleted_tpl"):
             v = locals().get(k)
             if v:
                 setattr(self, k, v)
