@@ -21,7 +21,6 @@ from __future__ import print_function
 # print(r)
 # show_sql_queries()
 
-
 import re
 import sys
 import textwrap
@@ -34,10 +33,11 @@ def p(kw, sql_width=60):
         "table: {table}\n"
         "Longest_time: {time}\n"
         "Queries: {count}\n"
-        "total_time: {total_time}".format(kw))
+        "total_time: {total_time}".format(kw)
+    )
     sql = "{sql1} {table} {sql2}".format(kw)
-    sql = sql.replace('"', '')
-    kw['sql'] = ("\n    ").join(textwrap.wrap(sql, sql_width))
+    sql = sql.replace('"', "")
+    kw["sql"] = ("\n    ").join(textwrap.wrap(sql, sql_width))
     print("sql: {sql}".format(kw))
 
 
@@ -61,16 +61,13 @@ def p(kw, sql_width=60):
 regex = r"^.*?\((?P<time>\S+?)\)\s+(?P<sql>.*);$"
 
 try:
-
     import sqlparse
     from sqlparse.sql import IdentifierList, Identifier
     from sqlparse.tokens import Keyword, DML
 
 except ImportError:
-
     sqlparse = None
 
-
 # Copied from https://github.com/andialbrecht/sqlparse/blob/master/examples/extract_table_names.py
 
 
@@ -78,7 +75,7 @@ def is_subselect(parsed):
     if not parsed.is_group:
         return False
     for item in parsed.tokens:
-        if item.ttype is DML and item.value.upper() == 'SELECT':
+        if item.ttype is DML and item.value.upper() == "SELECT":
             return True
     return False
 
@@ -94,7 +91,7 @@ def extract_from_part(parsed):
                 return
             else:
                 yield item
-        elif item.ttype is Keyword and item.value.upper() == 'FROM':
+        elif item.ttype is Keyword and item.value.upper() == "FROM":
             from_seen = True
 
 
@@ -110,6 +107,7 @@ def extract_table_identifiers(token_stream):
         elif item.ttype is Keyword:
             yield item.value
 
+
 def extract_tables(stmt):
     stream = extract_from_part(stmt)
     return list(extract_table_identifiers(stream))
@@ -121,7 +119,7 @@ class Entry(object):
         self.time = float(time)
         self.total_time = self.time
         self.count = 1
-        self.table = ''
+        self.table = ""
 
         stmt = sqlparse.parse(sql)[0]
         self.stmt_type = stmt.get_type()
@@ -135,7 +133,7 @@ class Entry(object):
         #         break
 
     def group_key(self):
-        return self.table + ' ' + self.stmt_type
+        return self.table + " " + self.stmt_type
 
     def collect(self, other):
         self.total_time += other.time
@@ -161,13 +159,13 @@ def sql_summary(lines, show_times=False, show_details=False, **options):
     # matches = []
     d = {}
     for l in lines:
-        l = l.replace('"', '')
-        l = l.replace('`', '')
+        l = l.replace('"', "")
+        l = l.replace("`", "")
 
         m = re.match(regex, l)
         if m:
             g = m.groupdict()
-            entry = Entry(g['sql'], g['time'])
+            entry = Entry(g["sql"], g["time"])
             k = entry.group_key()
             if k in d:
                 d[k].collect(entry)
@@ -207,10 +205,10 @@ def sql_summary(lines, show_times=False, show_details=False, **options):
             print("-------------------")
         else:
             if show_times:
-                headers = 'total_time count table stmt_type time'.split()
+                headers = "total_time count table stmt_type time".split()
                 values = sorted(d.values(), key=lambda x: -x.total_time)
             else:
-                headers = 'table stmt_type count'.split()
+                headers = "table stmt_type count".split()
                 values = sorted(d.values(), key=lambda x: x.table)
             rows = []
             for e in values:
