@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2013-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines some TestCase classes that are meant for writing general Python test
 cases (not Django test cases, which are defined in
 :mod:`lino.utils.djangotest`).
@@ -21,9 +20,10 @@ from lino.utils import AttrDict
 from lino.core import constants
 
 import collections
+
 HttpQuery = collections.namedtuple(
-    'HttpQuery',
-    ['username', 'url_base', 'json_fields', 'expected_rows', 'kwargs'])
+    "HttpQuery", ["username", "url_base", "json_fields", "expected_rows", "kwargs"]
+)
 
 
 class CommonTestCase(unittest.TestCase):
@@ -45,16 +45,19 @@ class CommonTestCase(unittest.TestCase):
         """
         return create_row(model, **values)
 
-    def check_json_result(self, response, expected_keys=None, msg=''):
+    def check_json_result(self, response, expected_keys=None, msg=""):
         """Checks the result of response which is expected to return a
         JSON-encoded dictionary with the expected_keys.
 
         """
         # print("20150129 response is %r" % response.content)
         self.assertEqual(
-            response.status_code, 200,
+            response.status_code,
+            200,
             "Response status ({0}) was {1} instead of 200".format(
-                msg, response.status_code))
+                msg, response.status_code
+            ),
+        )
         content = response.content.decode()
         try:
             result = json.loads(content)
@@ -74,6 +77,10 @@ class CommonTestCase(unittest.TestCase):
         """
         if a == b:
             return
+        if False:  # this can help to find the differing line
+            ta = a.strip().splitlines()
+            tb = b.strip().splitlines()
+            self.assertEqual(ta, tb)
         ta = a.strip().split()
         tb = b.strip().split()
         if ta == tb:
@@ -81,8 +88,8 @@ class CommonTestCase(unittest.TestCase):
         if report_plain:
             msg = "----- EXPECTED : -----\n%s\n----- GOT : -----\n%s" % (a, b)
         else:
-            msg =  "\nEXPECTED : %s" % (' '.join(ta))
-            msg += "\n     GOT : %s" % (' '.join(tb))
+            msg = "\nEXPECTED : %s" % (" ".join(ta))
+            msg += "\n     GOT : %s" % (" ".join(tb))
         if False:
             logger.warning(msg)
             self.fail("EXPECTED and GOT are not equivalent")
@@ -95,7 +102,7 @@ class CommonTestCase(unittest.TestCase):
         default.
 
         """
-        kw.update(content_type='application/x-www-form-urlencoded')
+        kw.update(content_type="application/x-www-form-urlencoded")
         response = self.client.put(url, data, **kw)
         self.assertEqual(response.status_code, 200)
         return response
@@ -116,6 +123,7 @@ class DocTest(unittest.TestCase):
     temporary test database.
 
     """
+
     doctest_files = ["index.rst"]
     """The files to be tested.
 
@@ -123,6 +131,7 @@ class DocTest(unittest.TestCase):
 
     def test_files(self):
         from django.conf import settings
+
         g = dict()
         g.update(settings=settings)
         # g.update(unicode_literals=unicode_literals)
@@ -155,8 +164,10 @@ class DemoTestCase(PythonTestCase, CommonTestCase):
     This is used by the :manage:`demotest` command.
 
     """
+
     def __call__(self, *args, **kw):
         from django.test import Client
+
         self.client = Client()
         return super(DemoTestCase, self).__call__(*args, **kw)
 
@@ -172,52 +183,53 @@ class DemoTestCase(PythonTestCase, CommonTestCase):
 
         data = {
             constants.URL_PARAM_FIELD_VALUES: [username, pwd],
-            constants.URL_PARAM_ACTION_NAME: 'sign_in'}
+            constants.URL_PARAM_ACTION_NAME: "sign_in",
+        }
         from django.conf import settings
-        # url = settings.SITE.site_prefix + "api/about/About/-99998"
-        url = settings.SITE.kernel.editing_front_end.build_plain_url("api/about/About/-99998")
+
+        url = settings.SITE.kernel.editing_front_end.build_plain_url(
+            "api/about/About/-99998"
+        )
         res = self.client.post(url, data)
         self.assertEqual(res.status_code, 200)
         content = res.content.decode()
         d = json.loads(content)
-        # return str(d.keys())
         return AttrDict(d)
 
-    def demo_get(self, username, url_base, json_fields, expected_rows,
-                 **kwargs):
+    def demo_get(self, username, url_base, json_fields, expected_rows, **kwargs):
         from django.conf import settings
-        case = HttpQuery(username, url_base, json_fields,
-                         expected_rows, kwargs)
+
+        case = HttpQuery(username, url_base, json_fields, expected_rows, kwargs)
         url = settings.SITE.buildurl(case.url_base, **case.kwargs)
 
-        msg = 'Using remote authentication, but no user credentials found.'
+        msg = "Using remote authentication, but no user credentials found."
         try:
             response = self.client.get(url)
             self.fail("Expected '%s'" % msg)
-            #~ raise Exception("Expected '%s'" % msg)
+            # ~ raise Exception("Expected '%s'" % msg)
         except Exception:
             pass
 
-        response = self.client.get(url, REMOTE_USER='foo')
+        response = self.client.get(url, REMOTE_USER="foo")
         self.assertEqual(
-            response.status_code, 403,
-            "Status code %s other than 403 for anonymous on GET %s" % (
-                response.status_code, url))
+            response.status_code,
+            403,
+            "Status code %s other than 403 for anonymous on GET %s"
+            % (response.status_code, url),
+        )
 
         response = self.client.get(url, REMOTE_USER=case.username)
-        user = settings.SITE.user_model.objects.get(
-            username=case.username)
+        user = settings.SITE.user_model.objects.get(username=case.username)
         result = self.check_json_result(
-            response, case.json_fields,
-            "GET %s for user %s" % (url, user))
+            response, case.json_fields, "GET %s for user %s" % (url, user)
+        )
 
         num = case.expected_rows
         if num is not None:
             if not isinstance(num, tuple):
                 num = [num]
-            if result['count'] not in num:
-                msg = "%s got %s rows instead of %s" % (
-                    url, result['count'], num)
-                #~ raise Exception("[%d] %s" % (i, msg))
+            if result["count"] not in num:
+                msg = "%s got %s rows instead of %s" % (url, result["count"], num)
+                # ~ raise Exception("[%d] %s" % (i, msg))
                 self.fail(msg)
-                #~ failures += 1
+                # ~ failures += 1
