@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 This is taken from Helio Perroni Filho's answer at
 https://stackoverflow.com/questions/328356/extracting-text-from-html-file-using-python/3987802#3987802
@@ -23,7 +22,6 @@ from traceback import print_exc
 
 
 class _DeHTMLParser(HTMLParser):
-
     def __init__(self):
         HTMLParser.__init__(self)
         self.__text = []
@@ -31,21 +29,21 @@ class _DeHTMLParser(HTMLParser):
     def handle_data(self, data):
         text = data.strip()
         if len(text) > 0:
-            text = sub('[ \t\r\n]+', ' ', text)
-            self.__text.append(text + ' ')
+            text = sub("[ \t\r\n]+", " ", text)
+            self.__text.append(text + " ")
 
     def handle_starttag(self, tag, attrs):
-        if tag == 'p':
-            self.__text.append('\n\n')
-        elif tag == 'br':
-            self.__text.append('\n')
+        if tag == "p":
+            self.__text.append("\n\n")
+        elif tag == "br":
+            self.__text.append("\n")
 
     def handle_startendtag(self, tag, attrs):
-        if tag == 'br':
-            self.__text.append('\n\n')
+        if tag == "br":
+            self.__text.append("\n\n")
 
     def text(self):
-        return ''.join(self.__text).strip()
+        return "".join(self.__text).strip()
 
 
 def dehtml(text):
@@ -60,13 +58,13 @@ def dehtml(text):
 
 
 def extract_summary(text):
-    if text.startswith('<'):
+    if text.startswith("<"):
         text = dehtml(text)
-    a = text.split('\n', 1)
+    a = text.split("\n", 1)
     ellipsis = False
     if len(a) > 1:
         ellipsis = True
-    ln = text.split('\n', 1)[0]
+    ln = text.split("\n", 1)[0]
     if len(ln) > 30:
         ln = ln[:30]
         ellipsis = True
@@ -76,7 +74,7 @@ def extract_summary(text):
 
 
 def main():
-    text = r'''
+    text = r"""
         <html>
             <body>
                 <b>Project:</b> DeHTML<br>
@@ -85,9 +83,9 @@ def main():
                 plain text.
             </body>
         </html>
-    '''
+    """
     print(dehtml(text))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
